@@ -1,3 +1,17 @@
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
+   All Rights Reserved.
+
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
+*/
 /* Parameters derived from machine and compiler architecture. */
 /* This file was generated mechanically by genarch.c, for a 64bit */
 /* Microsoft Windows machine, compiling with MSVC. */
@@ -17,7 +31,11 @@
 #define ARCH_LOG2_SIZEOF_SHORT 1
 #define ARCH_LOG2_SIZEOF_INT 2
 #define ARCH_LOG2_SIZEOF_LONG 2
+
+#ifndef ARCH_SIZEOF_GX_COLOR_INDEX
 #define ARCH_SIZEOF_GX_COLOR_INDEX 8
+#endif
+
 #define ARCH_SIZEOF_PTR 8
 #define ARCH_SIZEOF_FLOAT 4
 #define ARCH_SIZEOF_DOUBLE 8
