@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -112,6 +112,15 @@ typedef struct FT_IncrementalRec_
     gs_fapi_metrics_type metrics_type;  /* determines whether metrics are replaced, added, etc. */
 } FT_IncrementalRec;
 
+
+static void
+delete_inc_int(gs_fapi_server * a_server,
+               FT_Incremental_InterfaceRec * a_inc_int);
+
+static void
+delete_inc_int_info(gs_fapi_server * a_server,
+                    FT_IncrementalRec * a_inc_int_info);
+
 FT_CALLBACK_DEF(void *)
 FF_alloc(FT_Memory memory, long size)
 {
@@ -293,6 +302,8 @@ delete_face(gs_fapi_server * a_server, ff_face * a_face)
             }
             a_info->glyph_data = NULL;
             a_info->glyph_data_length = 0;
+            delete_inc_int(a_server, a_face->ft_inc_int);
+            a_face->ft_inc_int = NULL;
         }
         FT_Done_Face(a_face->ft_face);
 
@@ -355,7 +366,8 @@ get_fapi_glyph_data(FT_Incremental a_info, FT_UInt a_index, FT_Data * a_data)
         unsigned char *buffer = NULL;
 
         length = ff->get_glyph(ff, a_index, NULL, 0);
-        if (length == 65535)
+        if (length == gs_fapi_glyph_invalid_format
+            || length == gs_fapi_glyph_invalid_index)
             return FT_Err_Invalid_Glyph_Index;
 
         buffer = gs_malloc(mem, length, 1, "get_fapi_glyph_data");
@@ -363,7 +375,7 @@ get_fapi_glyph_data(FT_Incremental a_info, FT_UInt a_index, FT_Data * a_data)
             return FT_Err_Out_Of_Memory;
 
         length = ff->get_glyph(ff, a_index, buffer, length);
-        if (length == 65535) {
+        if (length == gs_fapi_glyph_invalid_format) {
             gs_free((gs_memory_t *) mem, buffer, 0, 0,
                     "get_fapi_glyph_data");
             return FT_Err_Invalid_Glyph_Index;
@@ -381,11 +393,16 @@ get_fapi_glyph_data(FT_Incremental a_info, FT_UInt a_index, FT_Data * a_data)
         length =
             ff->get_glyph(ff, a_index, a_info->glyph_data,
                           (ushort) a_info->glyph_data_length);
-        if (length == -1) {
+        if (length == gs_fapi_glyph_invalid_format) {
             ff->char_data = saved_char_data;
             return FT_Err_Unknown_File_Format;
         }
 
+        if (length == gs_fapi_glyph_invalid_index) {
+            ff->char_data = saved_char_data;
+            return FT_Err_Invalid_Glyph_Index;
+        }
+
         /* If the buffer was too small enlarge it and try again. */
         if (length > a_info->glyph_data_length) {
             if (a_info->glyph_data) {
@@ -403,8 +420,10 @@ get_fapi_glyph_data(FT_Incremental a_info, FT_UInt a_index, FT_Data * a_data)
             a_info->glyph_data_length = length;
             ff->char_data = saved_char_data;
             length = ff->get_glyph(ff, a_index, a_info->glyph_data, length);
-            if (length == -1)
+            if (length == gs_fapi_glyph_invalid_format)
                 return FT_Err_Unknown_File_Format;
+            if (length == gs_fapi_glyph_invalid_index)
+                return FT_Err_Invalid_Glyph_Index;
         }
 
         /* Set the returned pointer and length. */
@@ -491,7 +510,7 @@ new_inc_int(gs_fapi_server * a_server, gs_fapi_font * a_fapi_font)
 }
 
 static void
-delete_inc_int(gs_fapi_server * a_server,
+delete_inc_int(gs_fapi_server * a_server,
                FT_Incremental_InterfaceRec * a_inc_int)
 {
     ff_server *s = (ff_server *) a_server;
@@ -882,34 +901,30 @@ load_glyph(gs_fapi_server * a_server, gs_fapi_font * a_fapi_font,
  */
 static gs_fapi_retcode
 gs_fapi_ft_ensure_open(gs_fapi_server * a_server, const char * server_param,
-            int server_param_size)
+                       int server_param_size)
 {
     ff_server *s = (ff_server *) a_server;
+    FT_UInt tt_ins_version = TT_INTERPRETER_VERSION_35;
+    FT_Error ft_error;
 
-    if (!s->freetype_library) {
-        FT_Error ft_error;
+    if (s->freetype_library)
+        return 0;
 
-        /* As we want FT to use our memory management, we cannot use the convenience of
-         * FT_Init_FreeType(), we have to do each stage "manually"
-         */
-        s->ftmemory->user = s->mem;
-        s->ftmemory->alloc = FF_alloc;
-        s->ftmemory->free = FF_free;
-        s->ftmemory->realloc = FF_realloc;
+    /* As we want FT to use our memory management, we cannot use the convenience of
+     * FT_Init_FreeType(), we have to do each stage "manually"
+     */
+    s->ftmemory->user = s->mem;
+    s->ftmemory->alloc = FF_alloc;
+    s->ftmemory->free = FF_free;
+    s->ftmemory->realloc = FF_realloc;
 
-        ft_error = FT_New_Library(s->ftmemory, &s->freetype_library);
-        if (ft_error) {
-            gs_free(s->mem, s->ftmemory, 0, 0, "gs_fapi_ft_ensure_open");
-        }
-        else {
-            FT_UInt tt_ins_version = TT_INTERPRETER_VERSION_35;
-            FT_Add_Default_Modules(s->freetype_library);
-            FT_Property_Set( s->freetype_library, "truetype", "interpreter-version", &tt_ins_version);
-        }
+    ft_error = FT_New_Library(s->ftmemory, &s->freetype_library);
+    if (ft_error)
+        return ft_to_gs_error(ft_error);
+
+    FT_Add_Default_Modules(s->freetype_library);
+    FT_Property_Set( s->freetype_library, "truetype", "interpreter-version", &tt_ins_version);
 
-        if (ft_error)
-            return ft_to_gs_error(ft_error);
-    }
     return 0;
 }
 
@@ -967,10 +982,10 @@ transform_decompose(FT_Matrix * a_transform, FT_UInt * xresp, FT_UInt * yresp,
         factx = 64.0/scalex;
         facty = 64.0/scaley;
 
-        ftscale_mat.xx = a_transform->xx * factx;
-        ftscale_mat.xy = a_transform->xy * facty;
-        ftscale_mat.yx = a_transform->yx * factx;
-        ftscale_mat.yy = a_transform->yy * facty;
+        ftscale_mat.xx = (FT_Fixed)(a_transform->xx * factx);
+        ftscale_mat.xy = (FT_Fixed)(a_transform->xy * facty);
+        ftscale_mat.yx = (FT_Fixed)(a_transform->yx * factx);
+        ftscale_mat.yy = (FT_Fixed)(a_transform->yy * facty);
         memcpy(a_transform, &ftscale_mat, sizeof(ftscale_mat));
         scalex = hypot((double)a_transform->xx, (double)a_transform->xy);
         scaley = hypot((double)a_transform->yx, (double)a_transform->yy);
@@ -1000,13 +1015,13 @@ transform_decompose(FT_Matrix * a_transform, FT_UInt * xresp, FT_UInt * yresp,
         FT_Matrix_Multiply(&ftscale_mat, a_transform);
 
         xres = yres = (use_x ? (*xresp) : (*yresp));
-        xres = xres / factx;
-        yres = yres / facty;
+        xres = (FT_UInt)(xres / factx);
+        yres = (FT_UInt)(yres / facty);
     }
     else {
         /* Life is considerably easier when square resolutions are in use! */
-        xres = *xresp / factx;
-        yres = *yresp / facty;
+        xres = (FT_UInt)(*xresp / factx);
+        yres = (FT_UInt)(*yresp / facty);
     }
 
     scalex *= 1.0 / 65536.0;
@@ -1530,13 +1545,25 @@ typedef struct FF_path_info_s
     gs_fapi_path *path;
     int64_t x;
     int64_t y;
+    FT_Vector currentp;
 } FF_path_info;
 
+static inline int
+FF_points_equal(const FT_Vector *p1, const FT_Vector *p2)
+{
+    if (p1->x == p2->x && p1->y == p2->y)
+        return 1;
+    else
+        return 0;
+}
+
 static int
 move_to(const FT_Vector * aTo, void *aObject)
 {
     FF_path_info *p = (FF_path_info *) aObject;
 
+    p->currentp = *aTo;
+
     /* FAPI expects that co-ordinates will be as implied by frac_shift
      * in our case 16.16 fixed precision. True for 'low level' FT
      * routines (apparently), it isn't true for these routines where
@@ -1555,11 +1582,16 @@ line_to(const FT_Vector * aTo, void *aObject)
 {
     FF_path_info *p = (FF_path_info *) aObject;
 
-    /* See move_to() above */
-    p->x = ((int64_t) aTo->x) << 26;
-    p->y = ((int64_t) aTo->y) << 26;
+    if (!FF_points_equal(&p->currentp, aTo)) {
+        p->currentp = *aTo;
 
-    return p->path->lineto(p->path, p->x, p->y) ? -1 : 0;
+        /* See move_to() above */
+        p->x = ((int64_t) aTo->x) << 26;
+        p->y = ((int64_t) aTo->y) << 26;
+
+        return p->path->lineto(p->path, p->x, p->y) ? -1 : 0;
+    }
+    return 0;
 }
 
 static int
@@ -1570,40 +1602,47 @@ conic_to(const FT_Vector * aControl, const FT_Vector * aTo, void *aObject)
     int64_t Control1x, Control1y, Control2x, Control2y;
     double sx, sy;
 
-    /* More complicated than above, we need to do arithmetic on the
-     * co-ordinates, so we want them as floats and we will convert the
-     * result into 16.16 fixed precision for FAPI
-     *
-     * NB this code is funcitonally the same as the original, but I don't believe
-     * the comment (below) to be what the code is actually doing....
-     *
-     * NB2: the comment below was wrong, even though the code was correct(!!)
-     * The comment has now been amended.
-     *
-     * Convert a quadratic spline to a cubic. Do this by changing the three points
-     * A, B and C to A, 2/3(B,A), 2/3(B,C), C - that is, the two cubic control points are
-     * a third of the way from the single quadratic control point to the end points. This
-     * gives the same curve as the original quadratic.
-     */
+    if (!FF_points_equal(&p->currentp, aControl) ||
+        !FF_points_equal(&p->currentp, aTo) ||
+        !FF_points_equal(aControl, aTo)) {
+        p->currentp = *aTo;
 
-    sx = (double) (p->x >> 32);
-    sy = (double) (p->y >> 32);
+        /* More complicated than above, we need to do arithmetic on the
+         * co-ordinates, so we want them as floats and we will convert the
+         * result into 16.16 fixed precision for FAPI
+         *
+         * NB this code is funcitonally the same as the original, but I don't believe
+         * the comment (below) to be what the code is actually doing....
+         *
+         * NB2: the comment below was wrong, even though the code was correct(!!)
+         * The comment has now been amended.
+         *
+         * Convert a quadratic spline to a cubic. Do this by changing the three points
+         * A, B and C to A, 2/3(B,A), 2/3(B,C), C - that is, the two cubic control points are
+         * a third of the way from the single quadratic control point to the end points. This
+         * gives the same curve as the original quadratic.
+         */
 
-    x = aTo->x / 64.0;
-    p->x = ((int64_t) float2fixed(x)) << 24;
-    y = aTo->y / 64.0;
-    p->y = ((int64_t) float2fixed(y)) << 24;
-    Controlx = aControl->x / 64.0;
-    Controly = aControl->y / 64.0;
+        sx = (double) (p->x >> 32);
+        sy = (double) (p->y >> 32);
 
-    Control1x = ((int64_t) float2fixed((sx + Controlx * 2) / 3)) << 24;
-    Control1y = ((int64_t) float2fixed((sy + Controly * 2) / 3)) << 24;
-    Control2x = ((int64_t) float2fixed((x + Controlx * 2) / 3)) << 24;
-    Control2y = ((int64_t) float2fixed((y + Controly * 2) / 3)) << 24;
+        x = aTo->x / 64.0;
+        p->x = ((int64_t) float2fixed(x)) << 24;
+        y = aTo->y / 64.0;
+        p->y = ((int64_t) float2fixed(y)) << 24;
+        Controlx = aControl->x / 64.0;
+        Controly = aControl->y / 64.0;
 
-    return p->path->curveto(p->path, Control1x,
-                            Control1y,
-                            Control2x, Control2y, p->x, p->y) ? -1 : 0;
+        Control1x = ((int64_t) float2fixed((sx + Controlx * 2) / 3)) << 24;
+        Control1y = ((int64_t) float2fixed((sy + Controly * 2) / 3)) << 24;
+        Control2x = ((int64_t) float2fixed((x + Controlx * 2) / 3)) << 24;
+        Control2y = ((int64_t) float2fixed((y + Controly * 2) / 3)) << 24;
+
+        return p->path->curveto(p->path, Control1x,
+                                Control1y,
+                                Control2x, Control2y, p->x, p->y) ? -1 : 0;
+    }
+    return 0;
 }
 
 static int
@@ -1613,16 +1652,26 @@ cubic_to(const FT_Vector * aControl1, const FT_Vector * aControl2,
     FF_path_info *p = (FF_path_info *) aObject;
     int64_t Control1x, Control1y, Control2x, Control2y;
 
-    /* See move_to() above */
-    p->x = ((int64_t) aTo->x) << 26;
-    p->y = ((int64_t) aTo->y) << 26;
+    if (!FF_points_equal(&p->currentp, aControl1) ||
+        !FF_points_equal(&p->currentp, aControl2) ||
+        !FF_points_equal(&p->currentp, aTo) ||
+        !FF_points_equal(aControl1, aControl2) ||
+        !FF_points_equal(aControl1, aTo) ||
+        !FF_points_equal(aControl2, aTo)) {
+        p->currentp = *aTo;
 
-    Control1x = ((int64_t) aControl1->x) << 26;
-    Control1y = ((int64_t) aControl1->y) << 26;
-    Control2x = ((int64_t) aControl2->x) << 26;
-    Control2y = ((int64_t) aControl2->y) << 26;
-    return p->path->curveto(p->path, Control1x, Control1y, Control2x,
-                            Control2y, p->x, p->y) ? -1 : 0;
+        /* See move_to() above */
+        p->x = ((int64_t) aTo->x) << 26;
+        p->y = ((int64_t) aTo->y) << 26;
+
+        Control1x = ((int64_t) aControl1->x) << 26;
+        Control1y = ((int64_t) aControl1->y) << 26;
+        Control2x = ((int64_t) aControl2->x) << 26;
+        Control2y = ((int64_t) aControl2->y) << 26;
+        return p->path->curveto(p->path, Control1x, Control1y, Control2x,
+                                Control2y, p->x, p->y) ? -1 : 0;
+    }
+    return 0;
 }
 
 static const FT_Outline_Funcs TheFtOutlineFuncs = {
