@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2015 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Device for first/last page handling device */
@@ -100,7 +100,6 @@ static dev_proc_push_transparency_state(flp_push_transparency_state);
 static dev_proc_pop_transparency_state(flp_pop_transparency_state);
 static dev_proc_put_image(flp_put_image);
 static dev_proc_copy_planes(flp_copy_planes);
-static dev_proc_set_graphics_type_tag(flp_set_graphics_type_tag);
 static dev_proc_strip_copy_rop2(flp_strip_copy_rop2);
 static dev_proc_strip_tile_rect_devn(flp_strip_tile_rect_devn);
 static dev_proc_copy_alpha_hl_color(flp_copy_alpha_hl_color);
@@ -203,7 +202,7 @@ gx_device_flp gs_flp_device =
      default_subclass_dev_spec_op,                      /* dev_spec_op */
      flp_copy_planes,                      /* copy_planes */
      default_subclass_get_profile,                      /* get_profile */
-     flp_set_graphics_type_tag,                      /* set_graphics_type_tag */
+     default_subclass_set_graphics_type_tag,        /* set_graphics_type_tag */
      flp_strip_copy_rop2,
      flp_strip_tile_rect_devn,
      flp_copy_alpha_hl_color,
@@ -823,6 +822,14 @@ int flp_create_compositor(gx_device *dev, gx_device **pcdev, const gs_composite_
 {
     int code = SkipPage(dev);
 
+    /* The returned pointer 'pcdev' is a signal to the caller, if its not the same as the current
+     * device in the graphics state, then it gets set as the current device (basically if its not the
+     * same its a signal that we want to push a new compositor device to the head). So here we start
+     * by making sure that its set to the one in the graphics state. If the child device wants a
+     * compositor it will overwrite this value.
+     */
+    *pcdev = pgs->device;
+
     if (code < 0)
         return code;
     if (!code)
@@ -1128,18 +1135,6 @@ int flp_copy_planes(gx_device *dev, const byte *data, int data_x, int raster, gx
     return 0;
 }
 
-void flp_set_graphics_type_tag(gx_device *dev, gs_graphics_type_tag_t tag)
-{
-    int code = SkipPage(dev);
-
-    if (code < 0)
-        return;
-    if (!code)
-        default_subclass_set_graphics_type_tag(dev, tag);
-
-    return;
-}
-
 int flp_strip_copy_rop2(gx_device *dev, const byte *sdata, int sourcex, uint sraster, gx_bitmap_id id,
     const gx_color_index *scolors, const gx_strip_bitmap *textures, const gx_color_index *tcolors,
     int x, int y, int width, int height, int phase_x, int phase_y, gs_logical_operation_t lop, uint planar_height)
