@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Null and forwarding device implementation */
@@ -44,7 +44,9 @@ gx_device_set_target(gx_device_forward *fdev, gx_device *target)
     if (target && !fdev->finalize)
         fdev->finalize = gx_device_forward_finalize;
     rc_assign(fdev->target, target, "gx_device_set_target");
-    fdev->graphics_type_tag = target != NULL ? target->graphics_type_tag : 0;	/* initialize to same as target */
+    /* try to initialize to same as target, otherwise UNKNOWN */
+    fdev->graphics_type_tag = target != NULL ? target->graphics_type_tag : GS_UNKNOWN_TAG;
+    fdev->interpolate_control = target != NULL ? target->interpolate_control : 1;	/* the default */
 }
 
 /* Fill in NULL procedures in a forwarding device procedure record. */
@@ -715,17 +717,14 @@ fwd_map_gray_cs(gx_device * dev, frac gray, frac out[])
 {
     gx_device_forward * const fdev = (gx_device_forward *)dev;
     gx_device * tdev = fdev->target;
-    const gx_cm_color_map_procs * pprocs;
+    subclass_color_mappings scm;
 
-    pprocs = get_color_mapping_procs_subclass(tdev);
-    /* Verify that all of the pointers and procs are set */
-    /* If not then use a default routine.  This case should be an error */
-    if (tdev == 0 || dev_proc(tdev, get_color_mapping_procs) == 0 ||
-          pprocs == 0 ||
-          pprocs->map_gray == 0)
-        gray_cs_to_gray_cm(tdev, gray, out);   /* if all else fails */
+    if (tdev) {
+        scm = get_color_mapping_procs_subclass(tdev);
+        map_gray_subclass(scm, gray, out);
+    }
     else
-        map_gray_subclass(pprocs, tdev, gray, out);
+        gray_cs_to_gray_cm(tdev, gray, out);   /* if all else fails */
 }
 
 /*
@@ -737,17 +736,14 @@ fwd_map_rgb_cs(gx_device * dev, const gs_gstate *pgs,
 {
     gx_device_forward * const fdev = (gx_device_forward *)dev;
     gx_device * tdev = fdev->target;
-    const gx_cm_color_map_procs * pprocs;
+    subclass_color_mappings scm;
 
-    pprocs = get_color_mapping_procs_subclass(tdev);
-    /* Verify that all of the pointers and procs are set */
-    /* If not then use a default routine.  This case should be an error */
-    if (tdev == 0 || dev_proc(tdev, get_color_mapping_procs) == 0 ||
-          pprocs == 0 ||
-          pprocs->map_rgb == 0)
-        rgb_cs_to_rgb_cm(tdev, pgs, r, g, b, out);   /* if all else fails */
+    if (tdev) {
+        scm = get_color_mapping_procs_subclass(tdev);
+        map_rgb_subclass(scm, pgs, r, g, b, out);
+    }
     else
-        map_rgb_subclass(pprocs, tdev, pgs, r, g, b, out);
+        rgb_cs_to_rgb_cm(tdev, pgs, r, g, b, out);   /* if all else fails */
 }
 
 /*
@@ -758,17 +754,14 @@ fwd_map_cmyk_cs(gx_device * dev, frac c, frac m, frac y, frac k, frac out[])
 {
     gx_device_forward * const fdev = (gx_device_forward *)dev;
     gx_device * tdev = fdev->target;
-    const gx_cm_color_map_procs * pprocs;
+    subclass_color_mappings scm;
 
-    pprocs = get_color_mapping_procs_subclass(tdev);
-    /* Verify that all of the pointers and procs are set */
-    /* If not then use a default routine.  This case should be an error */
-    if (tdev == 0 || dev_proc(tdev, get_color_mapping_procs) == 0 ||
-          pprocs == 0 ||
-          pprocs->map_cmyk == 0)
-        cmyk_cs_to_cmyk_cm(tdev, c, m, y, k, out);   /* if all else fails */
+    if (tdev) {
+        scm = get_color_mapping_procs_subclass(tdev);
+        map_cmyk_subclass(scm, c, m, y, k, out);
+    }
     else
-        map_cmyk_subclass(pprocs, tdev, c, m, y, k, out);
+        cmyk_cs_to_cmyk_cm(tdev, c, m, y, k, out);   /* if all else fails */
 }
 
 static const gx_cm_color_map_procs FwdDevice_cm_map_procs = {
@@ -787,8 +780,7 @@ gx_forward_get_color_mapping_procs(const gx_device * dev)
     const gx_device_forward * fdev = (const gx_device_forward *)dev;
     gx_device * tdev = fdev->target;
 
-    return (tdev == 0 || dev_proc(tdev, get_color_mapping_procs) == 0
-        ? gx_default_DevGray_get_color_mapping_procs(dev)
+    return (tdev == 0 ? gx_default_DevGray_get_color_mapping_procs(dev)
         : &FwdDevice_cm_map_procs);
 }
 
@@ -839,11 +831,11 @@ gx_forward_dev_spec_op(gx_device * dev, int dev_spec_op, void *data, int size)
        so this function is unapplicable to clist. */
     if (tdev == 0) {
         if (dev_spec_op == gxdso_pattern_shfill_doesnt_need_path) {
-            return (dev->procs.fill_path == gx_default_fill_path);
+            return (dev_proc(dev, fill_path) == gx_default_fill_path);
         }
         return_error(gs_error_undefined);
     } else if (dev_spec_op == gxdso_pattern_handles_clip_path) {
-        if (dev->procs.fill_path == gx_default_fill_path)
+        if (dev_proc(dev, fill_path) == gx_default_fill_path)
             return 0;
     } else if (dev_spec_op == gxdso_device_child) {
         gxdso_device_child_request *d = (gxdso_device_child_request *)data;
@@ -1043,6 +1035,7 @@ static dev_proc_decode_color(null_decode_color);
 static dev_proc_strip_copy_rop(null_strip_copy_rop);
 static dev_proc_strip_copy_rop2(null_strip_copy_rop2);
 static dev_proc_strip_tile_rect_devn(null_strip_tile_rect_devn);
+static dev_proc_fill_rectangle_hl_color(null_fill_rectangle_hl_color);
 
 #define null_procs(get_initial_matrix, get_page_device) {\
         gx_default_open_device,\
@@ -1099,13 +1092,13 @@ static dev_proc_strip_tile_rect_devn(null_strip_tile_rect_devn);
         gx_default_gray_fast_encode,            /* encode_color */\
         null_decode_color,              /* decode_color */\
         NULL, /* pattern_manage */\
-        gx_default_fill_rectangle_hl_color,\
+        null_fill_rectangle_hl_color,\
         gx_default_include_color_space,\
         NULL, /* fill_line_sl */\
         NULL, /* fill_line_tr */\
         NULL, /* fill_line_tri */\
         NULL, /* up_spot_eq_col */\
-        NULL, /* ret_devn_params */\
+        gx_default_ret_devn_params, /* ret_devn_params */\
         NULL, /* fillpage */\
         NULL, /* push_transparency_state */\
         NULL, /* pop_transparency_state */\
@@ -1289,6 +1282,22 @@ null_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tiles,
     return 0;
 }
 
+/* We use this to erase a pattern background, if pdfwrite has pushed
+ * the NULL device to dispense with text (because its a strinwidth)
+ * we don't want to throw an error when erasing the pattern which
+ * fills the text, because pdfwrite manages the pattern itself and
+ * we don't need to erase hte background. More generally, since the
+ * null device is a bit bucket, it shouldn't really throw errors
+ * when asked to render anything at all.
+ */
+static int null_fill_rectangle_hl_color(gx_device *pdev,
+    const gs_fixed_rect *rect,
+    const gs_gstate *pgs, const gx_drawing_color *pdcolor,
+    const gx_clip_path *pcpath)
+{
+    return 0;
+}
+
 bool
 fwd_uses_fwd_cmap_procs(gx_device * dev)
 {
