@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -102,11 +102,11 @@ gdev_prn_open(gx_device * pdev)
     code = gdev_prn_allocate_memory(pdev, NULL, 0, 0);
     if (update_procs) {
         if (pdev->ObjectHandlerPushed) {
-            gx_copy_device_procs(&pdev->parent->procs, &pdev->procs, (gx_device_procs *)&gs_obj_filter_device.procs);
+            gx_copy_device_procs(pdev->parent, pdev, &gs_obj_filter_device);
             pdev = pdev->parent;
         }
         if (pdev->PageHandlerPushed)
-            gx_copy_device_procs(&pdev->parent->procs, &pdev->procs, (gx_device_procs *)&gs_flp_device.procs);
+            gx_copy_device_procs(pdev->parent, pdev, &gs_flp_device);
     }
     if (code < 0)
         return code;
@@ -177,10 +177,6 @@ gdev_prn_close(gx_device * pdev)
         code = gx_device_close_output_file(pdev, ppdev->fname, ppdev->file);
         ppdev->file = NULL;
     }
-    if (ppdev->saved_pages_list != NULL) {
-        gx_saved_pages_list_free(ppdev->saved_pages_list);
-        ppdev->saved_pages_list = NULL;
-    }
     return code;
 }
 
@@ -188,10 +184,15 @@ int
 gdev_prn_forwarding_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
 {
     gx_device_printer *ppdev = (gx_device_printer *)pdev;
+    int code;
 
-    if (ppdev->orig_procs.dev_spec_op)
-        return ppdev->orig_procs.dev_spec_op(pdev, dev_spec_op, data, size);
-    return gdev_prn_dev_spec_op(pdev, dev_spec_op, data, size);
+    /* if the device is a printer device, we will get here, but allow for a printer device */
+    /* that has a non-default dev_spec_op that may want to not support saved_pages, if so  */
+    /* that device can return an error from the supports_saved_pages spec_op.              */
+    code = ppdev->orig_procs.dev_spec_op(pdev, dev_spec_op, data, size);
+    if (dev_spec_op == gxdso_supports_saved_pages)	/* printer devices support saved pages */
+        return code == 0 ? 1: code;	/*default returns 0, but we still want saved-page support */
+    return code;
 }
 
 int
@@ -275,7 +276,7 @@ open_c:
              space >= space_params->BufferSpace &&
              !bufferSpace_is_exact
              ) {
-            space <<= 1;
+            space += space / 8;
             if (reallocate) {
                 base = gs_resize_object(buffer_memory,
                                         *the_memory, space,
@@ -561,7 +562,8 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
         COPY_PROC(update_spot_equivalent_colors);
         COPY_PROC(ret_devn_params);
         /* This can be set from the memory device (planar) or target */
-        fill_dev_proc(ppdev, put_image, ppdev->orig_procs.put_image);
+        if ( dev_proc(ppdev, put_image) == gx_default_put_image )
+            set_dev_proc(ppdev, put_image, ppdev->orig_procs.put_image);
 #undef COPY_PROC
         /* If using a command list, already opened the device. */
         if (is_command_list)
@@ -607,6 +609,19 @@ gdev_prn_free_memory(gx_device *pdev)
     return 0;
 }
 
+/* for saved pages, we need to make sure and free up the saved_pages_list */
+void
+gdev_prn_finalize(gx_device *pdev)
+{
+    gx_device_printer * const ppdev = (gx_device_printer *)pdev;
+
+    if (ppdev->saved_pages_list != NULL) {
+        gx_saved_pages_list_free(ppdev->saved_pages_list);
+        ppdev->saved_pages_list = NULL;
+    }
+}
+
+
 /* ------ Get/put parameters ------ */
 
 int
@@ -987,8 +1002,7 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
 {
     gx_device_printer * const ppdev = (gx_device_printer *)pdev;
     gs_devn_params *pdevn_params;
-    int outcode = 0, closecode = 0, errcode = 0, endcode;
-    bool upgraded_copypage = false;
+    int outcode = 0, errcode = 0, endcode, closecode = 0;
     int code;
 
     prn_finish_bg_print(ppdev);		/* finish any previous background printing */
@@ -1102,8 +1116,8 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
                 fflush(ppdev->file);
                 errcode = (ferror(ppdev->file) ? gs_note_error(gs_error_ioerror) : 0);
                 /* NB: background printing does this differently in its thread */
-                if (!upgraded_copypage)
-                    closecode = gdev_prn_close_printer(pdev);
+                closecode = gdev_prn_close_printer(pdev);
+
             }
         }
     }
@@ -1123,12 +1137,11 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
         return outcode;
     if (errcode < 0)
         return errcode;
-    if (closecode < 0)
-        return closecode;
     if (endcode < 0)
         return endcode;
     endcode = gx_finish_output_page(pdev, num_copies, flush);
-    return (endcode < 0 ? endcode : upgraded_copypage ? 1 : 0);
+    code = (endcode < 0 ? endcode : closecode < 0 ? closecode : 0);
+    return code;
 }
 
 int
