@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -165,6 +165,8 @@ static gs_memory_proc_free_string(i_free_string);
 static gs_memory_proc_register_root(i_register_root);
 static gs_memory_proc_unregister_root(i_unregister_root);
 static gs_memory_proc_enable_free(i_enable_free);
+static gs_memory_proc_set_object_type(i_set_object_type);
+static gs_memory_proc_defer_frees(i_defer_frees);
 
 /* We export the procedures for subclasses. */
 const gs_memory_procs_t gs_ref_memory_procs =
@@ -193,7 +195,9 @@ const gs_memory_procs_t gs_ref_memory_procs =
     i_free_string,
     i_register_root,
     i_unregister_root,
-    i_enable_free
+    i_enable_free,
+    i_set_object_type,
+    i_defer_frees
 };
 
 /*
@@ -767,7 +771,7 @@ ialloc_alloc_state(gs_memory_t * parent, uint clump_size)
     iimem->non_gc_memory = parent;
     iimem->thread_safe_memory = parent->thread_safe_memory;
     iimem->clump_size = clump_size;
-#ifdef MEMENTO
+#if defined(MEMENTO) || defined(SINGLE_OBJECT_MEMORY_BLOCKS_ONLY)
     iimem->large_size = 1;
 #else
     iimem->large_size = ((clump_size / 4) & -obj_align_mod) + 1;
@@ -1244,19 +1248,32 @@ i_alloc_struct_immovable(gs_memory_t * mem, gs_memory_type_ptr_t pstype,
     alloc_trace("|+<.", imem, cname, pstype, size, obj);
     return obj;
 }
+
+static inline bool
+alloc_array_check_size(ulong num_elements, ulong elt_size, ulong *lsize)
+{
+    int64_t s = (int64_t)num_elements * elt_size;
+    if (s > max_uint) {
+        return false;
+    }
+    *lsize = (ulong)s;
+    return true;
+}
+
 static byte *
 i_alloc_byte_array(gs_memory_t * mem, uint num_elements, uint elt_size,
                    client_name_t cname)
 {
     gs_ref_memory_t * const imem = (gs_ref_memory_t *)mem;
     obj_header_t *obj;
-
+    ulong lsize;
 #ifdef MEMENTO
     if (Memento_failThisEvent())
         return NULL;
 #endif
-
-    obj = alloc_obj(imem, (ulong) num_elements * elt_size,
+    if (alloc_array_check_size(num_elements, elt_size, &lsize) == false)
+        return NULL;
+    obj = alloc_obj(imem, lsize,
                     &st_bytes, ALLOC_DIRECT, cname);
 
     if_debug6m('A', mem, "[a%d:+b.]%s -bytes-*(%lu=%u*%u) = 0x%lx\n",
@@ -1271,13 +1288,14 @@ i_alloc_byte_array_immovable(gs_memory_t * mem, uint num_elements,
 {
     gs_ref_memory_t * const imem = (gs_ref_memory_t *)mem;
     obj_header_t *obj;
-
+    ulong lsize;
 #ifdef MEMENTO
     if (Memento_failThisEvent())
         return NULL;
 #endif
-
-    obj = alloc_obj(imem, (ulong) num_elements * elt_size,
+    if (alloc_array_check_size(num_elements, elt_size, &lsize) == false)
+        return NULL;
+    obj = alloc_obj(imem, lsize,
                     &st_bytes, ALLOC_IMMOVABLE | ALLOC_DIRECT,
                     cname);
 
@@ -1293,7 +1311,7 @@ i_alloc_struct_array(gs_memory_t * mem, uint num_elements,
 {
     gs_ref_memory_t * const imem = (gs_ref_memory_t *)mem;
     obj_header_t *obj;
-
+    ulong lsize;
 #ifdef MEMENTO
     if (Memento_failThisEvent())
         return NULL;
@@ -1307,9 +1325,9 @@ i_alloc_struct_array(gs_memory_t * mem, uint num_elements,
         return NULL;		/* fail */
     }
 #endif
-    obj = alloc_obj(imem,
-                    (ulong) num_elements * pstype->ssize,
-                    pstype, ALLOC_DIRECT, cname);
+    if (alloc_array_check_size(num_elements, pstype->ssize, &lsize) == false)
+        return NULL;
+    obj = alloc_obj(imem, lsize, pstype, ALLOC_DIRECT, cname);
     if_debug7m('A', mem, "[a%d:+<.]%s %s*(%lu=%u*%u) = 0x%lx\n",
                alloc_trace_space(imem), client_name_string(cname),
                struct_type_name_string(pstype),
@@ -1323,16 +1341,16 @@ i_alloc_struct_array_immovable(gs_memory_t * mem, uint num_elements,
 {
     gs_ref_memory_t * const imem = (gs_ref_memory_t *)mem;
     obj_header_t *obj;
-
+    ulong lsize;
 #ifdef MEMENTO
     if (Memento_failThisEvent())
         return NULL;
 #endif
 
     ALLOC_CHECK_SIZE(mem,pstype);
-    obj = alloc_obj(imem,
-                    (ulong) num_elements * pstype->ssize,
-                    pstype, ALLOC_IMMOVABLE | ALLOC_DIRECT, cname);
+    if (alloc_array_check_size(num_elements, pstype->ssize, &lsize) == false)
+        return NULL;
+    obj = alloc_obj(imem, lsize, pstype, ALLOC_IMMOVABLE | ALLOC_DIRECT, cname);
     if_debug7m('A', mem, "[a%d|+<.]%s %s*(%lu=%u*%u) = 0x%lx\n",
                alloc_trace_space(imem), client_name_string(cname),
                struct_type_name_string(pstype),
@@ -1783,6 +1801,20 @@ i_enable_free(gs_memory_t * mem, bool enable)
             mem->procs.free_string = gs_ignore_free_string;
 }
 
+static void i_set_object_type(gs_memory_t *mem, void *ptr, gs_memory_type_ptr_t type)
+{
+    obj_header_t *pp;
+
+    if (ptr == 0)
+        return;
+    pp = (obj_header_t *) ptr - 1;
+    pp->o_type = type;
+}
+
+static void i_defer_frees(gs_memory_t *mem, int defer)
+{
+}
+
 /* ------ Internal procedures ------ */
 
 /* Compute the amount of free object space by scanning free lists. */
@@ -2490,8 +2522,10 @@ alloc_close_clump(gs_ref_memory_t * mem)
 {
 #ifdef DEBUG
     if (gs_debug_c('A')) {
-        dmlprintf1((const gs_memory_t *)mem, "[a%d]", alloc_trace_space(mem));
-        dmprintf_clump((const gs_memory_t *)mem, "closing clump", mem->cc);
+        if (mem->cc != NULL) {
+            dmlprintf1((const gs_memory_t *)mem, "[a%d]", alloc_trace_space(mem));
+            dmprintf_clump((const gs_memory_t *)mem, "closing clump", mem->cc);
+        }
     }
 #endif
 }
@@ -2502,8 +2536,10 @@ alloc_open_clump(gs_ref_memory_t * mem)
 {
 #ifdef DEBUG
     if (gs_debug_c('A')) {
-        dmlprintf1((const gs_memory_t *)mem, "[a%d]", alloc_trace_space(mem));
-        dmprintf_clump((const gs_memory_t *)mem, "opening clump", mem->cc);
+        if (mem->cc != NULL) {
+            dmlprintf1((const gs_memory_t *)mem, "[a%d]", alloc_trace_space(mem));
+            dmprintf_clump((const gs_memory_t *)mem, "opening clump", mem->cc);
+        }
     }
 #endif
 }
