@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -353,7 +353,7 @@ gx_concrete_space_DeviceN(const gs_color_space * pcs,
         /* Need to handle PS CIE space */
         if (gs_color_space_is_PSCIE(pcs->base_space)) {
             if (pcs->base_space->icc_equivalent == NULL) {
-                gs_colorspace_set_icc_equivalent(pcs->base_space,
+                (void)gs_colorspace_set_icc_equivalent(pcs->base_space,
                                                     &is_lab, pgs->memory);
             }
             return (pcs->base_space->icc_equivalent);
@@ -401,11 +401,15 @@ gx_remap_DeviceN(const gs_client_color * pcc, const gs_color_space * pcs,
             mapped = gx_remap_named_color(pcc, pcs, pdc, pgs, dev, select);
         }
         if (!mapped) {
+            cmm_dev_profile_t *dev_profile;
+            code = dev_proc(dev, get_profile)(dev, &dev_profile);
+            if (code < 0)
+                return code;
             code = (*pcs->type->concretize_color)(pcc, pcs, conc, pgs, dev);
             if (code < 0)
                 return code;
             pconcs = cs_concrete_space(pcs, pgs);
-            code = (*pconcs->type->remap_concrete_color)(conc, pconcs, pdc, pgs, dev, select);
+            code = (*pconcs->type->remap_concrete_color)(pconcs, conc, pdc, pgs, dev, select, dev_profile);
         }
         /* Save original color space and color info into dev color */
         i = any_abs(i);
@@ -469,13 +473,16 @@ gx_concretize_DeviceN(const gs_client_color * pc, const gs_color_space * pcs,
             rescale_cie_colors(pacs, &cc);
             /* If we have not yet created the profile do that now */
             if (pacs->icc_equivalent == NULL) {
-                gs_colorspace_set_icc_equivalent(pacs, &(is_lab), pgs->memory);
+                code = gs_colorspace_set_icc_equivalent(pacs, &(is_lab), pgs->memory);
+                if (code < 0)
+                   return code;
             }
             /* Use the ICC equivalent color space */
             pacs = pacs->icc_equivalent;
         }
-        if (pacs->cmm_icc_profile_data->data_cs == gsCIELAB ||
-            pacs->cmm_icc_profile_data->islab) {
+        if (pacs->cmm_icc_profile_data &&
+            (pacs->cmm_icc_profile_data->data_cs == gsCIELAB ||
+            pacs->cmm_icc_profile_data->islab)) {
             /* Get the data in a form that is concrete for the CMM */
             cc.paint.values[0] /= 100.0;
             cc.paint.values[1] = (cc.paint.values[1]+128)/255.0;
@@ -494,9 +501,10 @@ gx_concretize_DeviceN(const gs_client_color * pc, const gs_color_space * pcs,
 }
 
 static int
-gx_remap_concrete_DeviceN(const frac * pconc, const gs_color_space * pcs,
-        gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-                          gs_color_select_t select)
+gx_remap_concrete_DeviceN(const gs_color_space * pcs, const frac * pconc,
+                          gx_device_color * pdc, const gs_gstate * pgs,
+                          gx_device * dev, gs_color_select_t select,
+                          const cmm_dev_profile_t *dev_profile)
 {
     int code = 0;
 
@@ -509,19 +517,16 @@ gx_remap_concrete_DeviceN(const frac * pconc, const gs_color_space * pcs,
 #endif
     if (pgs->color_component_map.use_alt_cspace) {
         const gs_color_space *pacs = pcs->base_space;
-
         return (*pacs->type->remap_concrete_color)
-                                (pconc, pacs, pdc, pgs, dev, select);
+                                (pacs, pconc, pdc, pgs, dev, select, dev_profile);
     } else {
     /* If we are going DeviceN out to real sep device that understands these,
        and if the destination profile is DeviceN, we print the colors directly.
        Make sure to disable the DeviceN profile color map so that is does not
        get used in gx_remap_concrete_devicen.  We probably should pass something
        through here but it is a pain due to the change in the proc. */
-        cmm_dev_profile_t *dev_profile;
         bool temp_val;
 
-        code = dev_proc(dev, get_profile)(dev, &dev_profile);
         if (dev_profile->spotnames != NULL && code >= 0) {
             temp_val = dev_profile->spotnames->equiv_cmyk_set;
             dev_profile->spotnames->equiv_cmyk_set = false;
