@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /*  Data type definitions when using the gscms  */
@@ -119,7 +119,7 @@ typedef struct gscms_procs_s {
 } gscms_procs_t;
 
 /* Allow different methods for releasing the opaque profile contents */
-typedef void(*gscms_free_profile_proc_t) (void *profile_handle);
+typedef void(*gscms_free_profile_proc_t) (void *profile_handle, gs_memory_t *memory);
 
 /* Enumerate the ICC rendering intents and other parameters.  A note on
    these.  0-3 are for different values.   4-7 are for Override cases
@@ -138,7 +138,10 @@ typedef enum {
     gsRELATIVECOLORIMETRIC_OR,  /* of when the source ri is going to */
     gsSATURATION_OR,            /* override the destination profile intent */
     gsABSOLUTECOLORIMETRIC_OR,  /* in particular through the clist */
-    gsRINOTSPECIFIED = 8            /* Used to ignore value when source based setting */
+    gsRINOTSPECIFIED = 8,       /* Used to ignore value when source based setting */
+
+    /* Stop modern C shrinking this enum to a byte */
+    gsicc_rendering_intent__FORCE_SIZE= 0x10000
 } gsicc_rendering_intents_t;
 
 /* We make an enumerated type in case someone wants to add different types
@@ -150,7 +153,10 @@ typedef enum {
     gsBLACKPTCOMP_ON,
     gsBLACKPTCOMP_OFF_OR = 4, /* These are needed for keeping track of the  */
     gsBLACKPTCOMP_ON_OR,      /* source blackpt is to overide dest. setting */
-    gsBPNOTSPECIFIED = 8     /* Used to ignore value when source based setting */
+    gsBPNOTSPECIFIED = 8,    /* Used to ignore value when source based setting */
+
+    /* Stop modern C shrinking this enum to a byte */
+    gsicc_blackptcomp__FORCE_SIZE= 0x10000
 } gsicc_blackptcomp_t;
 
 /* This is used mainly for when the sourcegtag option specifies us to use no
@@ -167,7 +173,10 @@ typedef enum {
     gsBLACKPRESERVE_OFF = 0,
     gsBLACKPRESERVE_KONLY,
     gsBLACKPRESERVE_KPLANE,
-    gsBKPRESNOTSPECIFIED = 8   /* Used to ignore value when source based setting */
+    gsBKPRESNOTSPECIFIED = 8,   /* Used to ignore value when source based setting */
+
+    /* Stop modern C shrinking this enum to a byte */
+    gsicc_blackpreserve__FORCE_SIZE= 0x10000
 } gsicc_blackpreserve_t;
 
 #define gsRI_OVERRIDE 0x4
@@ -186,7 +195,8 @@ typedef enum {
     gsPROOFPROFILE,
     gsLINKPROFILE,
     gsOIPROFILE,
-    gsPRPROFILE
+    gsPRPROFILE,
+    gsBLENDPROFILE
 } gsicc_profile_types_t;
 
 typedef enum {
@@ -199,11 +209,11 @@ typedef enum {
 
 /* The default is "unknown" which has value 0 and by default devices don't encode tags */
 typedef enum {
-    GS_UNKNOWN_TAG = 0x0,
+    GS_UNTOUCHED_TAG = 0x0,	/* UNTOUCHED *must* be 0 -- transparency code relies on this */
     GS_TEXT_TAG = 0x1,
     GS_IMAGE_TAG = 0x2,
     GS_PATH_TAG = 0x4,
-    GS_UNTOUCHED_TAG = 0x8,
+    GS_UNKNOWN_TAG = 0x40,
     GS_DEVICE_ENCODES_TAGS = 0x80
 } gs_graphics_type_tag_t;
 
@@ -260,11 +270,12 @@ struct gsicc_namelist_s {
 };
 
 /* Destination profiles for different objects */
-typedef struct cmm_dev_profile_s {
+struct cmm_dev_profile_s {
         cmm_profile_t  *device_profile[NUM_DEVICE_PROFILES];
         cmm_profile_t  *proof_profile;
         cmm_profile_t  *link_profile;
         cmm_profile_t  *oi_profile;  /* output intent profile */
+        cmm_profile_t  *blend_profile; /* blending color space */
         cmm_profile_t  *postren_profile;  /* Profile for use by devices post render */
         gsicc_rendering_param_t rendercond[NUM_DEVICE_PROFILES];
         bool devicegraytok;        /* Used for forcing gray to pure black */
@@ -277,7 +288,7 @@ typedef struct cmm_dev_profile_s {
         bool prebandthreshold;     /* Used to indicate use of HT pre-clist */
         gs_memory_t *memory;
         rc_header rc;
-} cmm_dev_profile_t;
+};
 
 /*  Used so that we can specify if we want to link with Device input color spaces
     during the link creation process. For the DeviceN case, the DeviceN profile
@@ -431,6 +442,7 @@ typedef struct gsicc_hashlink_s {
 
 struct gsicc_link_s {
     void *link_handle;
+    gs_memory_t *memory;
     gscms_procs_t procs;
     gsicc_hashlink_t hashcode;
     struct gsicc_link_cache_s *icc_link_cache;
