@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -169,6 +169,9 @@ gs_cspace_new_ICC(gs_memory_t *pmem, gs_gstate * pgs, int components)
     int code = 0;
     gs_color_space *pcspace = gs_cspace_alloc(pmem, &gs_color_space_type_ICC);
 
+    if (pcspace == NULL)
+        return pcspace;
+
     switch (components) {
         case -1: /* alpha case */
             if (icc_manage->smask_profiles == NULL) {
@@ -463,8 +466,7 @@ check_cmyk_color_model_comps(gx_device * dev)
     gx_device_color_info *          pcinfo = &dev->color_info;
     uchar                           ncomps = pcinfo->num_components;
     int                             cyan_c, magenta_c, yellow_c, black_c;
-    const gx_cm_color_map_procs *   pprocs;
-    cm_map_proc_cmyk((*map_cmyk));
+    subclass_color_mappings         scm;
     frac                            frac_14 = frac_1 / 4;
     frac                            out[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_index                  process_comps;
@@ -498,22 +500,18 @@ check_cmyk_color_model_comps(gx_device * dev)
         return 0;
 
     /* check the mapping */
-    pprocs = get_color_mapping_procs_subclass(dev);
+    scm = get_color_mapping_procs_subclass(dev);
 
-    if ( pprocs == 0 ||
-         (map_cmyk = pprocs->map_cmyk) == 0                            )
-        return 0;
-
-    map_cmyk_subclass(pprocs, dev, frac_14, frac_0, frac_0, frac_0, out);
+    map_cmyk_subclass(scm, frac_14, frac_0, frac_0, frac_0, out);
     if (!check_single_comp(cyan_c, frac_14, ncomps, out))
         return 0;
-    map_cmyk_subclass(pprocs, dev, frac_0, frac_14, frac_0, frac_0, out);
+    map_cmyk_subclass(scm, frac_0, frac_14, frac_0, frac_0, out);
     if (!check_single_comp(magenta_c, frac_14, ncomps, out))
         return 0;
-    map_cmyk_subclass(pprocs, dev, frac_0, frac_0, frac_14, frac_0, out);
+    map_cmyk_subclass(scm, frac_0, frac_0, frac_14, frac_0, out);
     if (!check_single_comp(yellow_c, frac_14, ncomps, out))
         return false;
-    map_cmyk_subclass(pprocs, dev, frac_0, frac_0, frac_0, frac_14, out);
+    map_cmyk_subclass(scm, frac_0, frac_0, frac_0, frac_14, out);
     if (!check_single_comp(black_c, frac_14, ncomps, out))
         return 0;
 
@@ -845,7 +843,7 @@ gx_cspace_is_linear_default(const gs_color_space *cs, const gs_gstate * pgs,
     /* With nc == 4 assuming a convex plain quadrangle in the client color space. */
     int code;
 
-    if (dev->color_info.separable_and_linear != GX_CINFO_SEP_LIN)
+    if (!colors_are_separable_and_linear(&dev->color_info))
         return_error(gs_error_rangecheck);
     if (c2 == NULL)
         return gx_cspace_is_linear_in_line(cs, pgs, dev, c0, c1, smoothness);
