@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -347,20 +347,9 @@ static void
 gx_device_make_struct_type(gs_memory_struct_type_t *st,
                            const gx_device *dev)
 {
-    const gx_device_procs *procs = dev->static_procs;
-
-    /*
-     * Try to figure out whether this is a forwarding device.  For printer
-     * devices, we rely on the prototype referencing the correct structure
-     * descriptor; for other devices, we look for a likely forwarding
-     * procedure in the vector.  The algorithm isn't foolproof, but it's the
-     * best we can come up with.
-     */
-    if (procs == 0)
-        procs = &dev->procs;
     if (dev->stype)
         *st = *dev->stype;
-    else if (procs->get_xfont_procs == gx_forward_get_xfont_procs)
+    else if (dev_proc(dev, get_xfont_procs) == gx_forward_get_xfont_procs)
         *st = st_device_forward;
     else
         *st = st_device;
@@ -409,8 +398,10 @@ gs_copydevice2(gx_device ** pnew_dev, const gx_device * dev, bool keep_open,
      */
     new_dev = gs_alloc_struct_immovable(mem, gx_device, new_std,
                                         "gs_copydevice(device)");
-    if (new_dev == 0)
+    if (new_dev == 0) {
+        gs_free_object(mem->non_gc_memory, a_std, "gs_copydevice(stype)");
         return_error(gs_error_VMerror);
+    }
     gx_device_init(new_dev, dev, mem, false);
     gx_device_set_procs(new_dev);
     new_dev->stype = new_std;
@@ -466,7 +457,7 @@ static void
 gs_gstate_update_device(gs_gstate *pgs, gx_device *dev)
 {
     gx_set_cmap_procs(pgs, dev);
-    gx_unset_dev_color(pgs);
+    gx_unset_both_dev_colors(pgs);
 }
 
 int
@@ -511,6 +502,10 @@ gs_setdevice_no_erase(gs_gstate * pgs, gx_device * dev)
         }
         /* Also, if the device profile is not yet set then take care of that
            before we start filling pages, if we can */
+        /* Although device methods should not be NULL, they are not completely filled in until
+         * gx_device_fill_in_procs is called, and its possible for us to get here before this
+         * happens, so we *must* make sure the method is not NULL before we use it.
+         */
         if (dev->procs.get_profile != NULL) {
             code = dev_proc(dev, get_profile)(dev, &dev_profile);
             if (code < 0) {
@@ -540,6 +535,10 @@ gs_setdevice_no_erase(gs_gstate * pgs, gx_device * dev)
          */
         if (libctx->io_device_table != NULL) {
             cmm_dev_profile_t *dev_profile;
+            /* Although device methods should not be NULL, they are not completely filled in until
+             * gx_device_fill_in_procs is called, and its possible for us to get here before this
+             * happens, so we *must* make sure the method is not NULL before we use it.
+             */
             if (dev->procs.get_profile != NULL) {
                 code = dev_proc(dev, get_profile)(dev, &dev_profile);
                 if (code < 0) {
@@ -652,6 +651,17 @@ gs_make_null_device(gx_device_null *dev_null, gx_device *dev,
         set_dev_proc(dn, decode_color, gx_forward_decode_color);
         set_dev_proc(dn, get_profile, gx_forward_get_profile);
         set_dev_proc(dn, set_graphics_type_tag, gx_forward_set_graphics_type_tag);
+        set_dev_proc(dn, begin_transparency_group, gx_default_begin_transparency_group);
+        set_dev_proc(dn, end_transparency_group, gx_default_end_transparency_group);
+        set_dev_proc(dn, begin_transparency_mask, gx_default_begin_transparency_mask);
+        set_dev_proc(dn, end_transparency_mask, gx_default_end_transparency_mask);
+        set_dev_proc(dn, discard_transparency_layer, gx_default_discard_transparency_layer);
+        set_dev_proc(dn, pattern_manage, gx_default_pattern_manage);
+        set_dev_proc(dn, push_transparency_state, gx_default_push_transparency_state);
+        set_dev_proc(dn, pop_transparency_state, gx_default_pop_transparency_state);
+        set_dev_proc(dn, put_image, gx_default_put_image);
+        set_dev_proc(dn, copy_planes, gx_default_copy_planes);
+        set_dev_proc(dn, copy_alpha_hl_color, gx_default_no_copy_alpha_hl_color);
         dn->graphics_type_tag = dev->graphics_type_tag;	/* initialize to same as target */
         gx_device_copy_color_params(dn, dev);
     }
@@ -680,9 +690,11 @@ gx_device_retain(gx_device *dev, bool retained)
 int
 gs_nulldevice(gs_gstate * pgs)
 {
+    int code = 0;
+
     if (pgs->device == 0 || !gx_device_is_null(pgs->device)) {
         gx_device *ndev;
-        int code = gs_copydevice(&ndev, (const gx_device *)&gs_null_device,
+        code = gs_copydevice(&ndev, (const gx_device *)&gs_null_device,
                                  pgs->memory);
 
         if (code < 0)
@@ -704,9 +716,10 @@ gs_nulldevice(gs_gstate * pgs)
             set_dev_proc(ndev, get_profile, gx_default_get_profile);
         }
 
-        return gs_setdevice_no_erase(pgs, ndev);
+        if ((code = gs_setdevice_no_erase(pgs, ndev)) < 0)
+            gs_free_object(pgs->memory, ndev, "gs_copydevice(device)");
     }
-    return 0;
+    return code;
 }
 
 /* Close a device.  The client is responsible for ensuring that */
@@ -752,8 +765,6 @@ gx_device_raster(const gx_device * dev, bool pad)
     raster = (uint)((bits + 7) >> 3);
     if (!pad)
         return raster;
-    if (dev->pad > 0)
-        raster += dev->pad;
     l2align = dev->log2_align_mod;
     if (l2align < log2_align_bitmap_mod)
         l2align = log2_align_bitmap_mod;
@@ -770,8 +781,6 @@ gx_device_raster_chunky(const gx_device * dev, bool pad)
     raster = (uint)((bits + 7) >> 3);
     if (!pad)
         return raster;
-    if (dev->pad > 0)
-        raster += dev->pad;
     l2align = dev->log2_align_mod;
     if (l2align < log2_align_bitmap_mod)
         l2align = log2_align_bitmap_mod;
@@ -783,12 +792,8 @@ gx_device_raster_plane(const gx_device * dev, const gx_render_plane_t *render_pl
     ulong bpc = (render_plane && render_plane->index >= 0 ?
         render_plane->depth : dev->color_info.depth/(dev->is_planar ? dev->color_info.num_components : 1));
     ulong bits = (ulong) dev->width * bpc;
-    ulong raster;
     int l2align;
 
-    raster = (uint)((bits + 7) >> 3);
-    if (dev->pad > 0)
-        raster += dev->pad;
     l2align = dev->log2_align_mod;
     if (l2align < log2_align_bitmap_mod)
         l2align = log2_align_bitmap_mod;
