@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Default device parameters for Ghostscript library */
@@ -78,6 +78,7 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     gsicc_rendering_intents_t profile_intents[NUM_DEVICE_PROFILES];
     gsicc_blackptcomp_t blackptcomps[NUM_DEVICE_PROFILES];
     gsicc_blackpreserve_t blackpreserve[NUM_DEVICE_PROFILES];
+    int color_accuracy = MAX_COLOR_ACCURACY;
     int depth = dev->color_info.depth;
     cmm_dev_profile_t *dev_profile;
     char null_str[1]={'\0'};
@@ -218,7 +219,7 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
         return param_write_int(plist, "MaxPatternBitmap", &dev->MaxPatternBitmap);
     }
     if (strcmp(Param, "PageUsesTransparency") == 0) {
-        param_write_bool(plist, "PageUsesTransparency", &dev->page_uses_transparency);
+        return param_write_bool(plist, "PageUsesTransparency", &dev->page_uses_transparency);
     }
     if (strcmp(Param, "MaxBitmap") == 0) {
         return param_write_long(plist, "MaxBitmap", &(dev->space_params.MaxBitmap));
@@ -235,11 +236,16 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     if (strcmp(Param, "BufferSpace") == 0) {
         return param_write_long(plist, "BufferSpace", &dev->space_params.BufferSpace);
     }
+    if (strcmp(Param, "InterpolateControl") == 0) {
+        int interpolate_control = dev->interpolate_control;
+        return param_write_int(plist, "InterpolateControl", &interpolate_control);
+    }
     if (strcmp(Param, "LeadingEdge") == 0) {
         if (dev->LeadingEdge & LEADINGEDGE_SET_MASK) {
             int leadingedge = dev->LeadingEdge & LEADINGEDGE_MASK;
             return param_write_int(plist, "LeadingEdge", &leadingedge);
-        }
+        } else
+            return param_write_null(plist, "LeadingEdge");
     }
 
     if (dev->color_info.num_components > 1) {
@@ -275,7 +281,11 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     /* Check if the device profile is null.  If it is, then we need to
        go ahead and get it set up at this time.  If the proc is not
        set up yet then we are not going to do anything yet */
-    if (dev->procs.get_profile != NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) != NULL) {
         int code;
         code = dev_proc(dev, get_profile)(dev,  &dev_profile);
         if (code < 0)
@@ -403,6 +413,9 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     if (strcmp(Param, "TextICCProfile") == 0) {
         return param_write_string(plist,"TextICCProfile", &(profile_array[3]));
     }
+    if (strcmp(Param, "ColorAccuracy") == 0) {
+        return param_write_int(plist, "ColorAccuracy", (const int *)(&(color_accuracy)));
+    }
     if (strcmp(Param, "RenderIntent") == 0) {
         return param_write_int(plist,"RenderIntent", (const int *) (&(profile_intents[0])));
     }
@@ -496,6 +509,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     bool sim_overprint = false;  /* By default do not simulate overprinting */
     bool prebandthreshold = true, temp_bool;
     int k;
+    int color_accuracy = MAX_COLOR_ACCURACY;
     gs_param_float_array msa, ibba, hwra, ma;
     gs_param_string_array scna;
     char null_str[1]={'\0'};
@@ -541,11 +555,22 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     /* Check if the device profile is null.  If it is, then we need to
        go ahead and get it set up at this time.  If the proc is not
        set up yet then we are not going to do anything yet */
-    if (dev->procs.get_profile != NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) != NULL) {
         code = dev_proc(dev, get_profile)(dev,  &dev_profile);
+        if (code < 0)
+            return code;
+
         if (dev_profile == NULL) {
             code = gsicc_init_device_profile_struct(dev, NULL, 0);
+            if (code < 0)
+                return code;
             code = dev_proc(dev, get_profile)(dev,  &dev_profile);
+            if (code < 0)
+                return code;
         }
         /* It is possible that the current device profile name is NULL if we
            have a pdf14 device in line with a transparency group that is in a
@@ -665,7 +690,8 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
         (code = param_write_string(plist, "BlendColorProfile", &(blend_profile))) < 0 ||
         (code = param_write_string(plist,"DeviceLinkProfile", &(link_profile))) < 0 ||
         (code = param_write_string(plist,"ICCOutputColors", &(icc_colorants))) < 0 ||
-        (code = param_write_int(plist,"RenderIntent", (const int *) (&(profile_intents[0])))) < 0 ||
+        (code = param_write_int(plist, "RenderIntent", (const int *)(&(profile_intents[0])))) < 0 ||
+        (code = param_write_int(plist, "ColorAccuracy", (const int *)(&(color_accuracy)))) < 0 ||
         (code = param_write_int(plist,"GraphicIntent", (const int *) &(profile_intents[1]))) < 0 ||
         (code = param_write_int(plist,"ImageIntent", (const int *) &(profile_intents[2]))) < 0 ||
         (code = param_write_int(plist,"TextIntent", (const int *) &(profile_intents[3]))) < 0 ||
@@ -699,7 +725,8 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
         (code = param_write_long(plist, "BandBufferSpace", &dev->space_params.band.BandBufferSpace)) < 0 ||
         (code = param_write_int(plist, "BandHeight", &dev->space_params.band.BandHeight)) < 0 ||
         (code = param_write_int(plist, "BandWidth", &dev->space_params.band.BandWidth)) < 0 ||
-        (code = param_write_long(plist, "BufferSpace", &dev->space_params.BufferSpace)) < 0
+        (code = param_write_long(plist, "BufferSpace", &dev->space_params.BufferSpace)) < 0 ||
+        (code = param_write_int(plist, "InterpolateControl", &dev->interpolate_control)) < 0
         )
         return code;
 
@@ -707,7 +734,8 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     if (dev->LeadingEdge & LEADINGEDGE_SET_MASK) {
         int leadingedge = dev->LeadingEdge & LEADINGEDGE_MASK;
         code = param_write_int(plist, "LeadingEdge", &leadingedge);
-    }
+    } else
+        code = param_write_null(plist, "LeadingEdge");
     if (code < 0)
         return code;
 
@@ -987,7 +1015,11 @@ gx_default_put_graydetection(bool graydetection, gx_device * dev)
     int code = 0;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1019,7 +1051,11 @@ gx_default_put_graytok(bool graytok, gx_device * dev)
     int code = 0;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1053,7 +1089,11 @@ gx_default_put_prebandthreshold(bool prebandthreshold, gx_device * dev)
     int code = 0;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1087,7 +1127,11 @@ gx_default_put_usefastcolor(bool fastcolor, gx_device * dev)
     int code = 0;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1121,7 +1165,11 @@ gx_default_put_simulateoverprint(bool sim_overprint, gx_device * dev)
     int code = 0;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1156,7 +1204,11 @@ gx_default_put_intent(gsicc_rendering_intents_t icc_intent, gx_device * dev,
     int code;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1170,6 +1222,8 @@ gx_default_put_intent(gsicc_rendering_intents_t icc_intent, gx_device * dev,
         code = gsicc_set_device_profile_intent(dev, icc_intent, index);
     } else {
         code = dev_proc(dev, get_profile)(dev,  &profile_struct);
+        if (code < 0)
+            return code;
         if (profile_struct == NULL) {
             /* Create now  */
             dev->icc_struct = gsicc_new_device_profile_array(dev->memory);
@@ -1188,7 +1242,11 @@ gx_default_put_blackpreserve(gsicc_blackpreserve_t blackpreserve, gx_device * de
     int code;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1225,7 +1283,11 @@ gx_default_put_blackptcomp(gsicc_blackptcomp_t blackptcomp, gx_device * dev,
     int code;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1345,6 +1407,7 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
     int tab = dev->color_info.anti_alias.text_bits;
     int gab = dev->color_info.anti_alias.graphics_bits;
     int mpbm = dev->MaxPatternBitmap;
+    int ic = dev->interpolate_control;
     bool page_uses_transparency = dev->page_uses_transparency;
     gdev_space_params sp = dev->space_params;
     gdev_space_params save_sp = dev->space_params;
@@ -1354,6 +1417,7 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
     gs_param_string cms, pagelist;
     int leadingedge = dev->LeadingEdge;
     int k;
+    int color_accuracy;
     bool devicegraytok = true;
     bool graydetection = false;
     bool usefastcolor = false;
@@ -1366,6 +1430,7 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
                                                gsIMAGEPROFILE,
                                                gsTEXTPROFILE};
 
+    color_accuracy = gsicc_currentcoloraccuracy(dev->memory);
     if (dev->icc_struct != NULL) {
         for (k = 0; k < NUM_DEVICE_PROFILES; k++) {
             rend_intent[k] = dev->icc_struct->rendercond[k].rendering_intent;
@@ -1473,7 +1538,6 @@ e:	param_signal_error(plist, param_name, ecode);\
             if (param_read_null(plist, "LeadingEdge") == 0) {
                 /* if param is null, clear explicitly-set flag */
                 leadingedge &= ~LEADINGEDGE_SET_MASK;
-                code = 0;
             } else {
                 ecode = code;
             }
@@ -1666,6 +1730,11 @@ nce:
         ecode = code;
         param_signal_error(plist, param_name, ecode);
     }
+    if ((code = param_read_int(plist, (param_name = "ColorAccuracy"),
+                                                        &color_accuracy)) < 0) {
+        ecode = code;
+        param_signal_error(plist, param_name, ecode);
+    }
     if ((code = param_read_bool(plist, (param_name = "DeviceGrayToK"),
                                                         &devicegraytok)) < 0) {
         ecode = code;
@@ -1703,6 +1772,8 @@ nce:
         ecode = code;
     if ((code = param_read_int(plist, "MaxPatternBitmap", &mpbm)) < 0)
         ecode = code;
+    if ((code = param_read_int(plist, "InterpolateControl", &ic)) < 0)
+        ecode = code;
     if ((code = param_read_bool(plist, (param_name = "PageUsesTransparency"),
                                 &page_uses_transparency)) < 0) {
         ecode = code;
@@ -1988,6 +2059,7 @@ label:\
                         dev->color_info.max_color), gab);
     dev->LockSafetyParams = locksafe;
     dev->MaxPatternBitmap = mpbm;
+    dev->interpolate_control = ic;
     dev->space_params = sp;
     dev->page_uses_transparency = page_uses_transparency;
     gx_device_decache_colors(dev);
@@ -2034,6 +2106,7 @@ label:\
                 return code;
         }
     }
+    gsicc_setcoloraccuracy(dev->memory, color_accuracy);
     code = gx_default_put_graytok(devicegraytok, dev);
     if (code < 0)
         return code;
@@ -2108,7 +2181,7 @@ param_MediaSize(gs_param_list * plist, gs_param_name pname,
             ecode = gs_note_error(gs_error_rangecheck);
 #define max_coord (max_fixed / fixed_1)
 #if max_coord < max_int
-        else if (width_new > max_coord || height_new > max_coord)
+        else if (width_new > (long)max_coord || height_new > (long)max_coord)
             ecode = gs_note_error(gs_error_limitcheck);
 #endif
 #undef max_coord
