@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Display PostScript extensions */
 #include "gx.h"
 #include "gserrors.h"
@@ -25,10 +27,10 @@
 /* ---------------- View clipping ---------------- */
 
 /* Forward references */
-static int common_viewclip(gs_state *, int);
+static int common_viewclip(gs_gstate *, int);
 
 int
-gs_initviewclip(gs_state * pgs)
+gs_initviewclip(gs_gstate * pgs)
 {
     gx_clip_path *pcpath = pgs->view_clip;
 
@@ -40,13 +42,13 @@ gs_initviewclip(gs_state * pgs)
 }
 
 int
-gs_viewclip(gs_state * pgs)
+gs_viewclip(gs_gstate * pgs)
 {
     return common_viewclip(pgs, gx_rule_winding_number);
 }
 
 int
-gs_eoviewclip(gs_state * pgs)
+gs_eoviewclip(gs_gstate * pgs)
 {
     return common_viewclip(pgs, gx_rule_even_odd);
 }
@@ -54,7 +56,7 @@ gs_eoviewclip(gs_state * pgs)
 /* This code is (almost) copied from common_clip in gspath.c. */
 /* Someday we'll find a way to merge them. */
 static int
-common_viewclip(gs_state * pgs, int rule)
+common_viewclip(gs_gstate * pgs, int rule)
 {
     gs_fixed_rect bbox;
     gx_clip_path rpath;
@@ -84,7 +86,7 @@ common_viewclip(gs_state * pgs, int rule)
 }
 
 int
-gs_viewclippath(gs_state * pgs)
+gs_viewclippath(gs_gstate * pgs)
 {
     gx_path cpath;
     gx_clip_path *pcpath = pgs->view_clip;
