@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -1209,8 +1209,7 @@ gx_ht_install(gs_gstate * pgs, const gs_halftone * pht,
         new_ht->rc = rc;
     }
     pgs->halftone = new_ht;
-    gx_unset_dev_color(pgs);
-    gx_unset_alt_dev_color(pgs);
+    gx_unset_both_dev_colors(pgs);
     return 0;
 }
 
@@ -1229,51 +1228,73 @@ gx_gstate_set_effective_xfer(gs_gstate * pgs)
     gx_device_halftone *pdht = pgs->dev_ht;
     gx_transfer_map *pmap;
     gx_ht_order *porder;
-    int i, component_num;
+    int i, component_num, non_id_count;
 
+    non_id_count = (pgs->set_transfer.gray->proc == &gs_identity_transfer) ? 0 : GX_DEVICE_COLOR_MAX_COMPONENTS;
     for (i = 0; i < GX_DEVICE_COLOR_MAX_COMPONENTS; i++)
         pgs->effective_transfer[i] = pgs->set_transfer.gray;    /* default */
 
     /* Check if we have a transfer functions from setcolortransfer */
     if (pgs->set_transfer.red) {
         component_num = pgs->set_transfer.red_component_num;
-        if (component_num >= 0)
-            pgs->effective_transfer[component_num] = pgs->set_transfer.red;;
+        if (component_num >= 0) {
+            if (pgs->effective_transfer[component_num]->proc != &gs_identity_transfer)
+               non_id_count--;
+            pgs->effective_transfer[component_num] = pgs->set_transfer.red;
+            if (pgs->effective_transfer[component_num]->proc != &gs_identity_transfer)
+               non_id_count++;
+        }
     }
     if (pgs->set_transfer.green) {
         component_num = pgs->set_transfer.green_component_num;
-        if (component_num >= 0)
+        if (component_num >= 0) {
+            if (pgs->effective_transfer[component_num]->proc != &gs_identity_transfer)
+               non_id_count--;
             pgs->effective_transfer[component_num] = pgs->set_transfer.green;
+            if (pgs->effective_transfer[component_num]->proc != &gs_identity_transfer)
+               non_id_count++;
+        }
     }
     if (pgs->set_transfer.blue) {
         component_num = pgs->set_transfer.blue_component_num;
-        if (component_num >= 0)
+        if (component_num >= 0) {
+            if (pgs->effective_transfer[component_num]->proc != &gs_identity_transfer)
+               non_id_count--;
             pgs->effective_transfer[component_num] = pgs->set_transfer.blue;
+            if (pgs->effective_transfer[component_num]->proc != &gs_identity_transfer)
+               non_id_count++;
+        }
     }
 
-    if (pdht == NULL)
-        return;                 /* not initialized yet */
+    if (pdht) { /* might not be initialized yet */
 
-    /* Since the transfer function is pickled into the threshold array (if any)*/
-    /*  we need to free it so it can be reconstructed with the current transfer */
-    porder = &(pdht->order);
-    if (porder->threshold != NULL) {
-        gs_free_object(porder->data_memory->non_gc_memory, porder->threshold,
-                       "set_effective_transfer(threshold)");
-        porder->threshold = 0;
-    }
-    for (i = 0; i < pdht->num_comp; i++) {
-        pmap = pdht->components[i].corder.transfer;
-        if (pmap != NULL)
-            pgs->effective_transfer[i] = pmap;
-        porder = &(pdht->components[i].corder);
+        /* Since the transfer function is pickled into the threshold array (if any)*/
+        /*  we need to free it so it can be reconstructed with the current transfer */
+        porder = &(pdht->order);
         if (porder->threshold != NULL) {
             gs_free_object(porder->data_memory->non_gc_memory, porder->threshold,
                            "set_effective_transfer(threshold)");
             porder->threshold = 0;
         }
+        for (i = 0; i < pdht->num_comp; i++) {
+            pmap = pdht->components[i].corder.transfer;
+            if (pmap != NULL) {
+                if (pgs->effective_transfer[i]->proc != &gs_identity_transfer)
+                    non_id_count--;
+                pgs->effective_transfer[i] = pmap;
+                if (pgs->effective_transfer[i]->proc != &gs_identity_transfer)
+                   non_id_count++;
+            }
+            porder = &(pdht->components[i].corder);
+            if (porder->threshold != NULL) {
+                gs_free_object(porder->data_memory->non_gc_memory, porder->threshold,
+                               "set_effective_transfer(threshold)");
+                porder->threshold = 0;
+            }
+        }
     }
 
+    pgs->effective_transfer_non_identity_count = non_id_count;
 }
 
 void
@@ -1362,7 +1383,6 @@ gx_ht_construct_threshold( gx_ht_order *d_order, gx_device *dev,
 
     /* search upwards to find the correct value for the last threshold value */
     /* Use this to initialize the threshold array (transition to all white) */
-    shade = 0;
     t_level = 0;
     do {
         t_level++;
