@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -219,7 +219,7 @@ gx_cspace_is_linear_ICC(const gs_color_space *cs, const gs_gstate * pgs,
     if (gx_device_must_halftone(dev)) return 0;
     if (icclink->is_identity) return 1; /* Transform is identity, linear! */
 
-    if (dev->color_info.separable_and_linear != GX_CINFO_SEP_LIN)
+    if (!colors_are_separable_and_linear(&dev->color_info))
         return_error(gs_error_rangecheck);
     if (c2 == NULL)
         return gx_icc_is_linear_in_line(cs, pgs, dev, c0, c1, smoothness, icclink);
@@ -299,16 +299,15 @@ gx_restrict_ICC(gs_client_color * pcc, const gs_color_space * pcs)
 }
 
 static int
-gx_remap_concrete_icc_devicen(const frac * pconc, const gs_color_space * pcs,
-        gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-                          gs_color_select_t select)
+gx_remap_concrete_icc_devicen(const gs_color_space * pcs, const frac * pconc,
+                              gx_device_color * pdc, const gs_gstate * pgs,
+                              gx_device * dev, gs_color_select_t select,
+                              const cmm_dev_profile_t *dev_profile)
 {
     /* Check if this is a device with a DeviceN ICC profile.  In this case,
        we need to do some special stuff */
-    cmm_dev_profile_t *dev_profile;
-    int code;
+    int code = 0;
 
-    code = dev_proc(dev, get_profile)(dev, &dev_profile);
     if (dev_profile->spotnames != NULL  &&
         !dev_profile->spotnames->equiv_cmyk_set) {
         /* This means that someone has specified a DeviceN (Ncolor)
@@ -329,35 +328,25 @@ gx_remap_concrete_icc_devicen(const frac * pconc, const gs_color_space * pcs,
    do any halftoning.  The remap is based upon the ICC profile defined
    in the device profile entry of the profile manager. */
 int
-gx_remap_concrete_ICC(const frac * pconc, const gs_color_space * pcs,
-        gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-                          gs_color_select_t select)
+gx_remap_concrete_ICC(const gs_color_space * pcs, const frac * pconc,
+                      gx_device_color * pdc, const gs_gstate * pgs,
+                      gx_device * dev, gs_color_select_t select,
+                      const cmm_dev_profile_t *dev_profile)
 {
-    int num_colorants;
-    int code;
-    cmm_dev_profile_t *dev_profile;
-
-    code = dev_proc(dev, get_profile)(dev, &dev_profile);
-    num_colorants = gsicc_get_device_profile_comps(dev_profile);
-    switch( num_colorants ) {
+    switch (gsicc_get_device_profile_comps(dev_profile)) {
         case 1:
-            code = gx_remap_concrete_DGray(pconc, pcs, pdc, pgs, dev, select);
-            break;
+            return gx_remap_concrete_DGray(pcs, pconc, pdc, pgs, dev, select, dev_profile);
         case 3:
-            code = gx_remap_concrete_DRGB(pconc, pcs, pdc, pgs, dev, select);
-            break;
+            return gx_remap_concrete_DRGB(pcs, pconc, pdc, pgs, dev, select, dev_profile);
         case 4:
-            code = gx_remap_concrete_DCMYK(pconc, pcs, pdc, pgs, dev, select);
-            break;
+            return gx_remap_concrete_DCMYK(pcs, pconc, pdc, pgs, dev, select, dev_profile);
         default:
             /* This is a special case where we have a source color and our
                output profile must be DeviceN.   We will need to map our
                colorants to the proper planes */
-            code = gx_remap_concrete_icc_devicen(pconc, pcs, pdc, pgs, dev, select);
-            break;
-        }
-    return code;
+            return gx_remap_concrete_icc_devicen(pcs, pconc, pdc, pgs, dev, select, dev_profile);
     }
+}
 
 /*
  * To device space
@@ -453,7 +442,7 @@ gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
     for ( k = 0; k < num_des_comps; k++){
         conc[k] = ushort2frac(psrc_temp[k]);
     }
-    gx_remap_concrete_ICC(conc, pcs, pdc, pgs, dev, select);
+    gx_remap_concrete_ICC(pcs, conc, pdc, pgs, dev, select, dev_profile);
 
     /* Save original color space and color info into dev color */
     i = pcs->cmm_icc_profile_data->num_comps;
@@ -525,7 +514,7 @@ gx_remap_ICC_imagelab(const gs_client_color * pcc, const gs_color_space * pcs,
     for ( k = 0; k < num_des_comps; k++){
         conc[k] = ushort2frac(psrc_temp[k]);
     }
-    gx_remap_concrete_ICC(conc, pcs, pdc, pgs, dev, select);
+    gx_remap_concrete_ICC(pcs, conc, pdc, pgs, dev, select, dev_profile);
 
     /* Save original color space and color info into dev color */
     i = pcs->cmm_icc_profile_data->num_comps;
