@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -202,7 +202,7 @@ is_image_visible(const gs_image_common_t * pic, gs_gstate * pgs, gx_clip_path *p
 /* Create an image enumerator given image parameters and a graphics state. */
 int
 gs_image_begin_typed(const gs_image_common_t * pic, gs_gstate * pgs,
-                     bool uses_color, gx_image_enum_common_t ** ppie)
+                     bool uses_color, bool image_is_text, gx_image_enum_common_t ** ppie)
 {
     gx_device *dev = gs_currentdevice(pgs);
     gx_clip_path *pcpath;
@@ -212,8 +212,8 @@ gs_image_begin_typed(const gs_image_common_t * pic, gs_gstate * pgs,
 
     if (code < 0)
         return code;
-    /* Processing an image object operation */
-    dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_IMAGE_TAG);
+    /* Processing an image object operation, but this may be for a text object */
+    ensure_tag_is_set(pgs, pgs->device, image_is_text ? GS_TEXT_TAG : GS_IMAGE_TAG);	/* NB: may unset_dev_color */
 
     if (uses_color) {
         code = gx_set_dev_color(pgs);
@@ -240,7 +240,7 @@ gs_image_begin_typed(const gs_image_common_t * pic, gs_gstate * pgs,
             if ((any_abs(mat.xy) > any_abs(mat.xx)) && (any_abs(mat.yx) > any_abs(mat.yy)))
                 transpose = true;		/* pure landscape */
             code = gx_image_fill_masked_start(dev, gs_currentdevicecolor_inline(pgs), transpose,
-                                              pcpath, pgs->memory, &dev2);
+                                              pcpath, pgs->memory, pgs->log_op, &dev2);
             if (code < 0)
                 return code;
         }
@@ -287,7 +287,7 @@ gs_image_enum_alloc(gs_memory_t * mem, client_name_t cname)
 /* Start processing an ImageType 1 image. */
 int
 gs_image_init(gs_image_enum * penum, const gs_image_t * pim, bool multi,
-              gs_gstate * pgs)
+              bool image_is_text, gs_gstate * pgs)
 {
     gs_image_t image;
     gx_image_enum_common_t *pie;
@@ -313,7 +313,7 @@ gs_image_init(gs_image_enum * penum, const gs_image_t * pim, bool multi,
     }
     code = gs_image_begin_typed((const gs_image_common_t *)&image, pgs,
                                 image.ImageMask | image.CombineWithColor,
-                                &pie);
+                                image_is_text, &pie);
     if (code < 0)
         return code;
     return gs_image_enum_init(penum, pie, (const gs_data_image_t *)&image,
@@ -588,8 +588,12 @@ gs_image_next_planes(gs_image_enum * penum,
                 penum->image_planes[i].data = penum->planes[i].row.data;
             } else if (pos == 0 && size >= raster) {
                 /* We can transfer 1 or more planes from the source. */
-                h = min(h, size / raster);
-                penum->image_planes[i].data = penum->planes[i].source.data;
+                if (raster) {
+                    h = min(h, size / raster);
+                    penum->image_planes[i].data = penum->planes[i].source.data;
+                }
+                else
+                    h = 0;
             } else
                 h = 0;		/* not enough data in this plane */
         }
