@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2007 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Test program for Ghostscript library */
 
 #include "stdio_.h"
@@ -45,26 +47,27 @@
 #include "gxht.h"		/* for gs_halftone */
 #include "gdevbbox.h"
 #include "gshtx.h"
+#include "gxiodev.h"
 
 /* Define whether we are processing captured data. */
 /*#define CAPTURE */
 
 /* Test programs */
-static int test1(gs_state *, gs_memory_t *);	/* kaleidoscope */
-static int test2(gs_state *, gs_memory_t *);	/* pattern fill */
-static int test3(gs_state *, gs_memory_t *);	/* RasterOp */
-static int test4(gs_state *, gs_memory_t *);	/* set resolution */
-static int test5(gs_state *, gs_memory_t *);	/* images */
-static int test6(gs_state *, gs_memory_t *);	/* CIE API, snapping */
-static int test7(gs_state *, gs_memory_t *);	/* non-monot HT */
-static int test8(gs_state *, gs_memory_t *);	/* transp patterns */
+static int test1(gs_gstate *, gs_memory_t *);	/* kaleidoscope */
+static int test2(gs_gstate *, gs_memory_t *);	/* pattern fill */
+static int test3(gs_gstate *, gs_memory_t *);	/* RasterOp */
+static int test4(gs_gstate *, gs_memory_t *);	/* set resolution */
+static int test5(gs_gstate *, gs_memory_t *);	/* images */
+static int test6(gs_gstate *, gs_memory_t *);	/* CIE API, snapping */
+static int test7(gs_gstate *, gs_memory_t *);	/* non-monot HT */
+static int test8(gs_gstate *, gs_memory_t *);	/* transp patterns */
 
 #ifdef CAPTURE
 #include "k/capture.c"
-static int test10(gs_state *, gs_memory_t *);	/* captured data */
+static int test10(gs_gstate *, gs_memory_t *);	/* captured data */
 
 #endif
-static int (*tests[]) (gs_state *, gs_memory_t *) =
+static int (*tests[]) (gs_gstate *, gs_memory_t *) =
 {
     test1, test2, test3, test4, test5,
         test6, test7, test8, 0
@@ -74,11 +77,10 @@ static int (*tests[]) (gs_state *, gs_memory_t *) =
 };
 
 /* Include the extern for the device stuff. */
-extern init_proc(gs_iodev_init);
 extern_gs_lib_device_list();
 
 /* Forward references */
-static float odsf(floatp, floatp);
+static float odsf(double, double);
 static void gs_abort(const gs_memory_t *);
 
 int
@@ -87,7 +89,7 @@ main(int argc, const char *argv[])
     char achar = '0';
     gs_memory_t *mem;
 
-    gs_state *pgs;
+    gs_gstate *pgs;
     const gx_device *const *list;
     gx_device *dev;
     gx_device_bbox *bbdev;
@@ -141,9 +143,9 @@ main(int argc, const char *argv[])
             lprintf1("reading Name failed! code = %d\n", code);
             gs_abort(mem);
         }
-        dputs("Device name = ");
-        debug_print_string(nstr.data, nstr.size);
-        dputs("\n");
+        dmputs(dev->memory, "Device name = ");
+        debug_print_string(dev->memory, nstr.data, nstr.size);
+        dmputs(dev->memory, "\n");
         gs_c_param_list_release(&list);
     }
     /*
@@ -170,7 +172,7 @@ main(int argc, const char *argv[])
         }
     }
     dev = (gx_device *) bbdev;
-    pgs = gs_state_alloc(mem);
+    pgs = gs_gstate_alloc(mem);
     gs_setdevice_no_erase(pgs, dev);	/* can't erase yet */
     {
         gs_point dpi;
@@ -194,12 +196,12 @@ main(int argc, const char *argv[])
         gs_rect bbox;
 
         gx_device_bbox_bbox(bbdev, &bbox);
-        dprintf4("Bounding box: [%g %g %g %g]\n",
+        dmprintf4(mem, "Bounding box: [%g %g %g %g]\n",
                  bbox.p.x, bbox.p.y, bbox.q.x, bbox.q.y);
     }
     if (code)
-        dprintf1("**** Test returned code = %d.\n", code);
-    dputs("Done.  Press <enter> to exit.");
+        dmprintf1(mem, "**** Test returned code = %d.\n", code);
+    dmputs(mem, "Done.  Press <enter> to exit.");
     fgetc(mem->gs_lib_ctx->fstdin);
     gs_lib_finit(0, 0, mem);
     return 0;
@@ -207,7 +209,7 @@ main(int argc, const char *argv[])
 }
 /* Ordered dither spot function */
 static float
-odsf(floatp x, floatp y)
+odsf(double x, double y)
 {
     static const byte dither[256] =
     {
@@ -236,7 +238,7 @@ odsf(floatp x, floatp y)
 
 /* Fill a rectangle. */
 static int
-fill_rect1(gs_state * pgs, floatp x, floatp y, floatp w, floatp h)
+fill_rect1(gs_gstate * pgs, double x, double y, double w, double h)
 {
     gs_rect r;
 
@@ -265,6 +267,8 @@ void
 gs_reloc_const_string(gs_const_string * sptr, gc_state_t * gcst)
 {
 }
+const gs_ptr_procs_t ptr_name_index_procs =
+{NULL, NULL, NULL};
 
 /* Other stubs */
 static void
@@ -279,7 +283,7 @@ gs_abort(const gs_memory_t *mem)
 /* Return the number with the magnitude of x and the sign of y. */
 /* This is a BSD addition to libm; not all compilers have it. */
 static double
-gs_copysign(floatp x, floatp y)
+gs_copysign(double x, double y)
 {
    return ( y >= 0  ? fabs(x) : -fabs(x) );
 }
@@ -307,7 +311,7 @@ rand(void)
     return rand_state;
 }
 static int
-test1(gs_state * pgs, gs_memory_t * mem)
+test1(gs_gstate * pgs, gs_memory_t * mem)
 {
     int n;
 
@@ -341,7 +345,7 @@ test1(gs_state * pgs, gs_memory_t * mem)
 /* Fill an area with a pattern. */
 
 static int
-test2(gs_state * pgs, gs_memory_t * mem)
+test2(gs_gstate * pgs, gs_memory_t * mem)
 {
     gs_client_color cc;
     gx_tile_bitmap tile;
@@ -402,7 +406,7 @@ test2(gs_state * pgs, gs_memory_t * mem)
 /* Currently, this only works with monobit devices. */
 
 static int
-test3(gs_state * pgs, gs_memory_t * mem)
+test3(gs_gstate * pgs, gs_memory_t * mem)
 {
     gx_device *dev = gs_currentdevice(pgs);
     gx_color_index black = gx_device_black(dev);
@@ -444,7 +448,7 @@ test3(gs_state * pgs, gs_memory_t * mem)
 /* Set the resolution dynamically. */
 
 static int
-test4(gs_state * pgs, gs_memory_t * mem)
+test4(gs_gstate * pgs, gs_memory_t * mem)
 {
     gs_c_param_list list;
     float resv[2];
@@ -491,7 +495,7 @@ test4(gs_state * pgs, gs_memory_t * mem)
 /* Test masked (and non-masked) images. */
 
 static int
-test5(gs_state * pgs, gs_memory_t * mem)
+test5(gs_gstate * pgs, gs_memory_t * mem)
 {
     gx_device *dev = gs_currentdevice(pgs);
     gx_image_enum_common_t *info;
@@ -537,7 +541,7 @@ test5(gs_state * pgs, gs_memory_t * mem)
 
 #define do_image(image, idata)\
   BEGIN\
-  code = gx_device_begin_typed_image(dev, (gs_imager_state *)pgs, NULL,\
+  code = gx_device_begin_typed_image(dev, pgs, NULL,\
      (gs_image_common_t *)&image, NULL, &dcolor, NULL, mem, &info);\
   /****** TEST code >= 0 ******/\
   planes[0].data = idata;\
@@ -571,7 +575,7 @@ test5(gs_state * pgs, gs_memory_t * mem)
         image1.BitsPerComponent = 8;
 
         gs_translate(pgs, 0.5, 4.0);
-        code = gx_device_begin_image(dev, (gs_imager_state *) pgs,
+        code = gx_device_begin_image(dev, pgs,
                                      &image1, gs_image_format_chunky,
                                      NULL, &dcolor, NULL, mem, &info1);
 /****** TEST code >= 0 ******/
@@ -628,7 +632,7 @@ test5(gs_state * pgs, gs_memory_t * mem)
 
         /* Display with 1-for-1 mask and image. */
         gs_translate(pgs, 0.5, 2.0);
-        code = gx_device_begin_typed_image(dev, (gs_imager_state *) pgs,
+        code = gx_device_begin_typed_image(dev, pgs,
                                        NULL, (gs_image_common_t *) & image3,
                                            NULL, &dcolor, NULL, mem, &info);
 /****** TEST code >= 0 ******/
@@ -651,7 +655,7 @@ test5(gs_state * pgs, gs_memory_t * mem)
         image3.MaskDict.Width *= 2;
         image3.MaskDict.Height *= 2;
         gs_translate(pgs, 1.5, 0.0);
-        code = gx_device_begin_typed_image(dev, (gs_imager_state *) pgs,
+        code = gx_device_begin_typed_image(dev, pgs,
                                        NULL, (gs_image_common_t *) & image3,
                                            NULL, &dcolor, NULL, mem, &info);
 /****** TEST code >= 0 ******/
@@ -718,7 +722,7 @@ test5(gs_state * pgs, gs_memory_t * mem)
 /* Test the C API for CIE CRDs, and color snapping. */
 
 static void
-spectrum(gs_state * pgs, int n)
+spectrum(gs_gstate * pgs, int n)
 {
     float den = n;
     float den1 = n - 1;
@@ -741,12 +745,12 @@ spectrum(gs_state * pgs, int n)
             }
 }
 static float
-render_abc(floatp v, const gs_cie_render * ignore_crd)
+render_abc(double v, const gs_cie_render * ignore_crd)
 {
     return v / 2;
 }
 static int
-test6(gs_state * pgs, gs_memory_t * mem)
+test6(gs_gstate * pgs, gs_memory_t * mem)
 {
     gs_color_space *pcs;
     gs_cie_abc *pabc;
@@ -793,7 +797,7 @@ test6(gs_state * pgs, gs_memory_t * mem)
 /* Test the C API for non-monotonic halftones. */
 
 static int
-test7(gs_state * pgs, gs_memory_t * mem)
+test7(gs_gstate * pgs, gs_memory_t * mem)
 {
     /* Define a non-monotonic 4 x 4 halftone with 4 gray levels. */
     static const byte masks[1 * 4 * 4] =
@@ -813,12 +817,12 @@ test7(gs_state * pgs, gs_memory_t * mem)
 
     /* Fabricate a Type 5 halftone. */
     code = gs_ht_build(&pht, 1, mem);
-    dprintf1("ht build code = %d\n", code);
+    dmprintf1(mem, "ht build code = %d\n", code);
     code = gs_ht_set_mask_comp(pht, 0,
                                4, 4, 4, masks, NULL, NULL);
-    dprintf1("set mask code = %d\n", code);
+    dmprintf1(mem, "set mask code = %d\n", code);
     code = gs_sethalftone(pgs, pht);
-    dprintf1("sethalftone code = %d\n", code);
+    dmprintf1(mem, "sethalftone code = %d\n", code);
     for (i = 0; i <= 4; ++i) {
         gs_setgray(pgs, i / 4.0);
         fill_rect1(pgs, 100 + i * 100, 100, 50, 50);
@@ -830,7 +834,7 @@ test7(gs_state * pgs, gs_memory_t * mem)
 /* Test partially transparent patterns */
 
 static int
-test8(gs_state * pgs, gs_memory_t * mem)
+test8(gs_gstate * pgs, gs_memory_t * mem)
 {
     /*
      * Define a 16 x 16 pattern using a 4-entry palette
@@ -914,7 +918,7 @@ static const float xmove_origin = 0.0;
 static const float ymove_origin = 0.0;
 
 static int
-test10(gs_state * pgs, gs_memory_t * mem)
+test10(gs_gstate * pgs, gs_memory_t * mem)
 {
     gs_c_param_list list;
     gs_param_string nstr, OFstr;
