@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -76,20 +76,18 @@ gs_fillpage(gs_gstate * pgs)
 {
     gx_device *dev = gs_currentdevice(pgs);
     int code;
-    gx_cm_color_map_procs *   pprocs;
 
-    pprocs = get_color_mapping_procs_subclass(dev);
-    /* If we get here without a valid get_color_mapping_procs, fail */
-    if (pprocs == NULL ||
-        /* Deliberately use the terminal device here */
-        dev_proc(dev, get_color_mapping_procs) ==  gx_error_get_color_mapping_procs) {
+    /* Deliberately use the terminal device here */
+    if (dev_proc(dev, get_color_mapping_procs) ==  gx_error_get_color_mapping_procs) {
         emprintf1(dev->memory,
                   "\n   *** Error: No get_color_mapping_procs for device: %s\n",
                   dev->dname);
         return_error(gs_error_Fatal);
     }
-    /* Processing a fill object operation */
-    dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_PATH_TAG);
+    /* Processing a fill object operation, but this counts as "UNTOUCHED" */
+    gx_unset_dev_color(pgs);		/* force update so we pick up the new tag */
+    gx_unset_alt_dev_color(pgs);
+    dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_UNTOUCHED_TAG);
 
     code = gx_set_dev_color(pgs);
     if (code != 0)
@@ -287,12 +285,11 @@ static int do_fill(gs_gstate *pgs, int rule)
        handle that, we'll have to add a flag to the path structure, or to the path
        segment structure (depending on how fine grained we require it to be).
      */
-    if (pgs->show_gstate == NULL) {
-        dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_PATH_TAG);
-    }
-    else {
-        dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_TEXT_TAG);
-    }
+    if (pgs->show_gstate == NULL)
+        ensure_tag_is_set(pgs, pgs->device, GS_PATH_TAG);	/* NB: may unset_dev_color */
+    else
+        ensure_tag_is_set(pgs, pgs->device, GS_TEXT_TAG);	/* NB: may unset_dev_color */
+
     code = gx_set_dev_color(pgs);
     if (code != 0)
         return code;
@@ -392,12 +389,11 @@ do_stroke(gs_gstate * pgs)
        handle that, we'll have to add a flag to the path structure, or to the path
        segment structure (depending on how fine grained we require it to be).
      */
-    if (pgs->show_gstate == NULL) {
-        dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_PATH_TAG);
-    }
-    else {
-        dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_TEXT_TAG);
-    }
+    if (pgs->show_gstate == NULL)
+        ensure_tag_is_set(pgs, pgs->device, GS_PATH_TAG);	/* NB: may unset_dev_color */
+    else
+        ensure_tag_is_set(pgs, pgs->device, GS_TEXT_TAG);	/* NB: may unset_dev_color */
+
     code = gx_set_dev_color(pgs);
     if (code != 0)
         return code;
