@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -191,6 +191,7 @@ gs_trans_group_params_init(gs_transparency_group_params_t *ptgp)
     ptgp->ColorSpace = NULL;    /* bogus, but can't do better */
     ptgp->Isolated = false;
     ptgp->Knockout = false;
+    ptgp->text_group = PDF14_TEXTGROUP_NO_BT;
     ptgp->image_with_SMask = false;
     ptgp->mask_id = 0;
     ptgp->iccprofile = NULL;
@@ -231,6 +232,7 @@ gs_begin_transparency_group(gs_gstate *pgs,
     params.opacity = pgs->opacity;
     params.shape = pgs->shape;
     params.blend_mode = pgs->blend_mode;
+    params.text_group = ptgp->text_group;
     /* This function is called during the c-list writer side.
        Store some information so that we know what the color space is
        so that we can adjust according later during the clist reader.
@@ -246,6 +248,8 @@ gs_begin_transparency_group(gs_gstate *pgs,
         blend_color_space = gs_currentcolorspace_inline(pgs);
     } else {
         blend_color_space = cs_concrete_space(blend_color_space, pgs);
+        if (!blend_color_space)
+            return_error(gs_error_undefined);
     }
     /* Note that if the /CS parameter was NOT present in the push
        of the transparency group, then we must actually inherent
@@ -253,7 +257,7 @@ gs_begin_transparency_group(gs_gstate *pgs,
        target device (process color model).  Here we just want
        to set it as a unknown type for clist writing, as we will take care
        of using the parent group color space later during clist reading.
-       Also, if the group was not isolated we MUST use the parent group 
+       Also, if the group was not isolated we MUST use the parent group
        color space regardless of what the group color space is specified to be
        */
     if (ptgp->ColorSpace == NULL || params.Isolated != true) {
@@ -320,8 +324,8 @@ gs_begin_transparency_group(gs_gstate *pgs,
         else
             dmputs(pgs->memory, "     (no CS)");
 
-        dmprintf2(pgs->memory, "  Isolated = %d  Knockout = %d\n",
-                 ptgp->Isolated, ptgp->Knockout);
+        dmprintf3(pgs->memory, "  Isolated = %d  Knockout = %d text_group = %d\n",
+                 ptgp->Isolated, ptgp->Knockout, ptgp->text_group);
     }
 #endif
     params.bbox = *pbbox;
@@ -342,6 +346,7 @@ gx_begin_transparency_group(gs_gstate * pgs, gx_device * pdev,
     tgp.Knockout = pparams->Knockout;
     tgp.idle = pparams->idle;
     tgp.mask_id = pparams->mask_id;
+    tgp.text_group = pparams->text_group;
 
     /* Needed so that we do proper blending */
     tgp.group_color = pparams->group_color;
@@ -372,11 +377,8 @@ gx_begin_transparency_group(gs_gstate * pgs, gx_device * pdev,
             dmprintf(pdev->memory, "     Have ICC Profile for blending\n");
     }
 #endif
-    if (dev_proc(pdev, begin_transparency_group) != 0)
-        return (*dev_proc(pdev, begin_transparency_group)) (pdev, &tgp, &bbox, pgs,
+    return (*dev_proc(pdev, begin_transparency_group)) (pdev, &tgp, &bbox, pgs,
                                                             NULL);
-    else
-        return 0;
 }
 
 int
@@ -393,14 +395,39 @@ gs_end_transparency_group(gs_gstate *pgs)
     return gs_gstate_update_pdf14trans(pgs, &params);
 }
 
+int
+gs_end_transparency_text_group(gs_gstate *pgs)
+{
+    gs_pdf14trans_params_t params = { 0 };
+
+    if (check_for_nontrans_pattern(pgs,
+        (unsigned char *)"gs_end_transparency_text_group")) {
+        return(0);
+    }
+    if_debug0m('v', pgs->memory, "[v]gs_end_transparency_text_group\n");
+    params.pdf14_op = PDF14_END_TRANS_TEXT_GROUP;  /* Other parameters not used */
+    return gs_gstate_update_pdf14trans(pgs, &params);
+}
+
+int
+gs_begin_transparency_text_group(gs_gstate *pgs)
+{
+    gs_pdf14trans_params_t params = { 0 };
+
+    if (check_for_nontrans_pattern(pgs,
+        (unsigned char *)"gs_begin_transparency_text_group")) {
+        return(0);
+    }
+    if_debug0m('v', pgs->memory, "[v]gs_begin_transparency_text_group\n");
+    params.pdf14_op = PDF14_BEGIN_TRANS_TEXT_GROUP;  /* Other parameters not used */
+    return gs_gstate_update_pdf14trans(pgs, &params);
+}
+
 int
 gx_end_transparency_group(gs_gstate * pgs, gx_device * pdev)
 {
     if_debug0m('v', pgs->memory, "[v]gx_end_transparency_group\n");
-    if (dev_proc(pdev, end_transparency_group) != 0)
-        return (*dev_proc(pdev, end_transparency_group)) (pdev, pgs);
-    else
-        return 0;
+    return (*dev_proc(pdev, end_transparency_group)) (pdev, pgs);
 }
 
 /* Commands for handling q softmask Q in graphic states */
@@ -474,20 +501,14 @@ int
 gx_pop_transparency_state(gs_gstate * pgs, gx_device * pdev)
 {
     if_debug0m('v', pgs->memory, "[v]gx_pop_transparency_state\n");
-    if (dev_proc(pdev, pop_transparency_state) != 0)
-        return (*dev_proc(pdev, pop_transparency_state)) (pdev, pgs);
-    else
-        return 0;
+    return (*dev_proc(pdev, pop_transparency_state)) (pdev, pgs);
 }
 
 int
 gx_push_transparency_state(gs_gstate * pgs, gx_device * pdev)
 {
     if_debug0m('v', pgs->memory, "[v]gx_push_transparency_state\n");
-    if (dev_proc(pdev, push_transparency_state) != 0)
-        return (*dev_proc(pdev, push_transparency_state)) (pdev, pgs);
-    else
-        return 0;
+    return (*dev_proc(pdev, push_transparency_state)) (pdev, pgs);
 }
 
 /*
@@ -568,6 +589,8 @@ gs_begin_transparency_mask(gs_gstate * pgs,
         if (code < 0)
             return(code);
         blend_color_space = gs_cspace_new_DeviceGray(pgs->memory);
+        if (blend_color_space == NULL)
+            return_error(gs_error_VMerror);
         blend_color_space->cmm_icc_profile_data = pgs->icc_manager->default_gray;
         rc_increment(blend_color_space->cmm_icc_profile_data);
         if_debug9m('v', pgs->memory, "[v](0x%lx)gs_begin_transparency_mask [%g %g %g %g]\n\
@@ -652,11 +675,8 @@ gx_begin_transparency_mask(gs_gstate * pgs, gx_device * pdev,
               tmp.group_color_numcomps,
               (tmp.function_is_identity ? "no TR" :
                "has TR"));
-    if (dev_proc(pdev, begin_transparency_mask) != 0)
-        return (*dev_proc(pdev, begin_transparency_mask))
+    return (*dev_proc(pdev, begin_transparency_mask))
                         (pdev, &tmp, &(pparams->bbox), pgs, NULL);
-    else
-        return 0;
 }
 
 int
@@ -704,10 +724,7 @@ gx_end_transparency_mask(gs_gstate * pgs, gx_device * pdev,
     if_debug2m('v', pgs->memory,
                "[v](0x%lx)gx_end_transparency_mask(%d)\n", (ulong)pgs,
                (int)pparams->csel);
-    if (dev_proc(pdev, end_transparency_mask) != 0)
-        return (*dev_proc(pdev, end_transparency_mask)) (pdev, pgs);
-    else
-        return 0;
+    return (*dev_proc(pdev, end_transparency_mask)) (pdev, pgs);
 }
 
 /*
@@ -805,9 +822,38 @@ int
 gx_abort_trans_device(gs_gstate * pgs, gx_device * pdev)
 {
     if_debug1m('v', pgs->memory, "[v](0x%lx)gx_abort_trans_device\n", (ulong)pgs);
-    if (dev_proc(pdev, discard_transparency_layer) != 0)
     return (*dev_proc(pdev, discard_transparency_layer)) (pdev, pgs);
-    else
+}
+
+int gs_setstrokeconstantalpha(gs_gstate *pgs, float alpha)
+{
+    pgs->strokeconstantalpha = alpha;
     return 0;
 }
 
+float gs_getstrokeconstantalpha(const gs_gstate *pgs)
+{
+    return pgs->strokeconstantalpha;
+}
+
+int gs_setfillconstantalpha(gs_gstate *pgs, float alpha)
+{
+    pgs->fillconstantalpha = (float)alpha;
+    return 0;
+}
+
+float gs_getfillconstantalpha(const gs_gstate *pgs)
+{
+    return pgs->fillconstantalpha;
+}
+
+int gs_setalphaisshape(gs_gstate *pgs, bool AIS)
+{
+    pgs->alphaisshape = AIS;
+    return 0;
+}
+
+bool gs_getalphaisshape(gs_gstate *pgs)
+{
+    return pgs->alphaisshape;
+}
