@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -210,6 +210,8 @@ gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
             pfont->data.glyf = offset;
             glyph_size = (uint)u32(tab + 12);
         } else if (!memcmp(tab, "GSUB", 4)) {
+            if (pfont->data.gsub_size != 0)
+                return_error(gs_error_invalidfont);
             pfont->data.gsub_size = u32(tab + 12);
             pfont->data.gsub = gs_alloc_byte_array(pfont->memory, pfont->data.gsub_size, 1,
                                                         "gs_type42_font_init(GSUB)");
@@ -339,8 +341,10 @@ gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
             if (pfont->data.len_glyphs == 0)
                 return_error(gs_error_VMerror);
             code = gs_font_notify_register((gs_font *)pfont, gs_len_glyphs_release, (void *)pfont);
-            if (code < 0)
+            if (code < 0) {
+                gs_free_object(pfont->memory, pfont->data.len_glyphs, "gs_len_glyphs_release");
                 return code;
+            }
             
             /* The 'loca' may not be in order, so we construct a glyph length array */
             /* Since 'loca' is usually sorted, first try the simple linear scan to  */
@@ -788,7 +792,7 @@ gs_type42_substitute_glyph_index_vertical(gs_font_type42 *pfont, uint glyph_inde
     LookupListTable lookup_list_table;
     byte *lookup_list_ptr;
 
-    if (WMode == 0)
+    if (WMode == 0 || gsub_ptr == NULL)
         return glyph_index;
 
     /* GSUB header */
@@ -859,7 +863,6 @@ gs_type42_substitute_glyph_index_vertical(gs_font_type42 *pfont, uint glyph_inde
                                         return new_glyph;
                                     }
                                 } else if (k0 >= k1 - 1) {
-                                    k += 0; /* A place for breakpoint. */
                                     break; /* Not found. */
                                 } else if (glyph_index < glyph)
                                     k1 = k;
@@ -898,7 +901,6 @@ gs_type42_substitute_glyph_index_vertical(gs_font_type42 *pfont, uint glyph_inde
                                         return new_glyph;
                                     }
                                 } else if (k0 >= k1 - 1) {
-                                    k += 0; /* A place for breakpoint. */
                                     break; /* Not found. */
                                 } else if (glyph_index < rr.Start)
                                     k1 = k;
@@ -1203,6 +1205,11 @@ gs_type42_default_get_metrics(gs_font_type42 * pfont, uint glyph_index,
             do {
                 uint comp_index = U16(gdata + 2);
 
+                if (comp_index == glyph_index) {
+                    result = gs_note_error(gs_error_invalidfont);
+                    goto done;
+                }
+
                 gs_type42_parse_component(&gdata, &flags, &mat, NULL, pfont, &mat);
                 if (flags & TT_CG_USE_MY_METRICS) {
                     result = pfont->data.get_metrics(pfont, comp_index, wmode, sbw);
