@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -485,7 +485,10 @@ gx_alloc_char_bits(gs_font_dir * dir, gx_device_memory * dev,
     int log2_yscale = pscale->y;
     int log2_depth = ilog2(depth);
     uint nwidth_bits = (iwidth >> log2_xscale) << log2_depth;
-    ulong isize, icdsize, isize2;
+    ulong isize, icdsize;
+#ifdef ENABLE_IMPOSSIBLE_ALPHA_CODE
+    ulong isize2;
+#endif
     uint iraster;
     cached_char *cc;
     gx_device_memory mdev;
