@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -608,6 +608,9 @@ set_cache_device(gs_show_enum * penum, gs_gstate * pgs, double llx, double lly,
                                  log2_scale.x + log2_scale.y > alpha_bits ?
                                  penum->dev_cache2 : NULL),
                                 iwidth, iheight, &log2_scale, depth, &cc);
+        if (code < 0)
+            return code;
+
         if (cc == 0) {
             /* too big for cache or no cache */
             gx_path box_path;
@@ -633,6 +636,8 @@ set_cache_device(gs_show_enum * penum, gs_gstate * pgs, double llx, double lly,
             if (code < 0)
                 return code;
             code = gx_cpath_clip(pgs, pgs->clip_path, &box_path, gx_rule_winding_number);
+            if (code < 0)
+                return code;
             gx_path_free(&box_path, "set_cache_device");
             pgs->in_cachedevice = CACHE_DEVICE_NONE_AND_CLIP;
             return 0;
@@ -1127,6 +1132,7 @@ show_proceed(gs_show_enum * penum)
             }
         }
     } else {
+        start = penum->index;
         /* Can't use cache */
         switch ((code = get_next_char_glyph((gs_text_enum_t *)penum,
                                             &chr, &glyph))
@@ -1145,6 +1151,7 @@ show_proceed(gs_show_enum * penum)
                     gs_log2_scale_point log2_scale;
                     gs_fixed_point subpix_origin;
 
+                    penum->bytes_decoded = penum->index - start;
                     code = compute_glyph_raster_params(penum, false, &alpha_bits, &depth, &subpix_origin, &log2_scale);
                     if (code < 0)
                         return code;
@@ -1224,9 +1231,8 @@ show_proceed(gs_show_enum * penum)
             cpt.x = float2fixed(fpx);
             cpt.y = float2fixed(fpy);
         }
-        gs_newpath(pgs);
-        code = show_origin_setup(pgs, cpt.x, cpt.y, penum);
-        if (code < 0)
+        if (((code = gs_newpath(pgs)) < 0) ||
+            ((code = show_origin_setup(pgs, cpt.x, cpt.y, penum)) < 0))
             goto rret;
     }
     penum->width_status = sws_none;
