@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -174,16 +174,16 @@ const gx_device_procs gs_clist_device_procs = {
     gx_forward_get_hardware_params,
     gx_default_text_begin,
     gx_default_finish_copydevice,
-    NULL,                       /* begin_transparency_group */
-    NULL,                       /* end_transparency_group */
-    NULL,                       /* begin_transparency_mask */
-    NULL,                       /* end_transparency_mask */
-    NULL,                       /* discard_transparency_layer */
+    gx_default_begin_transparency_group,                       /* begin_transparency_group */
+    gx_default_end_transparency_group,                       /* end_transparency_group */
+    gx_default_begin_transparency_mask,                       /* begin_transparency_mask */
+    gx_default_end_transparency_mask,                       /* end_transparency_mask */
+    gx_default_discard_transparency_layer,                       /* discard_transparency_layer */
     gx_forward_get_color_mapping_procs,
     gx_forward_get_color_comp_index,
     gx_forward_encode_color,
     gx_forward_decode_color,
-    NULL,                       /* pattern_manage */
+    gx_default_pattern_manage,                       /* pattern_manage */
     clist_fill_rectangle_hl_color,
     gx_default_include_color_space,
     gx_default_fill_linear_color_scanline,
@@ -192,9 +192,9 @@ const gx_device_procs gs_clist_device_procs = {
     gx_forward_update_spot_equivalent_colors,
     gx_forward_ret_devn_params,
     clist_fillpage,
-    NULL,                      /* push_transparency_state */
-    NULL,                      /* pop_transparency_state */
-    NULL,                      /* put_image */
+    gx_default_push_transparency_state,                      /* push_transparency_state */
+    gx_default_pop_transparency_state,                      /* pop_transparency_state */
+    gx_default_put_image,                      /* put_image */
     clist_dev_spec_op,
     clist_copy_planes,         /* copy planes */
     gx_default_get_profile,
@@ -322,7 +322,7 @@ clist_init_bands(gx_device * dev, gx_device_memory *bdev, uint data_size,
     int nbands;
     ulong space;
 
-    if (dev->procs.open_device == pattern_clist_open_device) {
+    if (dev_proc(dev, open_device) == pattern_clist_open_device) {
         /* We don't need bands really. */
         cdev->page_band_height = dev->height;
         cdev->nbands = 1;
@@ -416,7 +416,7 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
     /* copy_alpha in the commmand list device as well. */
     if (dev_proc(pbdev, copy_alpha) == gx_no_copy_alpha)
         cdev->disable_mask |= clist_disable_copy_alpha;
-    if (cdev->procs.open_device == pattern_clist_open_device) {
+    if (dev_proc(cdev, open_device) == pattern_clist_open_device) {
         bits_size = data_size / 2;
         cdev->page_line_ptrs_offset = 0;
     } else {
@@ -584,27 +584,6 @@ clist_init(gx_device * dev)
     return code;
 }
 
-/* (Re)init open band files for output (set block size, etc). */
-static int      /* ret 0 ok, -ve error code */
-clist_reinit_output_file(gx_device *dev)
-{    gx_device_clist_writer * const cdev =
-        &((gx_device_clist *)dev)->writer;
-    int code = 0;
-
-    /* bfile needs to guarantee cmd_blocks for: 1 band range, nbands */
-    /*  & terminating entry */
-    int b_block = sizeof(cmd_block) * (cdev->nbands + 2);
-
-    /* cfile needs to guarantee one writer buffer */
-    /*  + one end_clip cmd (if during image's clip path setup) */
-    /*  + an end_image cmd for each band (if during image) */
-    /*  + end_cmds for each band and one band range */
-    int c_block =
-        cdev->cend - cdev->cbuf + 2 + cdev->nbands * 2 + (cdev->nbands + 1);
-
-    return code;
-}
-
 /* Write out the current parameters that must be at the head of each page */
 /* if async rendering is in effect */
 static int
@@ -654,8 +633,7 @@ clist_open_output_file(gx_device *dev)
                             true)) < 0 ||
         (code = cdev->page_info.io_procs->fopen(cdev->page_bfname, fmode, &cdev->page_bfile,
                             cdev->bandlist_memory, cdev->bandlist_memory,
-                            false)) < 0 ||
-        (code = clist_reinit_output_file(dev)) < 0
+                            false)) < 0
         ) {
         clist_close_output_file(dev);
         cdev->permanent_error = code;
@@ -745,7 +723,7 @@ clist_close(gx_device *dev)
 
     if (cdev->do_not_open_or_close_bandfiles)
         return 0;
-    if (cdev->procs.open_device == pattern_clist_open_device) {
+    if (dev_proc(cdev, open_device) == pattern_clist_open_device) {
         gs_free_object(cdev->bandlist_memory, cdev->data, "clist_close");
         cdev->data = NULL;
     }
@@ -802,8 +780,6 @@ clist_finish_page(gx_device *dev, bool flush)
             cdev->page_info.io_procs->fseek(cdev->page_bfile, 0L, SEEK_END, cdev->page_bfname);
     }
     code = clist_init(dev);             /* reinitialize */
-    if (code >= 0)
-        code = clist_reinit_output_file(dev);
     if (code >= 0)
         code = clist_emit_page_header(dev);
 
@@ -821,6 +797,11 @@ clist_end_page(gx_device_clist_writer * cldev)
     int ecode = 0;
 
     code = cmd_write_buffer(cldev, cmd_opv_end_page);
+    if (code >= 0)
+        ecode |= code;
+    else
+        ecode = code;
+
     /* If we have ICC profiles present in the cfile save the table now,
        along with the ICC profiles. Table is stored in band maxband + 1. */
     if ( cldev->icc_table != NULL ) {
@@ -830,25 +811,26 @@ clist_end_page(gx_device_clist_writer * cldev)
         clist_free_icc_table(cldev->icc_table, cldev->memory);
         cldev->icc_table = NULL;
     }
-    if (code >= 0)
-        code = clist_write_color_usage_array(cldev);
     if (code >= 0) {
-        /*
-         * Write the terminating entry in the block file.
-         * Note that because of copypage, there may be many such entries.
-         */
-        memset(&cb, 0, sizeof(cb)); /* Zero the block, including any padding */
-        cb.band_min = cb.band_max = cmd_band_end;
-        cb.pos = (cldev->page_cfile == 0 ? 0 : cldev->page_info.io_procs->ftell(cldev->page_cfile));
-        code = cldev->page_info.io_procs->fwrite_chars(&cb, sizeof(cb), cldev->page_bfile);
-        if (code > 0)
-            code = 0;
+        code = clist_write_color_usage_array(cldev);
+        if (code >= 0) {
+            ecode |= code;
+            /*
+             * Write the terminating entry in the block file.
+             * Note that because of copypage, there may be many such entries.
+             */
+            memset(&cb, 0, sizeof(cb)); /* Zero the block, including any padding */
+            cb.band_min = cb.band_max = cmd_band_end;
+            cb.pos = (cldev->page_cfile == 0 ? 0 : cldev->page_info.io_procs->ftell(cldev->page_cfile));
+            code = cldev->page_info.io_procs->fwrite_chars(&cb, sizeof(cb), cldev->page_bfile);
+            if (code > 0)
+                code = 0;
+        }
     }
     if (code >= 0) {
         ecode |= code;
         cldev->page_bfile_end_pos = cldev->page_info.io_procs->ftell(cldev->page_bfile);
-    }
-    if (code < 0)
+    } else
         ecode = code;
 
     /* Reset warning margin to 0 to release reserve memory if mem files */
@@ -881,7 +863,7 @@ clist_end_page(gx_device_clist_writer * cldev)
         dprintf2("%d bands skipped out of %d\n", skip_count, cldev->nbands);
     }
 
-    return 0;
+    return ecode;
 }
 
 gx_color_usage_bits
@@ -1105,6 +1087,11 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
                                "clist_icc_addentry");
         if (entry == NULL)
             return gs_rethrow(-1, "insufficient memory to allocate entry in icc table");
+#ifdef PACIFY_VALGRIND
+        /* Avoid uninitialised padding upsetting valgrind when it's written
+         * into the clist. */
+        memset(entry, 0, sizeof(*entry));
+#endif
         entry->next = NULL;
         entry->serial_data.hashcode = hashcode;
         entry->serial_data.size = -1;
@@ -1140,6 +1127,11 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
                                                        "clist_icc_addentry");
         if (entry == NULL)
             return gs_rethrow(-1, "insufficient memory to allocate entry in icc table");
+#ifdef PACIFY_VALGRIND
+        /* Avoid uninitialised padding upsetting valgrind when it's written
+         * into the clist. */
+        memset(entry, 0, sizeof(*entry));
+#endif
         entry->next = NULL;
         entry->serial_data.hashcode = hashcode;
         entry->serial_data.size = -1;
@@ -1208,7 +1200,8 @@ clist_writer_push_no_cropping(gx_device_clist_writer *cdev)
 
     if (buf == NULL)
         return_error(gs_error_VMerror);
-    if_debug1m('v', cdev->memory, "[v]push cropping[%d]\n", cdev->cropping_level);
+    if_debug4m('v', cdev->memory, "[v]push cropping[%d], min=%d, max=%d, buf=%0x\n",
+               cdev->cropping_level, cdev->cropping_min, cdev->cropping_max, buf);
     buf->next = cdev->cropping_stack;
     cdev->cropping_stack = buf;
     buf->cropping_min = cdev->cropping_min;
@@ -1244,7 +1237,8 @@ clist_writer_pop_cropping(gx_device_clist_writer *cdev)
     cdev->temp_mask_id = buf->temp_mask_id;
     cdev->cropping_stack = buf->next;
     cdev->cropping_level--;
-    if_debug1m('v', cdev->memory, "[v]pop cropping[%d]\n", cdev->cropping_level);
+    if_debug4m('v', cdev->memory, "[v]pop cropping[%d] min=%d, max=%d, buf=%0x\n",
+               cdev->cropping_level, cdev->cropping_min, cdev->cropping_max, buf);
     gs_free_object(cdev->memory, buf, "clist_writer_transparency_pop");
     return 0;
 }
@@ -1371,7 +1365,8 @@ clist_make_accum_device(gx_device *target, const char *dname, void *base, int sp
         set_dev_proc(cwdev, get_clipping_box, gx_default_get_clipping_box);
         set_dev_proc(cwdev, get_profile, gx_forward_get_profile);
         set_dev_proc(cwdev, set_graphics_type_tag, gx_forward_set_graphics_type_tag);
-        cwdev->graphics_type_tag = target->graphics_type_tag;	/* initialize to same as target */
+        cwdev->graphics_type_tag = target->graphics_type_tag;		/* initialize to same as target */
+        cwdev->interpolate_control = target->interpolate_control;	/* initialize to same as target */
 
         /* to be set by caller: cwdev->finalize = finalize; */
 
