@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -61,7 +61,7 @@ gx_default_encode_color(gx_device * dev, const gx_color_value cv[])
     gx_color_index  color = 0;
 
 #ifdef DEBUG
-    if ( dev->color_info.separable_and_linear != GX_CINFO_SEP_LIN ) {
+    if (!colors_are_separable_and_linear(&dev->color_info)) {
         dmprintf(dev->memory, "gx_default_encode_color() requires separable and linear\n" );
         return gx_no_color_index;
     }
@@ -90,7 +90,7 @@ gx_default_decode_color(gx_device * dev, gx_color_index color, gx_color_value cv
     uint shift, ivalue, nbits, scale;
 
 #ifdef DEBUG
-    if ( dev->color_info.separable_and_linear != GX_CINFO_SEP_LIN ) {
+    if (!colors_are_separable_and_linear(&dev->color_info)) {
         dmprintf(dev->memory, "gx_default_decode_color() requires separable and linear\n" );
         return_error(gs_error_rangecheck);
     }
@@ -676,6 +676,9 @@ gs_color_select_t select)
                proper color management for the CMYK portions IF you are using
                an NCLR output profile. */
             code = dev_proc(dev, get_profile)(dev, &dev_profile);
+            if (code < 0)
+                return false;
+
             /* Check if the profile is DeviceN (NCLR) */
             if (dev_profile->device_profile[0]->data_cs == gsNCHANNEL) {
                 if (dev_profile->spotnames == NULL)
@@ -684,6 +687,8 @@ gs_color_select_t select)
                     /* Note that if the improper NCLR profile is used, then the
                        composite preview will be wrong. */
                     code = gsicc_set_devicen_equiv_colors(dev, pgs, dev_profile->device_profile[0]);
+                    if (code < 0)
+                        return false;
                     dev_profile->spotnames->equiv_cmyk_set = true;
                 }
                 gx_remap_concrete_devicen(conc, pdc, pgs, dev, select);
@@ -723,11 +728,17 @@ gx_default_remap_color(const gs_client_color * pcc, const gs_color_space * pcs,
     const gs_color_space *pconcs;
     int i = pcs->type->num_components(pcs);
     int code = (*pcs->type->concretize_color)(pcc, pcs, conc, pgs, dev);
+    cmm_dev_profile_t *dev_profile;
 
     if (code < 0)
         return code;
     pconcs = cs_concrete_space(pcs, pgs);
-    code = (*pconcs->type->remap_concrete_color)(conc, pconcs, pdc, pgs, dev, select);
+    if (!pconcs)
+        return gs_note_error(gs_error_undefined);
+    code = dev_proc(dev, get_profile)(dev, &dev_profile);
+    if (code < 0)
+        return code;
+    code = (*pconcs->type->remap_concrete_color)(pconcs, conc, pdc, pgs, dev, select, dev_profile);
 
     /* Save original color space and color info into dev color */
     i = any_abs(i);
@@ -750,9 +761,10 @@ gx_concretize_DeviceGray(const gs_client_color * pc, const gs_color_space * pcs,
     return 0;
 }
 int
-gx_remap_concrete_DGray(const frac * pconc, const gs_color_space * pcs,
-        gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-                        gs_color_select_t select)
+gx_remap_concrete_DGray(const gs_color_space * pcs, const frac * pconc,
+                        gx_device_color * pdc, const gs_gstate * pgs,
+                        gx_device * dev, gs_color_select_t select,
+                        const cmm_dev_profile_t *dev_profile)
 {
     if (pgs->alpha == gx_max_color_value)
         (*pgs->cmap_procs->map_gray)
@@ -765,8 +777,8 @@ gx_remap_concrete_DGray(const frac * pconc, const gs_color_space * pcs,
 }
 int
 gx_remap_DeviceGray(const gs_client_color * pc, const gs_color_space * pcs,
-        gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-                    gs_color_select_t select)
+                    gx_device_color * pdc, const gs_gstate * pgs,
+                    gx_device * dev, gs_color_select_t select)
 {
     frac fgray = gx_unit_frac(pc->paint.values[0]);
     int code;
@@ -790,7 +802,7 @@ gx_remap_DeviceGray(const gs_client_color * pc, const gs_color_space * pcs,
         return code;
     }
 
-    /* Save orgxiginal color space and color info into dev color */
+    /* Save original color space and color info into dev color */
     pdc->ccolor.paint.values[0] = pc->paint.values[0];
     pdc->ccolor_valid = true;
     if (pgs->alpha == gx_max_color_value)
@@ -813,9 +825,10 @@ gx_concretize_DeviceRGB(const gs_client_color * pc, const gs_color_space * pcs,
     return 0;
 }
 int
-gx_remap_concrete_DRGB(const frac * pconc, const gs_color_space * pcs,
-        gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-                       gs_color_select_t select)
+gx_remap_concrete_DRGB(const gs_color_space * pcs, const frac * pconc,
+                       gx_device_color * pdc, const gs_gstate * pgs,
+                       gx_device * dev, gs_color_select_t select,
+                       const cmm_dev_profile_t *dev_profile)
 {
     if (pgs->alpha == gx_max_color_value)
         gx_remap_concrete_rgb(pconc[0], pconc[1], pconc[2],
@@ -860,9 +873,10 @@ gx_concretize_DeviceCMYK(const gs_client_color * pc, const gs_color_space * pcs,
     return 0;
 }
 int
-gx_remap_concrete_DCMYK(const frac * pconc, const gs_color_space * pcs,
-        gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-                        gs_color_select_t select)
+gx_remap_concrete_DCMYK(const gs_color_space * pcs, const frac * pconc,
+                        gx_device_color * pdc, const gs_gstate * pgs,
+                        gx_device * dev, gs_color_select_t select,
+                        const cmm_dev_profile_t *dev_profile)
 {
 /****** IGNORE alpha ******/
     gx_remap_concrete_cmyk(pconc[0], pconc[1], pconc[2], pconc[3], pdc,
@@ -897,16 +911,17 @@ cmap_gray_halftoned(frac gray, gx_device_color * pdc,
 {
     uchar i, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
-    gx_cm_color_map_procs *   pprocs;
+    subclass_color_mappings scm;
 
     /* map to the color model */
-    for (i=0; i < ncomps; i++)
-        cm_comps[i] = 0;
-    pprocs = get_color_mapping_procs_subclass(dev);
-    map_gray_subclass(pprocs, dev, gray, cm_comps);
+    scm = get_color_mapping_procs_subclass(dev);
+    map_gray_subclass(scm, gray, cm_comps);
 
     /* apply the transfer function(s); convert to color values */
-    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+    if (pgs->effective_transfer_non_identity_count == 0) {
+        if (dev->color_info.polarity != GX_CINFO_POLARITY_ADDITIVE && dev->color_info.opmode == GX_CINFO_OPMODE_UNKNOWN)
+            check_cmyk_color_model_comps(dev);
+    } else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
         for (i = 0; i < ncomps; i++)
             cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
@@ -914,16 +929,13 @@ cmap_gray_halftoned(frac gray, gx_device_color * pdc,
         if (dev->color_info.opmode == GX_CINFO_OPMODE_UNKNOWN)
             check_cmyk_color_model_comps(dev);
         if (dev->color_info.opmode == GX_CINFO_OPMODE) {  /* CMYK-like color space */
-            uint k = dev->color_info.black_component;
-
-            for (i = 0; i < ncomps; i++) {
-                if (i == k) /* Else ignore transfer, see PLRM3 p. 494 */
-                    cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+            i = dev->color_info.black_component;
+            if (i < ncomps)
+                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
-            }
         } else {
             for (i = 0; i < ncomps; i++)
-                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+                    cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                             (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
         }
     }
@@ -940,46 +952,53 @@ cmap_gray_direct(frac gray, gx_device_color * pdc, const gs_gstate * pgs,
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_index color;
-    gx_cm_color_map_procs *   pprocs;
+    subclass_color_mappings scm;
 
     /* map to the color model */
-    for (i=0; i < ncomps; i++)
-        cm_comps[i] = 0;
-    pprocs = get_color_mapping_procs_subclass(dev);
-    map_gray_subclass(pprocs, dev, gray, cm_comps);
+    scm = get_color_mapping_procs_subclass(dev);
+    map_gray_subclass(scm, gray, cm_comps);
 
     /* apply the transfer function(s); convert to color values */
-    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+    if (pgs->effective_transfer_non_identity_count == 0) {
+        if (dev->color_info.polarity != GX_CINFO_POLARITY_ADDITIVE && dev->color_info.opmode == GX_CINFO_OPMODE_UNKNOWN)
+                check_cmyk_color_model_comps(dev);
         for (i = 0; i < ncomps; i++)
-            cv[i] = frac2cv(gx_map_color_frac(pgs,
-                                cm_comps[i], effective_transfer[i]));
+            cv[i] = frac2cv(cm_comps[i]);
+    } else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+        for (i = 0; i < ncomps; i++) {
+            cm_comps[i] = gx_map_color_frac(pgs,
+                                cm_comps[i], effective_transfer[i]);
+            cv[i] = frac2cv(cm_comps[i]);
+        }
     else {
         if (dev->color_info.opmode == GX_CINFO_OPMODE_UNKNOWN)
             check_cmyk_color_model_comps(dev);
         if (dev->color_info.opmode == GX_CINFO_OPMODE) {  /* CMYK-like color space */
-            uint k = dev->color_info.black_component;
-
-            for (i = 0; i < ncomps; i++) {
-                if (i == k)
-                    cv[i] = frac2cv(frac_1 - gx_map_color_frac(pgs,
-                        (frac)(frac_1 - cm_comps[i]), effective_transfer[i]));
-                else
-                    cv[i] = frac2cv(cm_comps[i]); /* Ignore transfer, see PLRM3 p. 494 */
-            }
-        } else {
+            i = dev->color_info.black_component;
+            if (i < ncomps)
+                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+                        (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
             for (i = 0; i < ncomps; i++)
-                cv[i] = frac2cv(frac_1 - gx_map_color_frac(pgs,
-                            (frac)(frac_1 - cm_comps[i]), effective_transfer[i]));
+                cv[i] = frac2cv(cm_comps[i]);
+        } else {
+            for (i = 0; i < ncomps; i++) {
+                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+                            (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
+                cv[i] = frac2cv(cm_comps[i]);
+            }
         }
     }
     /* encode as a color index */
     color = dev_proc(dev, encode_color)(dev, cv);
 
     /* check if the encoding was successful; we presume failure is rare */
-    if (color != gx_no_color_index)
+    if (color != gx_no_color_index) {
         color_set_pure(pdc, color);
-    else
-        cmap_gray_halftoned(gray, pdc, pgs, dev, select);
+        return;
+    }
+    if (gx_render_device_DeviceN(cm_comps, pdc, dev, pgs->dev_ht,
+                                        &pgs->screen_phase[select]) == 1)
+        gx_color_load_select(pdc, pgs, dev, select);
 }
 
 /* ------ Render RGB color. ------ */
@@ -990,23 +1009,23 @@ cmap_rgb_halftoned(frac r, frac g, frac b, gx_device_color * pdc,
 {
     uchar i, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
-    gx_cm_color_map_procs *pprocs;
+    subclass_color_mappings scm;
 
     /* map to the color model */
-    for (i = 0; i < ncomps; i++)
-        cm_comps[i] = 0;
-    pprocs = get_color_mapping_procs_subclass(dev);
-    map_rgb_subclass(pprocs, dev, pgs, r, g, b, cm_comps);
+    scm = get_color_mapping_procs_subclass(dev);
+    map_rgb_subclass(scm, pgs, r, g, b, cm_comps);
 
     /* apply the transfer function(s); convert to color values */
-    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-        for (i = 0; i < ncomps; i++)
-            cm_comps[i] = gx_map_color_frac(pgs,
+    if (pgs->effective_transfer_non_identity_count != 0) {
+        if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+            for (i = 0; i < ncomps; i++)
+                cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
-    else
-        for (i = 0; i < ncomps; i++)
-            cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+        else
+            for (i = 0; i < ncomps; i++)
+                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
+    }
 
     if (gx_render_device_DeviceN(cm_comps, pdc, dev, pgs->dev_ht,
                                         &pgs->screen_phase[select]) == 1)
@@ -1021,32 +1040,40 @@ cmap_rgb_direct(frac r, frac g, frac b, gx_device_color * pdc,
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_index color;
-    gx_cm_color_map_procs *   pprocs;
+    subclass_color_mappings scm;
 
     /* map to the color model */
-    for (i=0; i < ncomps; i++)
-        cm_comps[i] = 0;
-    pprocs = get_color_mapping_procs_subclass(dev);
-    map_rgb_subclass(pprocs, dev, pgs, r, g, b, cm_comps);
+    scm = get_color_mapping_procs_subclass(dev);
+    map_rgb_subclass(scm, pgs, r, g, b, cm_comps);
 
     /* apply the transfer function(s); convert to color values */
-    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+    if (pgs->effective_transfer_non_identity_count == 0) {
         for (i = 0; i < ncomps; i++)
-            cv[i] = frac2cv(gx_map_color_frac(pgs,
-                                cm_comps[i], effective_transfer[i]));
+            cv[i] = frac2cv(cm_comps[i]);
+    } else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+        for (i = 0; i < ncomps; i++) {
+            cm_comps[i] = gx_map_color_frac(pgs,
+                                cm_comps[i], effective_transfer[i]);
+            cv[i] = frac2cv(cm_comps[i]);
+        }
     else
-        for (i = 0; i < ncomps; i++)
-            cv[i] = frac2cv(frac_1 - gx_map_color_frac(pgs,
-                        (frac)(frac_1 - cm_comps[i]), effective_transfer[i]));
+        for (i = 0; i < ncomps; i++) {
+            cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+                        (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
+            cv[i] = frac2cv(cm_comps[i]);
+        }
 
     /* encode as a color index */
     color = dev_proc(dev, encode_color)(dev, cv);
 
     /* check if the encoding was successful; we presume failure is rare */
-    if (color != gx_no_color_index)
+    if (color != gx_no_color_index) {
         color_set_pure(pdc, color);
-    else
-        cmap_rgb_halftoned(r, g, b, pdc, pgs, dev, select);
+        return;
+    }
+    if (gx_render_device_DeviceN(cm_comps, pdc, dev, pgs->dev_ht,
+                                        &pgs->screen_phase[select]) == 1)
+        gx_color_load_select(pdc, pgs, dev, select);
 }
 
 /* ------ Render CMYK color. ------ */
@@ -1064,20 +1091,19 @@ cmap_cmyk_direct(frac c, frac m, frac y, frac k, gx_device_color * pdc,
     cmm_dev_profile_t *dev_profile;
     gsicc_colorbuffer_t src_space = gsUNDEFINED;
     bool gray_to_k;
-    gx_cm_color_map_procs *   pprocs;
+    subclass_color_mappings scm;
 
     /* map to the color model */
-    for (i=0; i < ncomps; i++)
-        cm_comps[i] = 0;
-    pprocs = get_color_mapping_procs_subclass(dev);
-    map_cmyk_subclass(pprocs, dev, c, m, y, k, cm_comps);
+    scm = get_color_mapping_procs_subclass(dev);
+    map_cmyk_subclass(scm, c, m, y, k, cm_comps);
 
     /* apply the transfer function(s); convert to color values */
-    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-        for (i = 0; i < ncomps; i++)
-            cm_comps[i] = gx_map_color_frac(pgs,
+    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE) {
+        if (pgs->effective_transfer_non_identity_count != 0)
+            for (i = 0; i < ncomps; i++)
+                cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
-    else {
+    } else {
         /* Check if source space is gray.  In this case we are to use only the
            transfer function on the K channel.  Do this only if gray to K is
            also set */
@@ -1095,11 +1121,10 @@ cmap_cmyk_direct(frac c, frac m, frac y, frac k, gx_device_color * pdc,
             cm_comps[black_index] = frac_1 - gx_map_color_frac(pgs,
                                     (frac)(frac_1 - cm_comps[black_index]),
                                     effective_transfer[black_index]);
-        } else {
+        } else if (pgs->effective_transfer_non_identity_count != 0)
             for (i = 0; i < ncomps; i++)
                 cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                             (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
-        }
     }
     /* We make a test for direct vs. halftoned, rather than */
     /* duplicating most of the code of this procedure. */
@@ -1139,13 +1164,11 @@ cmap_rgb_alpha_halftoned(frac r, frac g, frac b, frac alpha,
 {
     uchar i, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
-    gx_cm_color_map_procs *   pprocs;
+    subclass_color_mappings scm;
 
     /* map to the color model */
-    for (i=0; i < ncomps; i++)
-        cm_comps[i] = 0;
-    pprocs = get_color_mapping_procs_subclass(dev);
-    map_rgb_subclass(pprocs, dev, pgs, r, g, b, cm_comps);
+    scm = get_color_mapping_procs_subclass(dev);
+    map_rgb_subclass(scm, pgs, r, g, b, cm_comps);
 
     /* pre-multiply to account for the alpha weighting */
     if (alpha != frac_1) {
@@ -1160,14 +1183,16 @@ cmap_rgb_alpha_halftoned(frac r, frac g, frac b, frac alpha,
     }
 
     /* apply the transfer function(s); convert to color values */
-    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-        for (i = 0; i < ncomps; i++)
-            cm_comps[i] = gx_map_color_frac(pgs,
+    if (pgs->effective_transfer_non_identity_count != 0) {
+        if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+            for (i = 0; i < ncomps; i++)
+                cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
-    else
-        for (i = 0; i < ncomps; i++)
-            cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+        else
+            for (i = 0; i < ncomps; i++)
+                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
+    }
 
     if (gx_render_device_DeviceN(cm_comps, pdc, dev, pgs->dev_ht,
                                         &pgs->screen_phase[select]) == 1)
@@ -1182,13 +1207,11 @@ cmap_rgb_alpha_direct(frac r, frac g, frac b, frac alpha, gx_device_color * pdc,
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_value cv_alpha, cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_index color;
-    gx_cm_color_map_procs *   pprocs;
+    subclass_color_mappings scm;
 
     /* map to the color model */
-    for (i=0; i < ncomps; i++)
-        cm_comps[i] = 0;
-    pprocs = get_color_mapping_procs_subclass(dev);
-    map_rgb_subclass(pprocs, dev, pgs, r, g, b, cm_comps);
+    scm = get_color_mapping_procs_subclass(dev);
+    map_rgb_subclass(scm, pgs, r, g, b, cm_comps);
 
     /* pre-multiply to account for the alpha weighting */
     if (alpha != frac_1) {
@@ -1203,7 +1226,10 @@ cmap_rgb_alpha_direct(frac r, frac g, frac b, frac alpha, gx_device_color * pdc,
     }
 
     /* apply the transfer function(s); convert to color values */
-    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+    if (pgs->effective_transfer_non_identity_count == 0)
+        for (i = 0; i < ncomps; i++)
+            cv[i] = frac2cv(cm_comps[i]);
+    else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
         for (i = 0; i < ncomps; i++)
             cv[i] = frac2cv(gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]));
@@ -1220,10 +1246,13 @@ cmap_rgb_alpha_direct(frac r, frac g, frac b, frac alpha, gx_device_color * pdc,
         color = dev_proc(dev, encode_color)(dev, cv);
 
     /* check if the encoding was successful; we presume failure is rare */
-    if (color != gx_no_color_index)
+    if (color != gx_no_color_index) {
         color_set_pure(pdc, color);
-    else
-        cmap_rgb_alpha_halftoned(r, g, b, alpha, pdc, pgs, dev, select);
+        return;
+    }
+    if (gx_render_device_DeviceN(cm_comps, pdc, dev, pgs->dev_ht,
+                                        &pgs->screen_phase[select]) == 1)
+        gx_color_load_select(pdc, pgs, dev, select);
 }
 
 /* ------ Render Separation All color. ------ */
@@ -1270,8 +1299,6 @@ cmap_separation_halftoned(frac all, gx_device_color * pdc,
     frac comp_value = all;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
 
-    for (i=0; i < ncomps; i++)
-        cm_comps[i] = 0;
     if (pgs->color_component_map.sep_type == SEP_ALL) {
         /*
          * Invert the photometric interpretation for additive
@@ -1290,14 +1317,16 @@ cmap_separation_halftoned(frac all, gx_device_color * pdc,
     }
 
     /* apply the transfer function(s); convert to color values */
-    if (additive)
-        for (i = 0; i < ncomps; i++)
-            cm_comps[i] = gx_map_color_frac(pgs,
+    if (pgs->effective_transfer_non_identity_count != 0) {
+        if (additive)
+            for (i = 0; i < ncomps; i++)
+                cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
-    else
-        for (i = 0; i < ncomps; i++)
-            cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+        else
+            for (i = 0; i < ncomps; i++)
+                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
+    }
 
     if (gx_render_device_DeviceN(cm_comps, pdc, dev, pgs->dev_ht,
                                         &pgs->screen_phase[select]) == 1)
@@ -1322,8 +1351,6 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
     dev_proc(dev, get_profile)(dev,  &dev_profile);
     gsicc_extract_profile(dev->graphics_type_tag,
                           dev_profile, &des_profile, &render_cond);
-    for (i=0; i < ncomps; i++)
-        cm_comps[i] = 0;
     if (pgs->color_component_map.sep_type == SEP_ALL) {
         /*
          * Invert the photometric interpretation for additive
@@ -1351,14 +1378,21 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
     }
 
     /* apply the transfer function(s); convert to color values */
-    if (additive)
+    if (pgs->effective_transfer_non_identity_count == 0)
         for (i = 0; i < ncomps; i++)
-            cv[i] = frac2cv(gx_map_color_frac(pgs,
-                                cm_comps[i], effective_transfer[i]));
+            cv[i] = frac2cv(cm_comps[i]);
+    else if (additive)
+        for (i = 0; i < ncomps; i++) {
+            cm_comps[i] = gx_map_color_frac(pgs,
+                                cm_comps[i], effective_transfer[i]);
+            cv[i] = frac2cv(cm_comps[i]);
+        }
     else
-        for (i = 0; i < ncomps; i++)
-            cv[i] = frac2cv(frac_1 - gx_map_color_frac(pgs,
-                        (frac)(frac_1 - cm_comps[i]), effective_transfer[i]));
+        for (i = 0; i < ncomps; i++) {
+            cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+                        (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
+            cv[i] = frac2cv(cm_comps[i]);
+        }
 
     if (use_rgb2dev_icc && pgs->icc_manager->default_rgb != NULL) {
         /* After the transfer function go ahead and do the mapping from RGB to
@@ -1393,17 +1427,21 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
         for (i = 0; i < ncomps; i++)
             pdc->colors.devn.values[i] = cv[i];
         pdc->type = gx_dc_type_devn;
-    } else {
-        /* encode as a color index */
-        color = dev_proc(dev, encode_color)(dev, cv);
-
-        /* check if the encoding was successful; we presume failure is rare */
-        if (color != gx_no_color_index)
-            color_set_pure(pdc, color);
-        else
-            cmap_separation_halftoned(all, pdc, pgs, dev, select);
+        return;
     }
-    return;
+
+    /* encode as a color index */
+    color = dev_proc(dev, encode_color)(dev, cv);
+
+    /* check if the encoding was successful; we presume failure is rare */
+    if (color != gx_no_color_index) {
+        color_set_pure(pdc, color);
+        return;
+    }
+
+    if (gx_render_device_DeviceN(cm_comps, pdc, dev, pgs->dev_ht,
+                                        &pgs->screen_phase[select]) == 1)
+        gx_color_load_select(pdc, pgs, dev, select);
 }
 
 /* Routines for handling CM of CMYK components of a DeviceN color space */
@@ -1412,12 +1450,7 @@ devicen_has_cmyk(gx_device * dev)
 {
     gs_devn_params *devn_params;
 
-    /* Device may not have ret_devn_params! */
-    if (dev->procs.ret_devn_params != NULL) {
-        devn_params = dev_proc(dev, ret_devn_params)(dev);
-    } else {
-        return false;
-    }
+    devn_params = dev_proc(dev, ret_devn_params)(dev);
     if (devn_params == NULL) {
         return false;
     }
@@ -1495,8 +1528,6 @@ cmap_devicen_halftoned(const frac * pcc,
     gsicc_extract_profile(dev->graphics_type_tag,
                           dev_profile, &des_profile, &render_cond);
     /* map to the color model */
-    for (i=0; i < ncomps; i++)
-        cm_comps[i] = 0;
     map_components_to_colorants(pcc, &(pgs->color_component_map), cm_comps);
     /* See comments in cmap_devicen_direct for details on below operations */
     if (devicen_has_cmyk(dev) &&
@@ -1504,14 +1535,16 @@ cmap_devicen_halftoned(const frac * pcc,
         devicen_icc_cmyk(cm_comps, pgs, dev);
     }
     /* apply the transfer function(s); convert to color values */
-    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-        for (i = 0; i < ncomps; i++)
-            cm_comps[i] = gx_map_color_frac(pgs,
+    if (pgs->effective_transfer_non_identity_count != 0) {
+        if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+            for (i = 0; i < ncomps; i++)
+                cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
-    else
-        for (i = 0; i < ncomps; i++)
-            cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+        else
+            for (i = 0; i < ncomps; i++)
+                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
+    }
 
     /* We need to finish halftoning */
     if (gx_render_device_DeviceN(cm_comps, pdc, dev, pgs->dev_ht,
@@ -1541,8 +1574,6 @@ cmap_devicen_direct(const frac * pcc,
                           dev_profile, &des_profile, &render_cond);
     /*   See the comment below */
     /* map to the color model */
-    for (i = 0; i < ncomps; i++)
-        cm_comps[i] = 0;
     if (dev_profile->spotnames != NULL && dev_profile->spotnames->equiv_cmyk_set) {
         map_components_to_colorants(pcc, dev_profile->spotnames->color_map,
                                     cm_comps);
@@ -1572,7 +1603,10 @@ cmap_devicen_direct(const frac * pcc,
     /* apply the transfer function(s); convert to color values.
        assign directly if output device supports devn */
     if (dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0)) {
-        if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+        if (pgs->effective_transfer_non_identity_count == 0)
+            for (i = 0; i < ncomps; i++)
+                pdc->colors.devn.values[i] = frac2cv(cm_comps[i]);
+        else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
             for (i = 0; i < ncomps; i++)
                 pdc->colors.devn.values[i] = frac2cv(gx_map_color_frac(pgs,
                                     cm_comps[i], effective_transfer[i]));
@@ -1581,23 +1615,34 @@ cmap_devicen_direct(const frac * pcc,
                 pdc->colors.devn.values[i] = frac2cv(frac_1 - gx_map_color_frac(pgs,
                             (frac)(frac_1 - cm_comps[i]), effective_transfer[i]));
         pdc->type = gx_dc_type_devn;
-    } else {
-        if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-            for (i = 0; i < ncomps; i++)
-                cv[i] = frac2cv(gx_map_color_frac(pgs,
-                                    cm_comps[i], effective_transfer[i]));
-        else
-            for (i = 0; i < ncomps; i++)
-                cv[i] = frac2cv(frac_1 - gx_map_color_frac(pgs,
-                            (frac)(frac_1 - cm_comps[i]), effective_transfer[i]));
-        /* encode as a color index */
-        color = dev_proc(dev, encode_color)(dev, cv);
-        /* check if the encoding was successful; we presume failure is rare */
-        if (color != gx_no_color_index)
-            color_set_pure(pdc, color);
-        else
-            cmap_devicen_halftoned(pcc, pdc, pgs, dev, select);
+        return;
     }
+
+    if (pgs->effective_transfer_non_identity_count == 0)
+        for (i = 0; i < ncomps; i++)
+            cv[i] = frac2cv(cm_comps[i]);
+    else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+        for (i = 0; i < ncomps; i++) {
+            cm_comps[i] = gx_map_color_frac(pgs,
+                                    cm_comps[i], effective_transfer[i]);
+            cv[i] = frac2cv(cm_comps[i]);
+        }
+    else
+        for (i = 0; i < ncomps; i++) {
+            cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+                            (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
+            cv[i] = frac2cv(cm_comps[i]);
+        }
+    /* encode as a color index */
+    color = dev_proc(dev, encode_color)(dev, cv);
+    /* check if the encoding was successful; we presume failure is rare */
+    if (color != gx_no_color_index) {
+        color_set_pure(pdc, color);
+        return;
+    }
+    if (gx_render_device_DeviceN(cm_comps, pdc, dev, pgs->dev_ht,
+                                        &pgs->screen_phase[select]) == 1)
+        gx_color_load_select(pdc, pgs, dev, select);
 }
 
 /* ------ Halftoning check ----- */
@@ -1662,7 +1707,7 @@ gx_color_frac_map(frac cv, const frac * values)
     mdv = values[cmi + 1] - mv;
 #if ARCH_INTS_ARE_SHORT
     /* Only use long multiplication if necessary. */
-    if (mdv < -1 << (16 - cp_frac_bits) ||
+    if (mdv < -(1 << (16 - cp_frac_bits)) ||
         mdv > 1 << (16 - cp_frac_bits)
         )
         return mv + (uint) (((ulong) rem * mdv) >> cp_frac_bits);
@@ -1970,7 +2015,12 @@ cmap_transfer_halftone(gx_color_value *pconc, gx_device_color * pdc,
 
     /* apply the transfer function(s) */
     if (has_transfer) {
-        if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE) {
+        if (pgs->effective_transfer_non_identity_count == 0) {
+            if (dev->color_info.polarity != GX_CINFO_POLARITY_ADDITIVE && dev->color_info.opmode == GX_CINFO_OPMODE_UNKNOWN)
+                check_cmyk_color_model_comps(dev);
+            for (i = 0; i < ncomps; i++)
+                cv_frac[i] = cv2frac(pconc[i]);
+        } else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE) {
             for (i = 0; i < ncomps; i++) {
                 frac_value = cv2frac(pconc[i]);
                 cv_frac[i] = gx_map_color_frac(pgs,
@@ -2022,7 +2072,6 @@ cmap_transfer_halftone(gx_color_value *pconc, gx_device_color * pdc,
         /* check if the encoding was successful; we presume failure is rare */
         if (color != gx_no_color_index)
             color_set_pure(pdc, color);
-
     }
 }
 
@@ -2032,41 +2081,29 @@ void
 cmap_transfer(gx_color_value *pconc, const gs_gstate * pgs, gx_device * dev)
 {
     uchar ncomps = dev->color_info.num_components;
-    frac frac_value;
     uchar i;
-    frac cv_frac[GX_DEVICE_COLOR_MAX_COMPONENTS];
 
     /* apply the transfer function(s) */
-    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE) {
-        for (i = 0; i < ncomps; i++) {
-            frac_value = cv2frac(pconc[i]);
-            cv_frac[i] = gx_map_color_frac(pgs,
-                            frac_value, effective_transfer[i]);
-            pconc[i] = frac2cv(cv_frac[i]);
-        }
-    } else {
+    if (pgs->effective_transfer_non_identity_count == 0) {
+        if (dev->color_info.polarity != GX_CINFO_POLARITY_ADDITIVE && dev->color_info.opmode == GX_CINFO_OPMODE_UNKNOWN)
+            check_cmyk_color_model_comps(dev);
+    } else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+        for (i = 0; i < ncomps; i++)
+            pconc[i] = frac2cv(gx_map_color_frac(pgs,
+                               cv2frac(pconc[i]), effective_transfer[i]));
+    else {
         if (dev->color_info.opmode == GX_CINFO_OPMODE_UNKNOWN) {
             check_cmyk_color_model_comps(dev);
         }
         if (dev->color_info.opmode == GX_CINFO_OPMODE) {  /* CMYK-like color space */
-            uint k = dev->color_info.black_component;
-            for (i = 0; i < ncomps; i++) {
-                frac_value = cv2frac(pconc[i]);
-                if (i == k) {
-                    cv_frac[i] = frac_1 - gx_map_color_frac(pgs,
-                    (frac)(frac_1 - frac_value), effective_transfer[i]);
-                } else {
-                    cv_frac[i] = cv2frac(pconc[i]);  /* Ignore transfer, see PLRM3 p. 494 */
-                }
-                pconc[i] = frac2cv(cv_frac[i]);
-            }
+            i = dev->color_info.black_component;
+            if (i < ncomps)
+                pconc[i] = frac2cv(frac_1 - gx_map_color_frac(pgs,
+                                   (frac)(frac_1 - cv2frac(pconc[i])), effective_transfer[i]));
         } else {
-            for (i = 0; i < ncomps; i++) {
-                frac_value = cv2frac(pconc[i]);
-                cv_frac[i] = frac_1 - gx_map_color_frac(pgs,
-                        (frac)(frac_1 - frac_value), effective_transfer[i]);
-                pconc[i] = frac2cv(cv_frac[i]);
-            }
+            for (i = 0; i < ncomps; i++)
+                pconc[i] = frac2cv(frac_1 - gx_map_color_frac(pgs,
+                        (frac)(frac_1 - cv2frac(pconc[i])), effective_transfer[i]));
         }
     }
 }
@@ -2090,14 +2127,12 @@ cmap_transfer_plane(gx_color_value *pconc, const gs_gstate *pgs,
         }
         if (dev->color_info.opmode == GX_CINFO_OPMODE) {  /* CMYK-like color space */
             uint k = dev->color_info.black_component;
-            frac_value = cv2frac(pconc[0]);
             if (plane == k) {
+                frac_value = cv2frac(pconc[0]);
                 cv_frac = frac_1 - gx_map_color_frac(pgs,
                 (frac)(frac_1 - frac_value), effective_transfer[plane]);
-            } else {
-                cv_frac = cv2frac(pconc[0]);  /* Ignore transfer, see PLRM3 p. 494 */
+                pconc[0] = frac2cv(cv_frac);
             }
-            pconc[0] = frac2cv(cv_frac);
         } else {
             frac_value = cv2frac(pconc[0]);
             cv_frac = frac_1 - gx_map_color_frac(pgs,
@@ -2111,6 +2146,7 @@ cmap_transfer_plane(gx_color_value *pconc, const gs_gstate *pgs,
 bool
 gx_device_uses_std_cmap_procs(gx_device * dev, const gs_gstate * pgs)
 {
+    subclass_color_mappings scm;
     const gx_cm_color_map_procs *pprocs;
     gsicc_rendering_param_t render_cond;
     cmm_dev_profile_t *dev_profile = NULL;
@@ -2121,7 +2157,12 @@ gx_device_uses_std_cmap_procs(gx_device * dev, const gs_gstate * pgs)
                           dev_profile, &des_profile, &render_cond);
 
     if (des_profile != NULL) {
-        pprocs = get_color_mapping_procs_subclass(dev);
+        scm = get_color_mapping_procs_subclass(dev);
+        pprocs = scm.procs;
+        /* FIXME: This looks wrong to me. Presumably we should be finding
+         * the parentmost device, looking at the procs for that, and if
+         * they are forwarding ones, getting the procs for the forwarded
+         * device. This is NOT what this code does. */
         /* Check if they are forwarding procs */
         if (fwd_uses_fwd_cmap_procs(dev)) {
             pprocs = fwd_get_target_cmap_procs(dev);
