@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -548,9 +548,9 @@ cpath_set_rectangle(gx_clip_path * pcpath, gs_fixed_rect * pbox)
         int code = cpath_alloc_list(&pcpath->rect_list, pcpath->path.memory,
                                     "gx_cpath_from_rectangle");
 
+        rc_decrement(rlist, "gx_cpath_from_rectangle");
         if (code < 0)
             return code;
-        rc_decrement(rlist, "gx_cpath_from_rectangle");
         rlist = pcpath->rect_list;
     }
     cpath_init_rectangle(pcpath, pbox);
@@ -571,6 +571,7 @@ gx_cpath_reset(gx_clip_path * pcpath)
     gs_fixed_rect null_rect;
 
     null_rect.p.x = null_rect.p.y = null_rect.q.x = null_rect.q.y = 0;
+    rc_decrement(pcpath->path_list, "gx_cpath_reset");
     return gx_cpath_from_rectangle(pcpath, &null_rect);
 }
 
@@ -603,6 +604,15 @@ gx_cpath_clip(gs_gstate *pgs, gx_clip_path *pcpath,
     return gx_cpath_intersect(pcpath, ppath_orig, rule, pgs);
 }
 
+int
+gx_cpath_ensure_path_list(gx_clip_path *pcpath)
+{
+    if (pcpath == NULL || pcpath->path_list)
+        return 0;
+    return gx_cpath_path_list_new(pcpath->path.memory, pcpath, pcpath->rule,
+                                  &pcpath->path, NULL, &pcpath->path_list);
+}
+
 int
 gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
                    int rule, gs_gstate *pgs, const gx_fill_params * params)
@@ -611,6 +621,7 @@ gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_or
     /*const*/ gx_path *ppath = ppath_orig;
     gs_fixed_rect old_box, new_box;
     int code;
+    int pcpath_is_rect;
 
     pcpath->cached = NULL;
     /* Flatten the path if necessary. */
@@ -624,7 +635,8 @@ gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_or
         ppath = &fpath;
     }
 
-    if (gx_cpath_inner_box(pcpath, &old_box) &&
+    pcpath_is_rect = gx_cpath_inner_box(pcpath, &old_box);
+    if (pcpath_is_rect &&
         ((code = gx_path_is_rectangle(ppath, &new_box)) ||
          gx_path_is_void(ppath))
         ) {
@@ -687,19 +699,24 @@ gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_or
         }
     } else {
         /* New clip path is nontrivial.  Intersect the slow way. */
-        gx_cpath_path_list *next = pcpath->path_list;
+        gx_cpath_path_list *next = NULL;
         bool path_valid =
-            gx_cpath_inner_box(pcpath, &old_box) &&
+            pcpath_is_rect &&
             gx_path_bbox(ppath, &new_box) >= 0 &&
             gx_cpath_includes_rectangle(pcpath,
                                         new_box.p.x, new_box.p.y,
                                         new_box.q.x, new_box.q.y);
 
         if (!path_valid && next == NULL) {
-            code = gx_cpath_path_list_new(pcpath->path.memory, pcpath, pcpath->rule,
-                                            &pcpath->path, NULL, &next);
+            /* gx_cpaths should generally have a path_list set within
+             * them. In some cases (filled images), they may not. Ensure
+             * that they do, and remember the path_list */
+            code = gx_cpath_ensure_path_list(pcpath);
             if (code < 0)
                 goto ex;
+            /* gx_cpath_intersect_path_slow NULLs pcpath->path_list, so
+             * remember it here. */
+            next = pcpath->path_list;
         }
         code = gx_cpath_intersect_path_slow(pcpath, (params != NULL ? ppath_orig : ppath),
                             rule, pgs, params);
@@ -711,7 +728,7 @@ gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_or
             pcpath->rule = rule;
         } else {
             code = gx_cpath_path_list_new(pcpath->path.memory, NULL, rule,
-                                            ppath_orig, next, &pcpath->path_list);
+                                          ppath_orig, next, &pcpath->path_list);
         }
     }
 ex:
