@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -115,7 +115,7 @@
         { (float)((((width) * 72.0 + 0.5) - 0.5) / (x_dpi))/*MediaSize[0]*/,\
           (float)((((height) * 72.0 + 0.5) - 0.5) / (y_dpi))/*MediaSize[1]*/},\
         { 0, 0, 0, 0 }/*ImagingBBox*/, 0/*ImagingBBox_set*/,\
-        { x_dpi, y_dpi }/*HWResolution*/, { x_dpi, y_dpi }/*MarginsHWResolution*/
+        { x_dpi, y_dpi }/*HWResolution*/
 
 /* offsets and margins go here */
 #define std_device_part3_()\
@@ -131,7 +131,8 @@
            BandingAuto /* banding_type */\
         }, /*space_params*/\
         0/*Profile Array*/,\
-        0/* graphics_type_tag default GS_UNKNOWN_TAG */,\
+        0/* graphics_type_tag default GS_UNTOUCHED_TAG */,\
+        1/* interpolate_control default 1, uses image /Interpolate flag, full device resolution */,\
         { gx_default_install, gx_default_begin_page, gx_default_end_page }
 /*
  * We need a number of different variants of the std_device_ macro simply
@@ -312,6 +313,18 @@ dev_proc_strip_copy_rop2(gx_default_strip_copy_rop2);
 dev_proc_strip_tile_rect_devn(gx_default_strip_tile_rect_devn);
 dev_proc_copy_alpha_hl_color(gx_default_copy_alpha_hl_color);
 dev_proc_process_page(gx_default_process_page);
+dev_proc_begin_transparency_group(gx_default_begin_transparency_group);
+dev_proc_end_transparency_group(gx_default_end_transparency_group);
+dev_proc_begin_transparency_mask(gx_default_begin_transparency_mask);
+dev_proc_end_transparency_mask(gx_default_end_transparency_mask);
+dev_proc_discard_transparency_layer(gx_default_discard_transparency_layer);
+dev_proc_pattern_manage(gx_default_pattern_manage);
+dev_proc_push_transparency_state(gx_default_push_transparency_state);
+dev_proc_pop_transparency_state(gx_default_pop_transparency_state);
+dev_proc_put_image(gx_default_put_image);
+dev_proc_copy_alpha_hl_color(gx_default_no_copy_alpha_hl_color);
+dev_proc_copy_planes(gx_default_copy_planes);
+
 /* BACKWARD COMPATIBILITY */
 #define gx_non_imaging_create_compositor gx_null_create_compositor
 
@@ -421,6 +434,11 @@ void gx_device_forward_color_procs(gx_device_forward *);
  * for each colorant.  For more info see the routine's header.
  */
 void check_device_separable(gx_device * dev);
+/*
+ * Check if the device's encode_color routine uses a pdf14 compatible
+ * encoding.  For more info see the routine's header.
+ */
+void check_device_compatible_encoding(gx_device * dev);
 /*
  * If a device has a linear and separable encode color function then
  * set up the comp_bits, comp_mask, and comp_shift fields.
@@ -653,7 +671,7 @@ typedef struct {
 } generic_subclass_data;
 
 
-int gx_copy_device_procs(gx_device_procs *dest_procs, gx_device_procs *src_procs, gx_device_procs *prototype_procs);
+int gx_copy_device_procs(gx_device *dest, gx_device *src, gx_device *prototype);
 int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, unsigned int private_data_size);
 int gx_device_unsubclass(gx_device *dev);
 int gx_update_from_subclass(gx_device *dev);
