@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -7,8 +7,8 @@
    This software is distributed under license and may not be copied, modified
    or distributed except as expressly authorized under the terms of that
    license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   or contact Artifex Software, Inc.,  1305 Grant Avenue - Suite 200,
+   Novato, CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -32,6 +32,7 @@
 #include "gdebug.h"
 #include "gsimage.h"
 #include "gsbittab.h"
+#include "gzpath.h"
 
 #include "gxfapi.h"
 
@@ -261,8 +262,8 @@ using_transparency_pattern(gs_gstate *pgs)
     gx_device *dev = gs_currentdevice_inline(pgs);
 
     return ((!gs_color_writes_pure(pgs))
-            && dev->procs.begin_transparency_group != NULL
-            && dev->procs.end_transparency_group != NULL);
+            && dev_proc(dev, begin_transparency_group) != gx_default_begin_transparency_group
+            && dev_proc(dev, end_transparency_group) != gx_default_end_transparency_group);
 }
 
 static bool
@@ -468,6 +469,11 @@ gs_fapi_prepare_font(gs_font *pfont, gs_fapi_server *I, int subfont, const char
                                                            BBox_temp, units_temp))) < 0) {
                 break;
             }
+            code = gs_notify_register(&pbfont1->notify_list, notify_remove_font, pbfont1);
+            if (code < 0) {
+                emprintf(mem,
+                         "Ignoring gs_notify_register() failure for FAPI font.....");
+            }
         }
         if (i == n) {
             code =
@@ -511,8 +517,8 @@ gs_fapi_prepare_font(gs_font *pfont, gs_fapi_server *I, int subfont, const char
     code =
         gs_notify_register(&pbfont->notify_list, notify_remove_font, pbfont);
     if (code < 0) {
-        emprintf(mem,
-                 "Ignoring gs_notify_register() failure for FAPI font.....");
+        gs_fapi_release_typeface(I, &pbfont->FAPI_font_data);
+        return code;
     }
 
     return bbox_set;
@@ -767,6 +773,9 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_gstate *pgs, gs_show_enum *penum,
     int h, padbytes, cpbytes, dstr = bitmap_raster(rast->width);
     int sstr = rast->line_step;
 
+    double dx = penum_pgs->ctm.tx + (double)rast_orig_x / (1 << frac_pixel_shift) + 0.5;
+    double dy = penum_pgs->ctm.ty + (double)rast_orig_y / (1 << frac_pixel_shift) + 0.5;
+
     /* we can only safely use the gx_image_fill_masked() "shortcut" if we're drawing
      * a "simple" colour, rather than a pattern.
      */
@@ -813,30 +822,13 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_gstate *pgs, gs_show_enum *penum,
         }
 
         if (gs_object_type(penum->memory, penum) == &st_gs_show_enum) {
-            code = gx_image_fill_masked(dev, r, 0, dstr, 0,
-                                        (int)(penum_pgs->ctm.tx +
-                                              (double)rast_orig_x /
-                                              (1 << frac_pixel_shift) +
-                                              penum->fapi_glyph_shift.x +
-                                              0.5),
-                                        (int)(penum_pgs->ctm.ty +
-                                              (double)rast_orig_y /
-                                              (1 << frac_pixel_shift) +
-                                              penum->fapi_glyph_shift.y +
-                                              0.5), rast->width, rast->height,
-                                        pdcolor, 1, rop3_default, pcpath);
-        }
-        else {
-            code = gx_image_fill_masked(dev, r, 0, dstr, 0,
-                                        (int)(penum_pgs->ctm.tx +
-                                              (double)rast_orig_x /
-                                              (1 << frac_pixel_shift) + 0.5),
-                                        (int)(penum_pgs->ctm.ty +
-                                              (double)rast_orig_y /
-                                              (1 << frac_pixel_shift) + 0.5),
-                                        rast->width, rast->height, pdcolor, 1,
-                                        rop3_default, pcpath);
+            dx += penum->fapi_glyph_shift.x;
+            dy += penum->fapi_glyph_shift.y;
         }
+        code = gx_image_fill_masked(dev, r, 0, dstr, gx_no_bitmap_id,
+                                    (int)dx, (int)dy,
+                                    rast->width, rast->height,
+                                    pdcolor, 1, rop3_default, pcpath);
         if (rast->p != r) {
             gs_free_object(penum->memory, r, "fapi_finish_render_aux");
         }
@@ -848,7 +840,9 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_gstate *pgs, gs_show_enum *penum,
         int iy, nbytes;
         uint used;
         int code1;
-        int x, y, w, h;
+        int w, h;
+        int x = (int)dx;
+        int y = (int)dy;
         uint bold = 0;
         byte *bold_lines = NULL;
         byte *line = NULL;
@@ -859,10 +853,6 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_gstate *pgs, gs_show_enum *penum,
             return_error(gs_error_VMerror);
         }
 
-        x = (int)(penum_pgs->ctm.tx +
-                  (double)rast_orig_x / (1 << frac_pixel_shift) + 0.5);
-        y = (int)(penum_pgs->ctm.ty +
-                  (double)rast_orig_y / (1 << frac_pixel_shift) + 0.5);
         w = rast->width;
         h = rast->height;
         if (I->ff.embolden != 0.0) {
@@ -889,7 +879,7 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_gstate *pgs, gs_show_enum *penum,
         image.Width = w + bold;
         image.Height = h + bold;
         image.adjust = false;
-        code = gs_image_init(pie, &image, false, penum_pgs);
+        code = gs_image_init(pie, &image, false, false, penum_pgs);
         nbytes = (rast->width + 7) >> 3;
 
         switch (code) {
@@ -1798,8 +1788,11 @@ gs_fapi_passfont(gs_font *pfont, int subfont, char *font_file_path,
         if ((code =
              gs_fapi_renderer_retcode(mem, I,
                                       I->ensure_open(I, server_param,
-                                                     server_param_size))) < 0)
+                                                     server_param_size))) < 0) {
+            gs_free_object(mem->non_gc_memory, server_param,
+                           "gs_fapi_passfont server params");
             return code;
+        }
 
         if (free_params) {
             gs_free_object(mem->non_gc_memory, server_param,
