@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -128,7 +128,8 @@ gs_image_class_2_fracs(gx_image_enum * penum)
         if ( (gs_color_space_get_index(penum->pcs) == gs_color_space_index_DeviceN &&
             penum->pcs->cmm_icc_profile_data == NULL) || penum->use_mask_color ||
             penum->bps != 16 || !std_cmap_procs ||
-            gs_color_space_get_index(penum->pcs) == gs_color_space_index_DevicePixel) {
+            gs_color_space_get_index(penum->pcs) == gs_color_space_index_DevicePixel ||
+            gs_color_space_get_index(penum->pcs) == gs_color_space_index_Indexed) {
             /* DevicePixel color space used in mask from 3x type.  Basically
                a simple color space that just is scaled to the device bit
                depth when remapped. No CM needed */
