@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -427,9 +427,10 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
     {
         gs_rect rect, rect_out;
         gs_matrix mi;
+        const gs_matrix *m = pgs != NULL ? &ctm_only(pgs) : NULL;
         gs_fixed_rect obox;
         gs_int_rect irect;
-        if ((code = gs_matrix_invert(&ctm_only(pgs), &mi)) < 0 ||
+        if (m == NULL || (code = gs_matrix_invert(m, &mi)) < 0 ||
             (code = gs_matrix_multiply(&mi, &pic->ImageMatrix, &mi)) < 0) {
             /* Give up trying to shrink the render box, but continue processing */
             break;
@@ -636,8 +637,10 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
            is not yet set, go ahead and handle that now.  It may already
            be done due to the above init_colors which may go through remap. */
         if (gs_color_space_is_PSCIE(pcs) && pcs->icc_equivalent == NULL) {
-            gs_colorspace_set_icc_equivalent((gs_color_space *)pcs, &(penum->icc_setup.is_lab),
+            code = gs_colorspace_set_icc_equivalent((gs_color_space *)pcs, &(penum->icc_setup.is_lab),
                                                 pgs->memory);
+            if (code < 0)
+                return code;
             if (penum->icc_setup.is_lab) {
                 /* Free what ever profile was created and use the icc manager's
                    cielab profile */
@@ -755,7 +758,7 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
     penum->memory = mem;
     penum->buffer = buffer;
     penum->buffer_size = bsize;
-    penum->line = 0;
+    penum->line = NULL;
     penum->icc_link = NULL;
     penum->color_cache = NULL;
     penum->ht_buffer = NULL;
@@ -935,8 +938,6 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
                 break;
         if (i == gx_image_class_table_count) {
             /* No available class can handle this image. */
-            gx_default_end_image(dev, (gx_image_enum_common_t *) penum,
-                                 false);
             return_error(gs_error_rangecheck);
         }
     }
