@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -243,12 +243,12 @@ static void gx_pattern_accum_finalize_cw(gx_device * dev)
 
 bool gx_device_is_pattern_accum(gx_device *dev)
 {
-    return dev->procs.open_device == pattern_accum_open;
+    return dev_proc(dev, open_device) == pattern_accum_open;
 }
 
 bool gx_device_is_pattern_clist(gx_device *dev)
 {
-    return dev->procs.open_device == pattern_clist_open_device;
+    return dev_proc(dev, open_device) == pattern_clist_open_device;
 }
 
 /* Allocate a pattern accumulator, with an initial refct of 0. */
@@ -257,7 +257,7 @@ gx_pattern_accum_alloc(gs_memory_t * mem, gs_memory_t * storage_memory,
                        gs_pattern1_instance_t *pinst, client_name_t cname)
 {
     gx_device *tdev = pinst->saved->device;
-    bool has_tags = (tdev->graphics_type_tag & GS_DEVICE_ENCODES_TAGS) != 0;
+    bool has_tags = device_encodes_tags(tdev);
     int size = gx_pattern_size_estimate(pinst, has_tags);
     gx_device_forward *fdev;
     int force_no_clist = 0;
@@ -326,13 +326,14 @@ gx_pattern_accum_alloc(gs_memory_t * mem, gs_memory_t * storage_memory,
         }
         cwdev = (gx_device_clist_writer *)cdev;
         cwdev->finalize = gx_pattern_accum_finalize_cw;
-        cwdev->procs.open_device = pattern_clist_open_device;
+        set_dev_proc(cwdev, open_device, pattern_clist_open_device);
         fdev = (gx_device_forward *)cdev;
     }
     fdev->log2_align_mod = tdev->log2_align_mod;
     fdev->pad = tdev->pad;
     fdev->is_planar = tdev->is_planar;
-    check_device_separable((gx_device *)fdev);
+    fdev->graphics_type_tag = tdev->graphics_type_tag;
+    fdev->interpolate_control = tdev->interpolate_control;
     gx_device_forward_fill_in_procs(fdev);
     return fdev;
 }
@@ -543,10 +544,14 @@ pattern_accum_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
                                       const gx_clip_path *pcpath)
 {
     gx_device_pattern_accum *const padev = (gx_device_pattern_accum *) dev;
+    int code;
 
-    if (padev->bits)
-        (*dev_proc(padev->target, fill_rectangle_hl_color))
+    if (padev->bits) {
+        code = (*dev_proc(padev->target, fill_rectangle_hl_color))
             (padev->target, rect, pgs, pdcolor, pcpath);
+        if (code < 0)
+            return code;
+    }
     if (padev->mask) {
         int x, y, w, h;
 
@@ -558,8 +563,7 @@ pattern_accum_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
         return (*dev_proc(padev->mask, fill_rectangle))
             ((gx_device *) padev->mask, x, y, w, h, (gx_color_index) 1);
     }
-     else
-        return 0;
+    return 0;
 }
 
 /* Fill a rectangle */
@@ -833,8 +837,23 @@ gx_pattern_alloc_cache(gs_memory_t * mem, uint num_tiles, ulong max_bits)
         /* Clear the pointers to pacify the GC. */
         uid_set_invalid(&tiles->uid);
         tiles->bits_used = 0;
+#ifdef PACIFY_VALGRIND
+        /* The following memsets are required to avoid a valgrind warning
+         * in:
+         *   gs -I./gs/lib -sOutputFile=out.pgm -dMaxBitmap=10000
+         *      -sDEVICE=pgmraw -r300 -Z: -sDEFAULTPAPERSIZE=letter
+         *      -dNOPAUSE -dBATCH -K2000000 -dClusterJob -dJOBSERVER
+         *      tests_private/ps/ps3cet/11-14.PS
+         * Setting the individual elements of the structures directly is
+         * not enough, which leads me to believe that we are writing the
+         * entire structs out, padding and all.
+         */
+        memset(&tiles->tbits, 0, sizeof(tiles->tbits));
+        memset(&tiles->tmask, 0, sizeof(tiles->tmask));
+#else
         tiles->tbits.data = 0;
         tiles->tmask.data = 0;
+#endif
         tiles->index = i;
         tiles->cdev = NULL;
         tiles->ttrans = NULL;
@@ -924,7 +943,7 @@ gx_pattern_cache_free_entry(gx_pattern_cache * pcache, gx_color_tile * ctile)
         }
 
         if (ctile->ttrans != NULL) {
-            if_debug2m('?', mem,
+            if_debug2m('v', mem,
                        "[v*] Freeing trans pattern from cache, uid = %ld id = %ld \n",
                        ctile->uid.id, ctile->id);
             if ( ctile->ttrans->pdev14 == NULL) {
@@ -1018,7 +1037,7 @@ gx_pattern_cache_add_entry(gs_gstate * pgs,
         return code;
     pcache = pgs->pattern_cache;
 
-    if (fdev->procs.open_device != pattern_clist_open_device) {
+    if (dev_proc(fdev, open_device) != pattern_clist_open_device) {
         gx_device_pattern_accum *padev = (gx_device_pattern_accum *)fdev;
 
         mbits = padev->bits;
@@ -1093,7 +1112,7 @@ gx_pattern_cache_add_entry(gs_gstate * pgs,
         ctile->blending_mode = ((pdf14_device *)(pgs->device))->blend_mode;
     else
         ctile->blending_mode = 0;
-    if (fdev->procs.open_device != pattern_clist_open_device) {
+    if (dev_proc(fdev, open_device) != pattern_clist_open_device) {
         if (mbits != 0) {
             make_bitmap(&ctile->tbits, mbits, gs_next_ids(pgs->memory, 1));
             mbits->bitmap_memory = 0;   /* don't free the bits */
@@ -1105,7 +1124,7 @@ gx_pattern_cache_add_entry(gs_gstate * pgs,
         } else
             ctile->tmask.data = 0;
         if (trans != 0) {
-            if_debug2m('?', pgs->memory,
+            if_debug2m('v', pgs->memory,
                        "[v*] Adding trans pattern to cache, uid = %ld id = %ld \n",
                        ctile->uid.id, ctile->id);
             ctile->ttrans = trans;
@@ -1348,8 +1367,10 @@ gx_erase_colored_pattern(gs_gstate *pgs)
         rect.q.y = (double)pdev->height;
 
         code = gs_setmatrix(pgs, &identity);
-        if (code < 0)
+        if (code < 0) {
+            gs_grestore_only(pgs);
             return code;
+        }
         /* we don't want the fill rectangle device call to use the
            mask */
         mask = pdev->mask;
@@ -1357,8 +1378,10 @@ gx_erase_colored_pattern(gs_gstate *pgs)
         code = gs_rectfill(pgs, &rect, 1);
         /* restore the mask */
         pdev->mask = mask;
-        if (code < 0)
+        if (code < 0) {
+            gs_grestore_only(pgs);
             return code;
+        }
     }
     /* we don't need wraparound here */
     return gs_grestore_only(pgs);
@@ -1376,7 +1399,7 @@ gx_pattern_load(gx_device_color * pdc, const gs_gstate * pgs,
     gs_gstate *saved;
     gx_color_tile *ctile;
     gs_memory_t *mem = pgs->memory;
-    bool has_tags = (dev->graphics_type_tag & GS_DEVICE_ENCODES_TAGS) != 0;
+    bool has_tags = device_encodes_tags(dev);
     int code;
 
     if (pgs->pattern_cache == NULL)
@@ -1420,7 +1443,8 @@ gx_pattern_load(gx_device_color * pdc, const gs_gstate * pgs,
            make a similar change in zpcolor.c where much of this
            pattern code is duplicated to support high level stream
            patterns. */
-        if (pinst->templat.PaintType == 1 && !(pinst->is_clist))
+        if (pinst->templat.PaintType == 1 && !(pinst->is_clist)
+            && dev_proc(pinst->saved->device, dev_spec_op)(pinst->saved->device, gxdso_pattern_can_accum, NULL, 0) == 0)
             if ((code = gx_erase_colored_pattern(saved)) < 0)
                 return code;
     }
@@ -1481,7 +1505,7 @@ gx_pattern_load(gx_device_color * pdc, const gs_gstate * pgs,
         }
     }
 #ifdef DEBUG
-    if (gs_debug_c('B') && adev->procs.open_device == pattern_accum_open) {
+    if (gs_debug_c('B') && dev_proc(adev, open_device) == pattern_accum_open) {
         gx_device_pattern_accum *pdev = (gx_device_pattern_accum *)adev;
 
         if (pdev->mask)
@@ -1502,7 +1526,7 @@ gx_pattern_load(gx_device_color * pdc, const gs_gstate * pgs,
     gs_gstate_free_chain(saved);
     return code;
 fail:
-    if (adev->procs.open_device == pattern_clist_open_device) {
+    if (dev_proc(adev, open_device) == pattern_clist_open_device) {
         gx_device_clist *cdev = (gx_device_clist *)adev;
 
         gs_free_object(cdev->writer.bandlist_memory, cdev->common.data, "gx_pattern_load");
@@ -1531,8 +1555,11 @@ gs_pattern1_remap_color(const gs_client_color * pc, const gs_color_space * pcs,
         return 0;
     }
     if (pinst->templat.PaintType == 2) {       /* uncolored */
-        code = (pcs->base_space->type->remap_color)
-            (pc, pcs->base_space, pdc, pgs, dev, select);
+        if (pcs->base_space)
+            code = (pcs->base_space->type->remap_color)
+                (pc, pcs->base_space, pdc, pgs, dev, select);
+        else
+            code = gs_note_error(gs_error_unregistered);
         if (code < 0)
             return code;
         if (pdc->type == gx_dc_type_pure)
