@@ -1,26 +1,27 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Path copying and flattening */
 #include "math_.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gxfixed.h"
 #include "gxfarith.h"
-#include "gxistate.h"		/* for access to line params */
+#include "gxgstate.h"		/* for access to line params */
 #include "gzpath.h"
-#include "vdtrace.h"
 
 /* Forward declarations */
 static void adjust_point_to_tangent(segment *, const segment *,
@@ -52,12 +53,38 @@ break_line_if_long(gx_path *ppath, const segment *pseg)
     }
     return 0;
 }
+static inline int
+break_gap_if_long(gx_path *ppath, const segment *pseg)
+{
+    fixed x0 = ppath->position.x;
+    fixed y0 = ppath->position.y;
+
+    if (gx_check_fixed_diff_overflow(pseg->pt.x, x0) ||
+        gx_check_fixed_diff_overflow(pseg->pt.y, y0)) {
+        fixed x, y;
+
+        if (gx_check_fixed_sum_overflow(pseg->pt.x, x0))
+            x = (pseg->pt.x >> 1) + (x0 >> 1);
+        else
+            x = (pseg->pt.x + x0) >> 1;
+        if (gx_check_fixed_sum_overflow(pseg->pt.y, y0))
+            y = (pseg->pt.y >> 1) + (y0 >> 1);
+        else
+            y = (pseg->pt.y + y0) >> 1;
+        return gx_path_add_gap_notes(ppath, x, y, pseg->notes);
+        /* WARNING: Stringly speaking, the next half segment must get
+           the sn_not_first flag. We don't bother, because that flag
+           has no important meaning with colinear segments.
+         */
+    }
+    return 0;
+}
 
 /* Copy a path, optionally flattening or monotonizing it. */
 /* If the copy fails, free the new path. */
 int
 gx_path_copy_reducing(const gx_path *ppath_old, gx_path *ppath,
-                      fixed fixed_flatness, const gs_imager_state *pis,
+                      fixed fixed_flatness, const gs_gstate *pgs,
                       gx_path_copy_options options)
 {
     const segment *pseg;
@@ -77,22 +104,20 @@ gx_path_copy_reducing(const gx_path *ppath_old, gx_path *ppath,
 #endif
     if (options & pco_for_stroke) {
         /* Precompute the maximum expansion of the bounding box. */
-        double width = pis->line_params.half_width;
+        double width = pgs->line_params.half_width;
 
         expansion.x =
-            float2fixed((fabs(pis->ctm.xx) + fabs(pis->ctm.yx)) * width) * 2;
+            float2fixed((fabs(pgs->ctm.xx) + fabs(pgs->ctm.yx)) * width) * 2;
         expansion.y =
-            float2fixed((fabs(pis->ctm.xy) + fabs(pis->ctm.yy)) * width) * 2;
+            float2fixed((fabs(pgs->ctm.xy) + fabs(pgs->ctm.yy)) * width) * 2;
     } else
         expansion.x = expansion.y = 0; /* Quiet gcc warning. */
-    vd_setcolor(RGB(255,255,0));
     pseg = (const segment *)(ppath_old->first_subpath);
     while (pseg) {
         switch (pseg->type) {
             case s_start:
                 code = gx_path_add_point(ppath,
                                          pseg->pt.x, pseg->pt.y);
-                vd_moveto(pseg->pt.x, pseg->pt.y);
                 break;
             case s_curve:
                 {
@@ -176,7 +201,6 @@ gx_path_copy_reducing(const gx_path *ppath_old, gx_path *ppath,
                                                           notes);
                             if (code < 0)
                                 break;
-                            vd_lineto(x0, y0);
                             start = ppath->current_subpath->last;
                             notes |= sn_not_first;
                             cseg = *pc;
@@ -188,7 +212,6 @@ gx_path_copy_reducing(const gx_path *ppath_old, gx_path *ppath,
                              * they line up with the tangents.
                              */
                             end = ppath->current_subpath->last;
-                            vd_lineto(ppath->position.x, ppath->position.y);
                             if ((code = gx_path_add_line_notes(ppath,
                                                           ppath->position.x,
                                                           ppath->position.y,
@@ -219,7 +242,13 @@ gx_path_copy_reducing(const gx_path *ppath_old, gx_path *ppath,
                     break;
                 code = gx_path_add_line_notes(ppath,
                                        pseg->pt.x, pseg->pt.y, pseg->notes);
-                vd_lineto(pseg->pt.x, pseg->pt.y);
+                break;
+            case s_gap:
+                code = break_gap_if_long(ppath, pseg);
+                if (code < 0)
+                    break;
+                code = gx_path_add_gap_notes(ppath,
+                                       pseg->pt.x, pseg->pt.y, pseg->notes);
                 break;
             case s_dash:
                 {
@@ -234,7 +263,6 @@ gx_path_copy_reducing(const gx_path *ppath_old, gx_path *ppath,
                 if (code < 0)
                     break;
                 code = gx_path_close_subpath(ppath);
-                vd_closepath;
                 break;
             default:		/* can't happen */
                 code = gs_note_error(gs_error_unregistered);
@@ -245,9 +273,14 @@ gx_path_copy_reducing(const gx_path *ppath_old, gx_path *ppath,
         }
         pseg = pseg->next;
     }
-    if (path_last_is_moveto(ppath_old))
-        gx_path_add_point(ppath, ppath_old->position.x,
+    if (path_last_is_moveto(ppath_old)) {
+        code = gx_path_add_point(ppath, ppath_old->position.x,
                           ppath_old->position.y);
+        if (code < 0) {
+            gx_path_new(ppath);
+            return code;
+        }
+    }
     if (ppath_old->bbox_set) {
         if (ppath->bbox_set) {
             ppath->bbox.p.x = min(ppath_old->bbox.p.x, ppath->bbox.p.x);
@@ -348,6 +381,7 @@ gx_path__check_curves(const gx_path * ppath, gx_path_copy_options options, fixed
                 }
                 break;
             case s_line:
+            case s_gap:
                 if (gx_check_fixed_diff_overflow(pseg->pt.x, pt0.x) ||
                     gx_check_fixed_diff_overflow(pseg->pt.y, pt0.y))
                     return false;
@@ -428,8 +462,8 @@ gx_curve_monotonize(gx_path * ppath, const curve_segment * pc)
 {
     fixed x0 = ppath->position.x, y0 = ppath->position.y;
     segment_notes notes = pc->notes;
-    double t[4], tt = 1, tp;
-    int c[4];
+    double t[5], tt = 1, tp;
+    int c[5];
     int n0, n1, n, i, j, k = 0;
     fixed ax, bx, cx, ay, by, cy, v01, v12;
     fixed px, py, qx, qy, rx, ry, sx, sy;
@@ -697,12 +731,15 @@ find_contacting_segments(const subpath *sp0, segment *sp0last,
        Instead it either quickly finds something, or maybe not. */
     for (s0 = sp0last, count0 = 0; count0 < search_limit && s0 != (segment *)sp0; s0 = s0->prev, count0++) {
         s0s = s0->prev;
-        if (s0->type == s_line && (s0s->pt.x == s0->pt.x ||
-            (any_abs(s0s->pt.x - s0->pt.x) == 1 && any_abs(s0s->pt.y - s0->pt.y) > min_length))) {
+        if ((s0->type == s_line || s0->type == s_gap) &&
+            (s0s->pt.x == s0->pt.x ||
+             (any_abs(s0s->pt.x - s0->pt.x) == 1 &&
+              any_abs(s0s->pt.y - s0->pt.y) > min_length))) {
             for (s1 = sp1last, count1 = 0; count1 < search_limit && s1 != (segment *)sp1; s1 = s1->prev, count1++) {
                 s1s = s1->prev;
-                if (s1->type == s_line && (s1s->pt.x == s1->pt.x ||
-                    (any_abs(s1s->pt.x - s1->pt.x) == 1 && any_abs(s1s->pt.y - s1->pt.y) > min_length))) {
+                if ((s1->type == s_line || s1->type == s_gap) &&
+                    (s1s->pt.x == s1->pt.x ||
+                     (any_abs(s1s->pt.x - s1->pt.x) == 1 && any_abs(s1s->pt.y - s1->pt.y) > min_length))) {
                     if (s0s->pt.x == s1s->pt.x || s0->pt.x == s1->pt.x || s0->pt.x == s1s->pt.x || s0s->pt.x == s1->pt.x) {
                         if (s0s->pt.y < s0->pt.y && s1s->pt.y > s1->pt.y) {
                             fixed y0 = max(s0s->pt.y, s1->pt.y);
