@@ -1,4 +1,4 @@
-/* Copyright (C) 2009-2016 Artifex Software, Inc.
+/* Copyright (C) 2009-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -7,8 +7,8 @@
    This software is distributed under license and may not be copied, modified
    or distributed except as expressly authorized under the terms of that
    license. Refer to licensing information at http://www.artifex.com
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   or contact Artifex Software, Inc.,  1305 Grant Avenue - Suite 200,
+   Novato, CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Inspired by Fortify by Simon P Bullen. */
@@ -823,6 +823,30 @@ static void Memento_storeDetails(Memento_BlkHeader *head, int type)
 }
 #endif
 
+void (Memento_bt)(void)
+{
+#ifdef MEMENTO_STACKTRACE_METHOD
+    void *stack[MEMENTO_BACKTRACE_MAX];
+    int count;
+    int skip;
+
+    count = Memento_getStacktrace(stack, &skip);
+    Memento_showStacktrace(&stack[skip-2], count-skip+2);
+#endif
+}
+
+static void Memento_bt_internal(int skip2)
+{
+#ifdef MEMENTO_STACKTRACE_METHOD
+    void *stack[MEMENTO_BACKTRACE_MAX];
+    int count;
+    int skip;
+
+    count = Memento_getStacktrace(stack, &skip);
+    Memento_showStacktrace(&stack[skip+skip2], count-skip-skip2);
+#endif
+}
+
 void Memento_breakpoint(void)
 {
     /* A handy externally visible function for breakpointing */
@@ -1396,22 +1420,25 @@ static int Memento_nonLeakBlocksLeaked(void)
 void Memento_fin(void)
 {
     Memento_checkAllMemory();
-    Memento_endStats();
-    if (Memento_nonLeakBlocksLeaked()) {
-        Memento_listBlocks();
+    if (!memento.segv)
+    {
+        Memento_endStats();
+        if (Memento_nonLeakBlocksLeaked()) {
+            Memento_listBlocks();
 #ifdef MEMENTO_DETAILS
-        fprintf(stderr, "\n");
-        Memento_listBlockInfo();
+            fprintf(stderr, "\n");
+            Memento_listBlockInfo();
 #endif
-        Memento_breakpoint();
+            Memento_breakpoint();
+	}
     }
-    if (memento.segv) {
-        fprintf(stderr, "Memory dumped on SEGV while squeezing @ %d\n", memento.failAt);
-    } else if (memento.squeezing) {
+    if (memento.squeezing) {
         if (memento.pattern == 0)
-            fprintf(stderr, "Memory squeezing @ %d complete\n", memento.squeezeAt);
+            fprintf(stderr, "Memory squeezing @ %d complete%s\n", memento.squeezeAt, memento.segv ? " (with SEGV)" : "");
         else
-            fprintf(stderr, "Memory squeezing @ %d (%d) complete\n", memento.squeezeAt, memento.pattern);
+            fprintf(stderr, "Memory squeezing @ %d (%d) complete%s\n", memento.squeezeAt, memento.pattern, memento.segv ? " (with SEGV)" : "");
+    } else if (memento.segv) {
+        fprintf(stderr, "Memory squeezing complete (with SEGV)\n", memento.failAt);
     }
     if (memento.failing)
     {
@@ -1525,6 +1552,7 @@ void Memento_info(void *addr)
 #ifdef MEMENTO_HAS_FORK
 #include <unistd.h>
 #include <sys/wait.h>
+#include <time.h>
 #ifdef MEMENTO_STACKTRACE_METHOD
 #if MEMENTO_STACKTRACE_METHOD == 1
 #include <signal.h>
@@ -1540,24 +1568,12 @@ void Memento_info(void *addr)
 /* stashed_map[j] = i means that file descriptor i-1 was duplicated to j */
 int stashed_map[OPEN_MAX];
 
-static void Memento_signal(void)
+static void Memento_signal(int sig)
 {
-    fprintf(stderr, "SEGV after Memory squeezing @ %d\n", memento.squeezeAt);
-
-#ifdef MEMENTO_STACKTRACE_METHOD
-#if MEMENTO_STACKTRACE_METHOD == 1
-    {
-      void *array[100];
-      size_t size;
-
-      size = backtrace(array, 100);
-      fprintf(stderr, "------------------------------------------------------------------------\n");
-      fprintf(stderr, "Backtrace:\n");
-      backtrace_symbols_fd(array, size, 2);
-      fprintf(stderr, "------------------------------------------------------------------------\n");
-    }
-#endif
-#endif
+    (void)sig;
+    fprintf(stderr, "SEGV at:\n");
+    memento.segv = 1;
+    Memento_bt_internal(0);
 
     exit(1);
 }
@@ -1590,6 +1606,8 @@ static int squeeze(void)
         }
     }
 
+    fprintf(stderr, "Failing at:\n");
+    Memento_bt_internal(2);
     pid = fork();
     if (pid == 0) {
         /* Child */
@@ -1608,8 +1626,25 @@ static int squeeze(void)
     memento.pattern |= memento.patternBit;
     memento.patternBit <<= 1;
 
-    /* Wait for pid to finish */
-    waitpid(pid, &status, 0);
+    /* Wait for pid to finish, with a timeout. */
+    {
+        struct timespec tm = { 0, 10 * 1000 * 1000 }; /* 10ms = 100th sec */
+        int timeout = 30 * 1000 * 1000; /* time out in microseconds! */
+        while (waitpid(pid, &status, WNOHANG) == 0) {
+            nanosleep(&tm, NULL);
+            timeout -= (tm.tv_nsec/1000);
+            tm.tv_nsec *= 2;
+            if (tm.tv_nsec > 999999999)
+                tm.tv_nsec = 999999999;
+            if (timeout <= 0) {
+                char text[32];
+                fprintf(stderr, "Child is taking a long time to die. Killing it.\n");
+                sprintf(text, "kill %d", pid);
+                system(text);
+                break;
+            }
+        }
+    }
 
     if (status != 0) {
         fprintf(stderr, "Child status=%d\n", status);
@@ -1629,8 +1664,9 @@ static int squeeze(void)
 #else
 #include <signal.h>
 
-static void Memento_signal(void)
+static void Memento_signal(int sig)
 {
+    (void)sig;
     memento.segv = 1;
     /* If we just return from this function the SEGV will be unhandled, and
      * we'll launch into whatever JIT debugging system the OS provides. At
@@ -1691,6 +1727,11 @@ static int Memento_event(void)
     return 0;
 }
 
+int Memento_sequence(void)
+{
+    return memento.sequence;
+}
+
 int Memento_breakAt(int event)
 {
     memento.breakAt = event;
