@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -24,8 +24,9 @@
 #include "sjpx_openjpeg.h"
 #include "gxsync.h"
 #include "assert_.h"
+#if !defined(SHARE_JPX) || (SHARE_JPX == 0)
 #include "opj_malloc.h"
-
+#endif
 /* Some locking to get around the criminal lack of context
  * in the openjpeg library. */
 static gs_memory_t *opj_memory;
@@ -138,7 +139,7 @@ void opj_free(void *ptr)
     gs_free_object(opj_memory, ptr, "opj_malloc");
 }
 
-void * opj_aligned_malloc(size_t size)
+static inline void * opj_aligned_malloc_n(size_t size, size_t align)
 {
     uint8_t *ptr;
     int off;
@@ -146,15 +147,25 @@ void * opj_aligned_malloc(size_t size)
     if (size == 0)
         return NULL;
 
-    size += 16 + sizeof(uint8_t);
+    size += align + sizeof(uint8_t);
     ptr = opj_malloc(size);
     if (ptr == NULL)
         return NULL;
-    off = 16-(((int)(intptr_t)ptr) & 15);
+    off = align - (((int)(intptr_t)ptr) & (align - 1));
     ptr[off-1] = off;
     return ptr + off;
 }
 
+void * opj_aligned_malloc(size_t size)
+{
+    return opj_aligned_malloc_n(size, 16);
+}
+
+void *opj_aligned_32_malloc(size_t size)
+{
+    return opj_aligned_malloc_n(size, 32);
+}
+
 void opj_aligned_free(void* ptr_)
 {
     uint8_t *ptr = (uint8_t *)ptr_;
@@ -306,16 +317,6 @@ s_opjd_set_codec_format(stream_state * ss, OPJ_CODEC_FORMAT format)
     return 0;
 }
 
-/* calculate the real component data idx after scaling */
-static inline unsigned long
-get_scaled_idx(stream_jpxd_state *state, int compno, unsigned long idx, unsigned long x, unsigned long y)
-{
-    if (state->samescale)
-	return idx;
-	
-    return (y/state->image->comps[compno].dy*state->width + x)/state->image->comps[compno].dx;
-}
-
 static void
 ycc_to_rgb_8(unsigned char *row, unsigned long row_size)
 {
@@ -389,76 +390,6 @@ ycc_to_rgb_16(unsigned char *row, unsigned long row_size)
     while (row_size);
 }
 
-/* convert state->image from YCrCb to RGB */
-static int
-s_jpxd_ycc_to_rgb(stream_jpxd_state *state)
-{
-    unsigned int max_value = ~(-1 << state->image->comps[0].prec); /* maximum of channel value */
-    int flip_value = 1 << (state->image->comps[0].prec-1);
-    int p[3], q[3], i;
-    int sgnd[2];  /* Cr, Cb */
-    unsigned long x, y, idx;
-    int *row_bufs[3];
-
-    if (state->out_numcomps != 3)
-    return -1;
-
-    for (i=0; i<2; i++)
-        sgnd[i] = state->image->comps[i+1].sgnd;
-
-    for (i=0; i<3; i++)
-    	row_bufs[i] = (int *)gs_alloc_byte_array(state->memory->non_gc_memory, sizeof(int)*state->width, 1, "s_jpxd_ycc_to_rgb");
-
-    if (!row_bufs[0] || !row_bufs[1] || !row_bufs[2])
-        return_error(gs_error_VMerror);
-
-    idx=0;
-    for (y=0; y<state->height; y++)
-    {
-        /* backup one row. the real buffer might be overriden when there is scale */
-        for (i=0; i<3; i++)
-        {
-            if ((y % state->image->comps[i].dy) == 0) /* get the buffer once every dy rows */
-                memcpy(row_bufs[i], &(state->image->comps[i].data[get_scaled_idx(state, i, idx, 0, y)]), sizeof(int)*state->width/state->image->comps[i].dx);
-        }
-        for (x=0; x<state->width; x++, idx++)
-        {
-            for (i = 0; i < 3; i++)
-                p[i] = row_bufs[i][x/state->image->comps[i].dx];
-
-            if (!sgnd[0])
-                p[1] -= flip_value;
-            if (!sgnd[1])
-                p[2] -= flip_value;
-
-            /* rotate to RGB */
-#ifdef JPX_USE_IRT
-            q[1] = p[0] - ((p[1] + p[2])>>2);
-            q[0] = p[1] + q[1];
-            q[2] = p[2] + q[1];
-#else
-            q[0] = (int)((double)p[0] + 1.402 * p[2]);
-            q[1] = (int)((double)p[0] - 0.34413 * p[1] - 0.71414 * p[2]);
-            q[2] = (int)((double)p[0] + 1.772 * p[1]);
-#endif
-            /* clamp */
-            for (i = 0; i < 3; i++){
-                if (q[i] < 0) q[i] = 0;
-                else if (q[i] > max_value) q[i] = max_value;
-            }
-
-            /* write out the pixel */
-            for (i = 0; i < 3; i++)
-                state->image->comps[i].data[get_scaled_idx(state, i, idx, x, y)] = q[i];
-        }
-    }
-
-    for (i=0; i<3; i++)
-    	gs_free_object(state->memory->non_gc_memory, row_bufs[i],"s_jpxd_ycc_to_rgb");
-
-    return 0;
-}
-
 static int decode_image(stream_jpxd_state * const state)
 {
     int numprimcomp = 0, alpha_comp = -1, compno, rowbytes;
@@ -582,12 +513,10 @@ static int process_one_trunk(stream_jpxd_state * const state, stream_cursor_writ
     int compno;
     unsigned long i;
     int b;
-    byte *pend = pw->ptr+write_size+1; /* end of write data */
     byte *row;
     unsigned int x_offset;
     unsigned int y_offset;
     unsigned int row_size = (state->width * state->out_numcomps * state->bpp + 7)>>3;
-    unsigned long image_total = state->width * state->height;
 
     /* If nothing to write, nothing to do */
     if (write_size == 0)
@@ -647,7 +576,6 @@ static int process_one_trunk(stream_jpxd_state * const state, stream_cursor_writ
                     /* shift_bit = 0, bpp < 8 */
                     int bt=0;
                     int bit_pos = 0;
-                    int rowbytes = (state->width*state->bpp*state->out_numcomps+7)/8; /*row bytes */
                     for (i = state->width; i > 0; i--)
                     {
                         for (compno=0; compno<img_numcomps; compno++)
@@ -701,7 +629,6 @@ static int process_one_trunk(stream_jpxd_state * const state, stream_cursor_writ
             }
             else
             {
-                unsigned long image_total = state->width*state->height;
                 int compno = state->alpha ? state->alpha_comp : 0;
                 int bt=0;
                 int ppbyte1 = 8/state->bpp;
