@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* gdevdsp.c */
@@ -306,7 +306,7 @@ display_sync_output(gx_device * dev)
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return 0;	/* ignore the call */
     display_set_separations(ddev);
 
     (*(ddev->callback->display_sync))(ddev->pHandle, dev);
@@ -321,7 +321,7 @@ display_output_page(gx_device * dev, int copies, int flush)
     gx_device_display *ddev = (gx_device_display *) dev;
     int code;
     if (ddev->callback == NULL)
-        return 0;
+        return gs_error_Fatal;
     display_set_separations(ddev);
 
     code = (*(ddev->callback->display_page))
@@ -338,7 +338,7 @@ display_close(gx_device * dev)
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return 0;	/* ignore the call since we were never properly opened */
 
     /* Tell caller that device is about to be closed. */
     (*(ddev->callback->display_preclose))(ddev->pHandle, dev);
@@ -709,7 +709,7 @@ display_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return 0;		/* ignore -- needed for fillpage when device wasn't really opened */
     dev_proc(ddev->mdev, fill_rectangle)((gx_device *)ddev->mdev,
         x, y, w, h, color);
     if (ddev->callback->display_update)
@@ -726,7 +726,7 @@ display_copy_mono(gx_device * dev,
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return gs_error_Fatal;
     dev_proc(ddev->mdev, copy_mono)((gx_device *)ddev->mdev,
         base, sourcex, raster, id, x, y, w, h, zero, one);
     if (ddev->callback->display_update)
@@ -742,7 +742,7 @@ display_copy_color(gx_device * dev,
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return gs_error_Fatal;
     dev_proc(ddev->mdev, copy_color)((gx_device *)ddev->mdev,
         base, sourcex, raster, id, x, y, w, h);
     if (ddev->callback->display_update)
@@ -755,7 +755,7 @@ display_get_bits(gx_device * dev, int y, byte * str, byte ** actual_data)
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return gs_error_Fatal;
     return dev_proc(ddev->mdev, get_bits)((gx_device *)ddev->mdev,
         y, str, actual_data);
 }
@@ -1032,9 +1032,20 @@ display_put_params(gx_device * dev, gs_param_list * plist)
 
         code = display_alloc_bitmap(ddev, dev);
         if (code < 0) {
-            /* No bitmap, so tell the caller it is zero size */
-            (*ddev->callback->display_size)(ddev->pHandle, dev,
-                0, 0, 0, ddev->nFormat, NULL);
+            int ecode;
+
+            /* if we failed (probably VMerror) try to revert to old settings */
+            *pdevn_params = saved_devn_params;
+            *pequiv_colors = saved_equiv_colors;
+            display_set_color_format(ddev, old_format);
+            ddev->nFormat = old_format;
+            dev->width = old_width;
+            dev->height = old_height;
+            ecode = display_alloc_bitmap(ddev, dev);
+            if (ecode < 0) {
+                emprintf(dev->memory, "*** Fatal error in display_put_params, could not allocate bitmap ***\n");
+                return_error(gs_error_Fatal);
+            }
             return_error(code);
         }
 
@@ -1318,7 +1329,7 @@ display_alloc_bitmap(gx_device_display * ddev, gx_device * param_dev)
     int ccode;
     const gx_device_memory *mdproto;
     if (ddev->callback == NULL)
-        return 0;
+        return gs_error_Fatal;
 
     /* free old bitmap (if any) */
     display_free_bitmap(ddev);
@@ -1390,7 +1401,7 @@ display_alloc_bitmap(gx_device_display * ddev, gx_device * param_dev)
                 ? gx_max_color_value : 0;
         dev_proc(ddev, fill_rectangle)((gx_device *)ddev,
                  0, 0, ddev->width, ddev->height,
-                 ddev->procs.encode_color((gx_device *)ddev, cv));
+                 dev_proc(ddev, encode_color)((gx_device *)ddev, cv));
     }
 
     return ccode;
