@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* JPEG output driver */
@@ -374,7 +374,7 @@ jpeg_get_initial_matrix(gx_device *dev, gs_matrix *pmat)
 
     /* NB this device has no paper margins */
 
-    switch(pdev->LeadingEdge) {
+    switch(pdev->LeadingEdge & LEADINGEDGE_MASK) {
     case 1:
         pmat->xx = 0;
         pmat->xy = -ss_res;
@@ -544,7 +544,6 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
 
     /* Copy the data to the output. */
     for (lnum = 0; lnum < jcdp->cinfo.image_height; ++lnum) {
-        byte *data;
         uint ignore_used;
 
         if (jstrm.end_status) {
