@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -297,14 +297,14 @@ const gx_device_pngalpha gs_pngalpha_device = {
          256 /* dither grays */,
          256 /* dither colors */,
          { 4, 4 } /* antialias info text, graphics */,
-         GX_CINFO_SEP_LIN_NONE /* separable_and_linear */,
+         GX_CINFO_UNKNOWN_SEP_LIN /* separable_and_linear */,
          { 0 } /* component shift */,
          { 0 } /* component bits */,
          { 0 } /* component mask */,
          "DeviceRGB" /* process color name */,
          GX_CINFO_OPMODE_UNKNOWN /* opmode */,
          0 /* process_cmps */,
-         0 /* icc_locations */    
+         0 /* icc_locations */
         },
         std_device_part2_(
           (int)((float)(DEFAULT_WIDTH_10THS) * (X_DPI) / 10 + 0.5),
@@ -341,7 +341,7 @@ png_put_params_downscale(gx_device *dev, gs_param_list *plist)
 {
     gx_device_png *pdev = (gx_device_png *)dev;
     int code, ecode;
-    
+
     ecode = gx_downscaler_read_params(plist, &pdev->downscale, 0);
 
     code = gdev_prn_put_params(dev, plist);
@@ -356,7 +356,7 @@ png_get_params_downscale_mfs(gx_device *dev, gs_param_list *plist)
 {
     gx_device_png *pdev = (gx_device_png *)dev;
     int code, ecode;
-    
+
     ecode = gx_downscaler_write_params(plist, &pdev->downscale,
                                       GX_DOWNSCALER_PARAMS_MFS);
 
@@ -372,7 +372,7 @@ png_put_params_downscale_mfs(gx_device *dev, gs_param_list *plist)
 {
     gx_device_png *pdev = (gx_device_png *)dev;
     int code, ecode;
-    
+
     ecode = gx_downscaler_read_params(plist, &pdev->downscale,
                                       GX_DOWNSCALER_PARAMS_MFS);
 
@@ -611,6 +611,19 @@ do_png_print_page(gx_device_png * pdev, FILE * file, bool monod)
         png_set_PLTE(png_ptr, info_ptr, palettep, num_palette);
 
     png_set_text(png_ptr, info_ptr, &text_png, 1);
+
+    if (pdev->icc_struct != NULL && pdev->icc_struct->device_profile[0] != NULL) {
+        cmm_profile_t *icc_profile = pdev->icc_struct->device_profile[0];
+        /* PNG can only be RGB or gray.  No CIELAB :(  */
+        if (icc_profile->data_cs == gsRGB || icc_profile->data_cs == gsGRAY) {
+            if (icc_profile->num_comps == pdev->color_info.num_components &&
+                !(pdev->icc_struct->usefastcolor)) {
+                png_set_iCCP(png_ptr, info_ptr, icc_profile->name,
+                    PNG_COMPRESSION_TYPE_DEFAULT, icc_profile->buffer,
+                    icc_profile->buffer_size);
+            }
+        }
+    }
 #else
     info_ptr->bit_depth = bit_depth;
     info_ptr->color_type = color_type;
@@ -628,7 +641,7 @@ do_png_print_page(gx_device_png * pdev, FILE * file, bool monod)
     if (pdev->icc_struct != NULL && pdev->icc_struct->device_profile[0] != NULL) {
         cmm_profile_t *icc_profile = pdev->icc_struct->device_profile[0];
         /* PNG can only be RGB or gray.  No CIELAB :(  */
-        if (icc_profile->data_cs == gsRGB || icc_profile->data_cs == gsGRAY) { 
+        if (icc_profile->data_cs == gsRGB || icc_profile->data_cs == gsGRAY) {
             if (icc_profile->num_comps == pdev->color_info.num_components &&
                 !(pdev->icc_struct->usefastcolor)) {
                 info_ptr->iccp_name = icc_profile->name;
@@ -637,7 +650,7 @@ do_png_print_page(gx_device_png * pdev, FILE * file, bool monod)
                 info_ptr->valid |= PNG_INFO_iCCP;
             }
         }
-    } 
+    }
 #endif
     if (invert) {
         if (depth == 32)
@@ -845,9 +858,9 @@ pngalpha_fillpage(gx_device *dev, gs_gstate * pgs, gx_device_color *pdevc)
 
 /* Handle the RGBA planes from the PDF 1.4 compositor */
 static int
-pngalpha_put_image (gx_device *pdev, const byte *buffer, int num_chan, int xstart,
+pngalpha_put_image (gx_device *pdev, const byte **buffers, int num_chan, int xstart,
               int ystart, int width, int height, int row_stride,
-              int plane_stride, int alpha_plane_index, int tag_plane_index)
+              int alpha_plane_index, int tag_plane_index)
 {
     gx_device_memory *pmemdev = (gx_device_memory *)pdev;
     byte *buffer_prn;
@@ -856,9 +869,6 @@ pngalpha_put_image (gx_device *pdev, const byte *buffer, int num_chan, int xstar
     int x, y;
     int src_position, des_position;
 
-    /* Eventually, the pdf14 device might be chunky pixels, punt for now */
-    if (plane_stride == 0)
-        return 0;
     if (num_chan != 3 || alpha_plane_index <= 0)
             return_error(gs_error_unknownerror);        /* can't handle these cases */
 
@@ -873,11 +883,11 @@ pngalpha_put_image (gx_device *pdev, const byte *buffer, int num_chan, int xstar
         src_position = (y - ystart) * row_stride;
         des_position = y * pmemdev->raster + xstart * 4;
         for ( x = xstart; x < xend; x++ ) {
-            buffer_prn[des_position++] =  buffer[src_position];
-            buffer_prn[des_position++] =  buffer[src_position + plane_stride];
-            buffer_prn[des_position++] =  buffer[src_position + 2 * plane_stride];
+            buffer_prn[des_position++] =  buffers[0][src_position];
+            buffer_prn[des_position++] =  buffers[1][src_position];
+            buffer_prn[des_position++] =  buffers[2][src_position];
             /* Alpha data in low bits. Note that Alpha is inverted. */
-            buffer_prn[des_position++] = (255 - buffer[src_position + alpha_plane_index * plane_stride]);
+            buffer_prn[des_position++] = (255 - buffers[alpha_plane_index][src_position]);
             src_position += 1;
         }
     }
@@ -937,7 +947,7 @@ pngalpha_copy_alpha(gx_device * dev, const byte * data, int data_x,
             for (sx = data_x, rx = x; sx < data_x + width; ++sx, ++rx) {
                 gx_color_index previous = gx_no_color_index;
                 gx_color_index composite;
-                int alpha2, alpha;
+                uint32_t alpha2, alpha;
 
                 switch(depth)
                 {
@@ -970,8 +980,8 @@ pngalpha_copy_alpha(gx_device * dev, const byte * data, int data_x,
                     } else {	/* Blend values. */
                         gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
                         int i;
-                        int old_coverage;
-                        int new_coverage;
+                        uint32_t old_coverage;
+                        uint32_t new_coverage;
 
                         (*dev_proc(dev, decode_color)) (dev, previous, cv);
                         /* decode color doesn't give us coverage */
@@ -981,8 +991,8 @@ pngalpha_copy_alpha(gx_device * dev, const byte * data, int data_x,
                             (255 * alpha + old_coverage * (255 - alpha)) / 255;
                         for (i=0; i<ncomps; i++)
                             cv[i] = min(((255 * alpha * color_cv[i]) +
-                                (old_coverage * (255 - alpha ) * cv[i]))
-                                / (new_coverage * 255), gx_max_color_value);
+                                         (old_coverage * (255 - alpha ) * cv[i]))
+                                        / (new_coverage * 255), gx_max_color_value);
                         composite =
                             (*dev_proc(dev, encode_color)) (dev, cv);
                         /* encode color doesn't include coverage */
