@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -362,13 +362,17 @@ x_wrap_get_params(gx_device * dev, gs_param_list * plist)
 {
     gx_device *tdev;
     /* We assume that a get_params call has no side effects.... */
-    gx_device_X save_dev;
+    gx_device_X save_dev, *xdev;
     int ecode;
     int code;
 
     if ((code = get_dev_target(&tdev, dev)) < 0)
         return code;
-    save_dev = *(gx_device_X *) tdev;
+    xdev = (gx_device_X *) tdev;
+    xdev->orig_color_info = xdev->color_info;
+
+    save_dev = *xdev;
+
     if (tdev->is_open)
         tdev->color_info = dev->color_info;
     tdev->dname = dev->dname;
@@ -381,7 +385,7 @@ static int
 x_wrap_put_params(gx_device * dev, gs_param_list * plist)
 {
     gx_device *tdev;
-    gx_device_color_info cinfo;
+    gx_device_X *xdev;
     const char *dname;
     int rcode, code;
 
@@ -391,12 +395,13 @@ x_wrap_put_params(gx_device * dev, gs_param_list * plist)
      * put_params will choke if we simply feed it the output of
      * get_params; we have to substitute color_info the same way.
      */
-    cinfo = tdev->color_info;
+    xdev = (gx_device_X *) tdev;
+    xdev->orig_color_info = xdev->color_info;
     dname = tdev->dname;
     tdev->color_info = dev->color_info;
     tdev->dname = dev->dname;
     rcode = (*dev_proc(tdev, put_params)) (tdev, plist);
-    tdev->color_info = cinfo;
+    tdev->color_info = xdev->orig_color_info;
     tdev->dname = dname;
     if (rcode < 0)
         return rcode;
@@ -448,7 +453,6 @@ get_target_info(gx_device * dev)
     copy4(ImagingBBox);
     copy(ImagingBBox_set);
     copy2(HWResolution);
-    copy2(MarginsHWResolution);
     copy2(Margins);
     copy4(HWMargins);
     if (dev->color_info.num_components == 3) {
