@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -412,8 +412,13 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
                     cl = (float)1.5;
                 else if (cl < (float)1.65)
                     cl = (float)1.6;
-                else
+                else if (cl < (float)1.75)
                     cl = (float)1.7;
+                else {
+                    cl = (float)2.0;
+                    if (pdev->params.TransferFunctionInfo == tfi_Preserve)
+                        pdev->params.TransferFunctionInfo = tfi_Apply;
+                }
             }
         case 1:
             break;
@@ -553,6 +558,12 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
     ecode = gdev_psdf_put_params(dev, plist);
     if (ecode < 0)
         goto fail;
+
+    if (pdev->CompatibilityLevel > 1.7 && pdev->params.TransferFunctionInfo == tfi_Preserve) {
+        pdev->params.TransferFunctionInfo = tfi_Apply;
+        emprintf(pdev->memory, "\nIt is not possible to preserve transfer functions in PDF 2.0\ntransfer functions will be applied instead\n");
+    }
+
     if (pdev->params.ConvertCMYKImagesToRGB) {
         if (pdev->params.ColorConversionStrategy == ccs_CMYK) {
             emprintf(pdev->memory, "ConvertCMYKImagesToRGB is not compatible with ColorConversionStrategy of CMYK\n");
@@ -571,13 +582,16 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
         }
     }
     switch (pdev->params.ColorConversionStrategy) {
+        case ccs_ByObjectType:
         case ccs_LeaveColorUnchanged:
+            break;
         case ccs_UseDeviceDependentColor:
         case ccs_UseDeviceIndependentColor:
         case ccs_UseDeviceIndependentColorForImages:
-        case ccs_ByObjectType:
+            pdev->params.TransferFunctionInfo = tfi_Apply;
             break;
         case ccs_CMYK:
+            pdev->params.TransferFunctionInfo = tfi_Apply;
             if (pdev->icc_struct)
                 rc_decrement(pdev->icc_struct,
                              "reset default profile\n");
@@ -587,6 +601,7 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
                 goto fail;
             break;
         case ccs_Gray:
+            pdev->params.TransferFunctionInfo = tfi_Apply;
             if (pdev->icc_struct)
                 rc_decrement(pdev->icc_struct,
                              "reset default profile\n");
@@ -597,6 +612,7 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
             break;
         case ccs_sRGB:
         case ccs_RGB:
+            pdev->params.TransferFunctionInfo = tfi_Apply;
             /* Only bother to do this if we didn't handle it above */
             if (!pdev->params.ConvertCMYKImagesToRGB) {
                 if (pdev->icc_struct)
@@ -781,19 +797,19 @@ pdf_dsc_process(gx_device_pdf * pdev, const gs_param_string_array * pma)
          * but we do the same -- we ignore %%CreationDate here.
          */
 
-        if (pdf_key_eq(pkey, "Creator")) {
+        if (pdf_key_eq(pkey, "Creator") && pdev->CompatibilityLevel <= 1.7) {
             key = "/Creator";
             newsize = unescape_octals(pdev, (char *)pvalue->data, pvalue->size);
             code = cos_dict_put_c_key_string(pdev->Info, key,
                                              pvalue->data, newsize);
             continue;
-        } else if (pdf_key_eq(pkey, "Title")) {
+        } else if (pdf_key_eq(pkey, "Title") && pdev->CompatibilityLevel <= 1.7) {
             key = "/Title";
             newsize = unescape_octals(pdev, (char *)pvalue->data, pvalue->size);
             code = cos_dict_put_c_key_string(pdev->Info, key,
                                              pvalue->data, newsize);
             continue;
-        } else if (pdf_key_eq(pkey, "For")) {
+        } else if (pdf_key_eq(pkey, "For") && pdev->CompatibilityLevel <= 1.7) {
             key = "/Author";
             newsize = unescape_octals(pdev, (char *)pvalue->data, pvalue->size);
             code = cos_dict_put_c_key_string(pdev->Info, key,
