@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -601,7 +601,6 @@ struct gx_device_pdf_s {
                               a bitmap representation of a shading.
                               (Bigger shadings to be downsampled). */
     long MaxInlineImageSize;
-    gs_param_int_array DSCEncodingToUnicode;
     /* Encryption parameters */
     gs_param_string OwnerPassword;
     gs_param_string UserPassword;
@@ -884,6 +883,16 @@ struct gx_device_pdf_s {
                                      * used, the content of ths string is written out as part of the
                                      * metadata referenced by the Catalog.
                                      */
+    char *PDFFormName;              /* If present, we are processing (or about to process) a Form XObject
+                                     * which we wish to handle as a Form XObject, not flatten. Currently
+                                     * this is only the case for Annotation Appearances. This must be NULL
+                                     * after the form is processed. The name will be used to create a
+                                     * local named object which pdfmark can reference.
+                                     */
+    stream *PassThroughWriter;      /* A copy of the stream that the image enumerator points to, if we are
+                                     * doing JPEG pass through we write the JPEG data here, and don't write
+                                     * anything in the image processing routines.
+                                     */
 };
 
 #define is_in_page(pdev)\
@@ -911,14 +920,14 @@ struct gx_device_pdf_s {
  m(28,sbstack) m(29,substream_Resources) m(30,font3)\
  m(31,accumulating_substream_resource) \
  m(32,pres_soft_mask_dict) m(33,PDFXTrimBoxToMediaBoxOffset.data)\
- m(34,PDFXBleedBoxToTrimBoxOffset.data) m(35, DSCEncodingToUnicode.data)\
- m(36,Identity_ToUnicode_CMaps[0]) m(37,Identity_ToUnicode_CMaps[1])\
- m(38,vgstack)\
- m(39, outline_levels)
- m(40, gx_device_pdf, EmbeddedFiles);
- m(41, gx_device_pdf, pdf_font_dir);
- m(42, gx_device_pdf, Extension_Metadata);*/
-#define gx_device_pdf_num_ptrs 43
+ m(34,PDFXBleedBoxToTrimBoxOffset.data)
+ m(35,Identity_ToUnicode_CMaps[0]) m(36,Identity_ToUnicode_CMaps[1])\
+ m(37,vgstack)\
+ m(38, outline_levels)
+ m(39, gx_device_pdf, EmbeddedFiles);
+ m(40, gx_device_pdf, pdf_font_dir);
+ m(41, gx_device_pdf, Extension_Metadata);*/
+#define gx_device_pdf_num_ptrs 42
 #define gx_device_pdf_do_param_strings(m)\
     m(0, OwnerPassword) m(1, UserPassword) m(2, NoEncrypt)\
     m(3, DocumentUUID) m(4, InstanceUUID)
@@ -1075,6 +1084,13 @@ pdf_resource_t *pdf_find_resource_by_gs_id(gx_device_pdf * pdev,
                                            pdf_resource_type_t rtype,
                                            gs_id rid);
 
+/* Remove a resource from a chain of resources but do not free it. The resource will
+ * have to be listed elsewhere. This is primarily useful for moving existing resources
+ * to local named resources.
+ */
+void
+pdf_drop_resource_from_chain(gx_device_pdf * pdev, pdf_resource_t *pres1, pdf_resource_type_t rtype);
+
 void pdf_drop_resources(gx_device_pdf * pdev, pdf_resource_type_t rtype,
         int (*cond)(gx_device_pdf * pdev, pdf_resource_t *pres));
 
@@ -1308,6 +1324,11 @@ int pdf_function_scaled(gx_device_pdf *pdev, const gs_function_t *pfn,
 int pdf_write_function(gx_device_pdf *pdev, const gs_function_t *pfn,
                        long *pid);
 
+/* If a stitching function references an array of other functions, we need
+ * to 'unreference' those before freeing the function. otherwise we end up
+ * trying to free the referenced functions twice.
+ */
+int free_function_refs(gx_device_pdf *pdev, cos_object_t *pco);
 /* ------ Fonts ------ */
 
 /* Write a FontBBox dictionary element. */
