@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -684,10 +684,17 @@ scan_cmap_text(pdf_text_enum_t *pte, void *vbuf)
                         w0[cid] = widths.Width.w;
                     }
                     if (pdsubf->u.cidfont.CIDToGIDMap != 0) {
+                        uint gid = 0;
                         gs_font_cid2 *subfont2 = (gs_font_cid2 *)subfont;
 
-                        pdsubf->u.cidfont.CIDToGIDMap[cid] =
-                        subfont2->cidata.CIDMap_proc(subfont2, glyph);
+                        gid = subfont2->cidata.CIDMap_proc(subfont2, glyph);
+
+                        /* If this is a TrueType CIDFont, check the GSUB table to see if there's
+                         * a suitable substitute glyph.
+                         */
+                        if (subfont2->FontType == ft_CID_TrueType)
+                            gid = subfont2->data.substitute_glyph_index_vertical((gs_font_type42 *)subfont, gid, subfont2->WMode, glyph);
+                        pdsubf->u.cidfont.CIDToGIDMap[cid] = gid;
                     }
                 }
                 if (wmode)
@@ -880,6 +887,7 @@ process_cmap_text(gs_text_enum_t *penum, void *vbuf, uint bsize)
     int code;
     pdf_text_enum_t *pte = (pdf_text_enum_t *)penum;
     byte *save;
+    uint start = pte->index;
 
     if (pte->text.operation &
         (TEXT_FROM_ANY - (TEXT_FROM_STRING | TEXT_FROM_BYTES))
@@ -906,6 +914,7 @@ process_cmap_text(gs_text_enum_t *penum, void *vbuf, uint bsize)
     code = scan_cmap_text(pte, vbuf);
     gs_free_string(pte->memory, (byte *)pte->text.data.bytes,  pte->text.size, "pdf_text_process");
     pte->text.data.bytes = save;
+    pte->bytes_decoded = pte->index - start;
 
     if (code == TEXT_PROCESS_CDEVPROC)
         pte->cdevproc_callout = true;
