@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -153,7 +153,7 @@ pdf_add_ToUnicode(gx_device_pdf *pdev, gs_font *font, pdf_font_resource_t *pdfon
         }
 
         if (!unicode) {
-            unicode = (ushort *)gs_alloc_bytes(pdev->memory, length, "temporary Unicode array");
+            unicode = (ushort *)gs_alloc_bytes(pdev->memory, length * sizeof(short), "temporary Unicode array");
             length = font->procs.decode_glyph((gs_font *)font, glyph, ch, unicode, length);
         }
 
@@ -533,11 +533,12 @@ pdf_process_string(pdf_text_enum_t *penum, gs_string *pstr,
     gx_device_pdf *const pdev = (gx_device_pdf *)penum->dev;
     gs_font_base *font = (gs_font_base *)penum->current_font;
     pdf_font_resource_t *pdfont;
-    const gs_text_params_t *text = &penum->text;
+    gs_text_params_t *text = &penum->text;
     int code = 0, mask;
     gs_point width_pt;
     int accepted;
     gs_rect text_bbox = {{0, 0}, {0, 0}}, glyphs_bbox = {{10000,10000}, {0,0}};
+    unsigned int operation = text->operation;
 
     code = pdf_obtain_font_resource(penum, pstr, &pdfont);
     if (code < 0)
@@ -554,7 +555,7 @@ pdf_process_string(pdf_text_enum_t *penum, gs_string *pstr,
     if (penum->pgs->text_rendering_mode != 3 && !(text->operation & TEXT_DO_NONE)) {
         /*
          * Acrobat Reader can't handle text with huge coordinates,
-         * so skip the text if it is outside the clip bbox
+         * so don't emit the text if it is outside the clip bbox
          * (Note : it ever fails with type 3 fonts).
          */
 
@@ -573,7 +574,7 @@ pdf_process_string(pdf_text_enum_t *penum, gs_string *pstr,
                 rect_intersect(rect, text_bbox);
                 if (rect.p.x > rect.q.x || rect.p.y > rect.q.y) {
                     penum->index += pstr->size;
-                    goto finish;
+                    text->operation &= ~TEXT_DO_DRAW;
                 }
             }
         } else {
@@ -583,7 +584,7 @@ pdf_process_string(pdf_text_enum_t *penum, gs_string *pstr,
 
             code = gx_path_current_point(penum->path, &origin);
             if (code < 0)
-                return code;
+                goto done;
 
             m = ctm_only(penum->pgs);
             m.tx = fixed2float(origin.x);
@@ -630,7 +631,7 @@ pdf_process_string(pdf_text_enum_t *penum, gs_string *pstr,
         }
     }
     if (code < 0)
-        return code;
+        goto done;
     mask = code;
 
     if (text->operation & TEXT_REPLACE_WIDTHS)
@@ -646,20 +647,22 @@ pdf_process_string(pdf_text_enum_t *penum, gs_string *pstr,
          * the string character-by-character.  process_text_return_width
          * will tell us what we need to know.
          */
-        if (!(text->operation & (TEXT_DO_DRAW | TEXT_RETURN_WIDTH)))
-            return 0;
+        if (!(text->operation & (TEXT_DO_DRAW | TEXT_RETURN_WIDTH))) {
+            code = 0;
+            goto done;
+        }
         code = process_text_return_width(penum, font, ppts,
                                          (gs_const_string *)pstr, gdata,
                                          &width_pt, &accepted, &glyphs_bbox);
         if (code < 0)
-            return code;
+            goto done;
         if (code == 0) {
             /* No characters with redefined widths -- the fast case. */
             if (text->operation & TEXT_DO_DRAW || penum->pgs->text_rendering_mode == 3) {
                 code = pdf_append_chars(pdev, pstr->data, accepted,
                                         width_pt.x, width_pt.y, false);
                 if (code < 0)
-                    return code;
+                    goto done;
                 adjust_first_last_char(pdfont, pstr->data, accepted);
                 penum->index += accepted;
             } else if (text->operation & TEXT_DO_NONE)
@@ -709,10 +712,9 @@ pdf_process_string(pdf_text_enum_t *penum, gs_string *pstr,
         penum->text = text;
         penum->index += index0;
         if (code < 0)
-            return code;
+            goto done;
     }
 
-finish:
     /* Finally, return the total width if requested. */
     if (pdev->Eps2Write && penum->pcpath) {
         gx_device_clip cdev;
@@ -725,6 +727,9 @@ finish:
             gs_point p0, p1, p2, p3;
 
             code = gx_path_current_point(penum->path, &origin);
+            if (code < 0)
+                return code;
+
             m = ctm_only(penum->pgs);
             m.tx = fixed2float(origin.x);
             m.ty = fixed2float(origin.y);
@@ -770,9 +775,11 @@ finish:
                                  bx2, by2, &devc, lop_default);
         pdev->AccumulatingBBox--;
     }
-    if (!(text->operation & TEXT_RETURN_WIDTH))
-        return 0;
-    if (text->operation & TEXT_DO_NONE) {
+    if (!(operation & TEXT_RETURN_WIDTH)) {
+        code = 0;
+        goto done;
+    }
+    if (operation & TEXT_DO_NONE) {
         /* stringwidth needs to transform to user space. */
         gs_point p;
 
@@ -781,7 +788,11 @@ finish:
         penum->returned.total_width.y += p.y;
     } else
         penum->returned.total_width = width_pt;
-    return pdf_shift_text_currentpoint(penum, &width_pt);
+    code = pdf_shift_text_currentpoint(penum, &width_pt);
+
+done:
+    text->operation = operation;
+    return code;
 }
 
 /*
@@ -1300,11 +1311,24 @@ process_text_modify_width(pdf_text_enum_t *pte, gs_font *font,
         if (pte->text.operation & TEXT_REPLACE_WIDTHS) {
             gs_point dpt;
 
+            /* We are applying a width override, from x/y/xyshow. This coudl be from
+             * a PostScript file, or it could be from a PDF file where we have a font
+             * with a FontMatrix which is neither horizontal nor vertical. If we use TJ
+             * for that, then we end up applying the displacement twice, once here where
+             * we add a TJ, and once when we actually draw the glyph (TJ is added *after*
+             * the glyph is drawn, unlike xshow). So in this case we don't want to try
+             * and use a TJ, we need to position the glyphs using text positioning
+             * operators.
+             */
+            if(cw.Width.xy.x != cw.real_width.xy.x || cw.Width.xy.y != cw.real_width.xy.y)
+                pdev->text->text_state->can_use_TJ = false;
+
             code = gs_text_replaced_width(&pte->text, pte->xy_index++, &dpt);
             if (code < 0)
                 return_error(gs_error_unregistered);
             gs_distance_transform(dpt.x, dpt.y, &ctm_only(pte->pgs), &wanted);
         } else {
+            pdev->text->text_state->can_use_TJ = true;
             gs_distance_transform(cw.real_width.xy.x * ppts->values.size,
                                   cw.real_width.xy.y * ppts->values.size,
                                   &ppts->values.matrix, &wanted);
