@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -26,6 +26,7 @@
 #include "gscspace.h"
 #include "gxdevcli.h"
 #include "gxgstate.h"
+#include "gsicc_manage.h"
 
 /* ---------------- Convert between 1/2/4/12 and 8 bits ---------------- */
 
@@ -1160,7 +1161,7 @@ s_compr_chooser__get_choice(stream_compr_chooser_state *ss, bool force)
     return 0;
 }
 
-/* ---------------- Am image color conversion filter ---------------- */
+/* ---------------- An image color conversion filter ---------------- */
 
 private_st_image_colors_state();
 
@@ -1231,7 +1232,7 @@ s_image_colors_convert_to_device_color(stream_image_colors_state * ss)
     return 0;
 }
 
-/* Set masc colors dimensions. */
+/* Set mask colors dimensions. */
 void
 s_image_colors_set_mask_colors(stream_image_colors_state * ss, uint *MaskColor)
 {
@@ -1268,21 +1269,6 @@ s_image_colors_set_color_space(stream_image_colors_state * ss, gx_device *pdev,
     memcpy(ss->Decode, Decode, ss->depth * sizeof(Decode[0]) * 2);
 }
 
-void
-s_new_image_colors_set_color_space(stream_image_colors_state * ss, gx_device *pdev,
-                               const gs_color_space *pcs, const gs_gstate *pgs,
-                               float *Decode)
-{
-    ss->output_depth = pdev->color_info.num_components;
-    ss->output_component_index = ss->output_depth;
-    ss->output_bits_per_sample = pdev->color_info.comp_bits[0]; /* Same precision for all components. */
-    ss->convert_color = s_image_colors_convert_to_device_color;
-    ss->pdev = pdev;
-    ss->pcs = pcs;
-    ss->pgs = pgs;
-    memcpy(ss->Decode, Decode, ss->depth * sizeof(Decode[0]) * 2);
-}
-
 /* Process a buffer. */
 static int
 s_image_colors_process(stream_state * st, stream_cursor_read * pr,
