@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -1580,6 +1580,9 @@ psf_write_cid0_font(stream *s, gs_font_cid0 *pfont, int options,
     int num_fonts = pfont->cidata.FDArray_size;
     int code;
 
+    memset(&subrs_count, 0x00, 256 * sizeof(uint));
+    memset(&subrs_size, 0x00, 256 * sizeof(uint));
+
     /* Initialize the enumeration of the glyphs. */
     psf_enumerate_cids_begin(&genum, (gs_font *)pfont, subset_cids,
                              subset_size);
