@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -1466,7 +1466,7 @@ pclxl_lineto(gx_device_vector * vdev, double x0, double y0, double x,
 {
     gx_device_pclxl *const xdev = (gx_device_pclxl *) vdev;
 
-    if (xdev->points.type != POINTS_LINES || xdev->points.count >= NUM_POINTS) {
+    if (xdev->points.type != POINTS_LINES || xdev->points.count >= NUM_POINTS - 2) {
         if (xdev->points.type != POINTS_NONE) {
             int code = pclxl_flush_points(xdev);
 
@@ -1476,6 +1476,12 @@ pclxl_lineto(gx_device_vector * vdev, double x0, double y0, double x,
         xdev->points.current.x = (int)(x0 + 0.5);
         xdev->points.current.y = (int)(y0 + 0.5);
         xdev->points.type = POINTS_LINES;
+
+        /* This can only happen if we get two 'POINTS_NONE' in a row, in which case
+         * just overwrite the previous one below.
+         */
+        if (xdev->points.count >= NUM_POINTS - 1)
+            xdev->points.count--;
     }
     {
         gs_int_point *ppt = &xdev->points.data[xdev->points.count++];
@@ -1493,7 +1499,7 @@ pclxl_curveto(gx_device_vector * vdev, double x0, double y0,
     gx_device_pclxl *const xdev = (gx_device_pclxl *) vdev;
 
     if (xdev->points.type != POINTS_CURVES ||
-        xdev->points.count >= NUM_POINTS - 2) {
+        xdev->points.count >= NUM_POINTS - 4) {
         if (xdev->points.type != POINTS_NONE) {
             int code = pclxl_flush_points(xdev);
 
@@ -1503,6 +1509,12 @@ pclxl_curveto(gx_device_vector * vdev, double x0, double y0,
         xdev->points.current.x = (int)(x0 + 0.5);
         xdev->points.current.y = (int)(y0 + 0.5);
         xdev->points.type = POINTS_CURVES;
+
+        /* This can only happen if we get multiple 'POINTS_NONE' in a row, in which case
+         * just overwrite the previous one below.
+         */
+        if (xdev->points.count >= NUM_POINTS - 3)
+            xdev->points.count -= 3;
     }
     {
         gs_int_point *ppt = &xdev->points.data[xdev->points.count];
@@ -1607,7 +1619,7 @@ pclxl_open_device(gx_device * dev)
     gx_device_pclxl *xdev = (gx_device_pclxl *) dev;
     int code;
 
-    vdev->v_memory = dev->memory;       /****** WRONG ******/
+    vdev->v_memory = dev->memory->stable_memory;
     vdev->vec_procs = &pclxl_vector_procs;
     code = gdev_vector_open_file_options(vdev, 512,
                                          VECTOR_OPEN_FILE_SEQUENTIAL);
@@ -1693,7 +1705,7 @@ pclxl_copy_mono(gx_device * dev, const byte * data, int data_x, int raster,
     int code;
     stream *s;
     gx_color_index color0 = zero, color1 = one;
-    gx_color_index white = (1 << dev->color_info.depth) - 1;
+    gx_color_index white = ((gx_color_index)1 << dev->color_info.depth) - 1;
     gx_color_index black = 0;
     gs_logical_operation_t lop;
     byte palette[2 * 3];
@@ -1923,7 +1935,7 @@ pclxl_fill_mask(gx_device * dev,
     /* This is similiar to the copy_mono white-on-mask,
      * except we are drawing white on the black of a black/white mask,
      * so we invert source, compared to copy_mono */
-    if (foreground == (1 << dev->color_info.depth) - 1) {       /* white */
+    if (foreground == ((gx_color_index)1 << dev->color_info.depth) - 1) {       /* white */
         lop = rop3_not(rop3_S) | (rop3_D & rop3_S);
     } else if (foreground == 0) {       /* black */
         lop = (rop3_S & rop3_D);
@@ -2227,7 +2239,7 @@ pclxl_begin_image(gx_device * dev,
             /* This is similiar to the copy_mono white-on-mask,
              * except we are drawing white on the black of a black/white mask,
              * so we invert source, compared to copy_mono */
-            if (foreground == (1 << dev->color_info.depth) - 1) {       /* white */
+            if (foreground == ((gx_color_index)1 << dev->color_info.depth) - 1) {       /* white */
                 lop = rop3_not(rop3_S) | (rop3_D & rop3_S);
             } else if (foreground == 0) {       /* black */
                 lop = (rop3_S & rop3_D);
@@ -2525,7 +2537,7 @@ pclxl_image_plane_data(gx_image_enum_common_t * info,
                                                   planes[0].raster * i +
                                                   (data_bit >> 3)) /*src */ ,
                                          pie->rows.data + out_raster_stride * flipped_strip_offset      /*des */
-                );
+                                         );
         }
     }
     *rows_used = height;
@@ -2609,8 +2621,8 @@ pclxl_get_params(gx_device * dev,       /* I - Device info */
 				&(xdev->MediaPosition))) < 0)
         return (code);
 
-    if ((code = param_string_from_string(s, xdev->MediaType)) < 0)
-        return (code);
+    param_string_from_string(s, xdev->MediaType);
+
     if ((code = param_write_string(plist, "MediaType", &s)) < 0)
         return (code);
 
