@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /*$Id: gdevtxtw.c 7795 2007-03-23 13:56:11Z tim $ */
@@ -596,8 +596,17 @@ static int simple_text_output(gx_device_txtwrite_t *tdev)
         x_entry = y_list->x_ordered_list;
         while (x_entry) {
             width = (x_entry->end.x - x_entry->start.x) / x_entry->Unicode_Text_Size;
-            if (width < min_width_size && width >= (float)min_size * 0.75)
-                min_width_size = width;
+            if (x_entry->next) {
+                /* If we have following text, check to see if *not* using the newly calculated size would result in
+                 * the end of the text going past the beginning of the following text. If it does then we must
+                 * use the new minimum, regardless of how small it is!
+                 */
+                if (x_entry->start.x + ((x_entry->Unicode_Text_Size + 1) * min_width_size) > x_entry->next->start.x && width > 0)
+                    min_width_size = width;
+            } else {
+                if (width < min_width_size && width >= (float)min_size * 0.75)
+                    min_width_size = width;
+            }
             x_entry = x_entry->next;
         }
         y_list = y_list->next;
@@ -1695,9 +1704,8 @@ static int get_unicode(textw_text_enum_t *penum, gs_font *font, gs_glyph glyph,
                 char *d3 = strchr(hexdigits, gnstr.data[6]);
 
                 if (d0 != NULL && d1 != NULL && d2 != NULL && d3 != NULL) {
-                    *Buffer++ = ((d0 - hexdigits) << 8) + ((d1 - hexdigits));
-                    *Buffer++ = ((d2 - hexdigits) << 8) + ((d3 - hexdigits));
-                    return 2;
+                    *Buffer++ = ((d0 - hexdigits) << 12) + ((d1 - hexdigits) << 8) + ((d2 - hexdigits) << 4) + (d3 - hexdigits);
+                    return 1;
                 }
             }
         }
@@ -1782,9 +1790,22 @@ static int get_unicode(textw_text_enum_t *penum, gs_font *font, gs_glyph glyph,
         *Buffer = fallback;
         return 1;
     } else {
+        char *b, *u;
+        int l = length - 1;
+
         unicode = (ushort *)gs_alloc_bytes(penum->dev->memory, length, "temporary Unicode array");
         length = font->procs.decode_glyph((gs_font *)font, glyph, ch, unicode, length);
+#if ARCH_IS_BIG_ENDIAN
         memcpy(Buffer, unicode, length);
+#else
+        b = (char *)Buffer;
+        u = (char *)unicode;
+        while (l >= 0) {
+            *b++ = *(u + l);
+            l--;
+        }
+
+#endif
         gs_free_object(penum->dev->memory, unicode, "free temporary unicode buffer");
         return length / sizeof(short);
     }
@@ -1838,7 +1859,11 @@ txtwrite_process_cmap_text(gs_text_enum_t *pte)
                     pte->index = scan.index;
                 }
                 code = gs_matrix_multiply(&subfont->FontMatrix, &pte->orig_font->FontMatrix, &m3);
+                if (code < 0)
+                    return code;
                 code = txt_update_text_state(penum->text_state, (textw_text_enum_t *)pte, pte->orig_font, &m3);
+                if (code < 0)
+                    return code;
                 txt_char_widths_to_uts(pte->orig_font, &widths); /* convert design->text space */
                 gs_distance_transform(widths.real_width.xy.x * penum->text_state->size,
                           widths.real_width.xy.y * penum->text_state->size,
