@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* prevent gp.h redefining sprintf */
@@ -408,7 +408,7 @@ int display_presize(void *handle, void *device, int width, int height,
 {
     IMAGE *img;
 #ifdef DISPLAY_DEBUG
-    fprintf(stdout, "display_presize(0x%x 0x%x, %d, %d, %d, %d, %ld)\n",
+    fprintf(stdout, "display_presize(0x%x 0x%x, %d, %d, %d, %ld)\n",
         handle, device, width, height, raster, format);
 #endif
     img = image_find(handle, device);
@@ -424,7 +424,7 @@ int display_size(void *handle, void *device, int width, int height,
 {
     IMAGE *img;
 #ifdef DISPLAY_DEBUG
-    fprintf(stdout, "display_size(0x%x 0x%x, %d, %d, %d, %d, %ld, 0x%x)\n",
+    fprintf(stdout, "display_size(0x%x 0x%x, %d, %d, %d, %ld, 0x%x)\n",
         handle, device, width, height, raster, format, pimage);
 #endif
     img = image_find(handle, device);
@@ -663,14 +663,14 @@ static int main_utf8(int argc, char *argv[])
         sprintf(dformat, "-dDisplayFormat=%d", format);
     }
     nargc = argc + 2;
-    nargv = (char **)malloc((nargc + 1) * sizeof(char *));
+    nargv = (char **)malloc(nargc * sizeof(char *));
     if (nargv == NULL) {
         fprintf(stderr, "Malloc failure!\n");
     } else {
         nargv[0] = argv[0];
         nargv[1] = dformat;
         nargv[2] = ddpi;
-        memcpy(&nargv[3], &argv[1], argc * sizeof(char *));
+        memcpy(&nargv[3], &argv[1], (argc-1) * sizeof(char *));
 #endif
 
 #if defined(_MSC_VER) || defined(__BORLANDC__)
