@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -87,7 +87,7 @@ display_set_callback(gs_main_instance *minst, display_callback *callback)
         if (was_open) {
             code = gs_closedevice(dev);
             if (code < 0)
-                return_error(code);
+                return code;
         }
 
         ddev = (gx_device_display *) dev;
@@ -96,8 +96,8 @@ display_set_callback(gs_main_instance *minst, display_callback *callback)
         if (was_open) {
             code = gs_opendevice(dev);
             if (code < 0) {
-                dprintf("**** Unable to open the display device, quitting.\n");
-                return_error(code);
+                dmprintf(dev->memory, "**** Unable to open the display device, quitting.\n");
+                return code;
             }
         }
         pop(1);	/* device */
