@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -71,12 +71,7 @@
 gs_main_instance*
 get_minst_from_memory(const gs_memory_t *mem)
 {
-#ifdef PSI_INCLUDED
-    extern gs_main_instance *ps_impl_get_minst( const gs_memory_t *mem );
-    return ps_impl_get_minst(mem);
-#else
     return (gs_main_instance*)mem->gs_lib_ctx->top_of_system;
-#endif
 }
 
 /** construct main instance caller needs to retain */
@@ -94,12 +89,7 @@ gs_main_alloc_instance(gs_memory_t *mem)
         return NULL;
     memcpy(minst, &gs_main_instance_init_values, sizeof(gs_main_instance_init_values));
     minst->heap = mem;
-
-#   ifndef PSI_INCLUDED
     mem->gs_lib_ctx->top_of_system = minst;
-    /* else top of system is pl_universe */
-#   endif
-
     return minst;
 }
 
@@ -146,16 +136,14 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
     gp_init();
 
     /* Initialize the imager. */
-#   ifndef PSI_INCLUDED
+
     /* Reset debugging flags */
 #ifdef PACIFY_VALGRIND
     VALGRIND_HG_DISABLE_CHECKING(gs_debug, 128);
 #endif
     memset(gs_debug, 0, 128);
     gs_log_errors = 0;  /* gs_debug['#'] = 0 */
-#   else
-    /* plmain settings remain in effect */
-#   endif
+
     gp_get_realtime(minst->base_time);
 
     /* Initialize the file search paths. */
@@ -199,10 +187,12 @@ gs_main_init1(gs_main_instance * minst)
             gs_memory_t *mem = (gs_memory_t *)idmem.space_system;
             nt = names_init(minst->name_table_size, idmem.space_system);
 
-            if (nt == 0)
-                return_error(gs_error_VMerror);
+            if (nt == 0) {
+                code = gs_note_error(gs_error_VMerror);
+                goto fail;
+            }
             mem->gs_lib_ctx->gs_name_table = nt;
-            code = gs_register_struct_root(mem, NULL,
+            code = gs_register_struct_root(mem, mem->gs_lib_ctx->name_table_root,
                                            (void **)&mem->gs_lib_ctx->gs_name_table,
                                            "the_gs_name_table");
             if (code < 0)
@@ -212,18 +202,20 @@ gs_main_init1(gs_main_instance * minst)
         code = obj_init(&minst->i_ctx_p, &idmem);  /* requires name_init */
         if (code < 0)
             goto fail;
+        minst->init_done = 1;
         code = i_plugin_init(minst->i_ctx_p);
         if (code < 0)
             goto fail;
-        code = i_iodev_init(minst->i_ctx_p);
+        code = i_iodev_init(&idmem);
         if (code < 0)
             goto fail;
-        minst->init_done = 1;
     }
     return 0;
 
 fail:
     names_free(nt);
+    if (minst->i_ctx_p == NULL)
+        ialloc_finit(&idmem);
     return code;
 }
 
@@ -299,7 +291,6 @@ int gs_main_init2aux(gs_main_instance * minst) {
         if ((code = display_set_callback(minst, minst->display)) < 0)
             return code;
 
-#ifndef PSI_INCLUDED
         if ((code = gs_main_run_string(minst,
                 "JOBSERVER "
                 " { false 0 .startnewjob } "
@@ -307,7 +298,6 @@ int gs_main_init2aux(gs_main_instance * minst) {
                 "ifelse", 0, &exit_code,
                 &error_object)) < 0)
            return code;
-#endif /* PSI_INCLUDED */
     }
     return 0;
 }
@@ -321,7 +311,7 @@ gs_main_init2(gs_main_instance * minst)
 
     if (code < 0)
         return code;
-    i_ctx_p = minst->i_ctx_p;
+
     code = gs_main_init2aux(minst);
     if (code < 0)
        return code;
@@ -333,7 +323,7 @@ gs_main_init2(gs_main_instance * minst)
        gx_device_printer *ppdev = (gx_device_printer *)pdev;
 
         if (minst->saved_pages_test_mode) {
-            if ((dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) == 0)) {
+            if ((dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) <= 0)) {
                 /* no warning or error if saved-pages-test mode is used, just disable it */
                 minst->saved_pages_test_mode = false;  /* device doesn't support it */
             } else {
@@ -344,7 +334,7 @@ gs_main_init2(gs_main_instance * minst)
                         return code;
             }
         } else if (minst->saved_pages_initial_arg != NULL) {
-            if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) == 0) {
+            if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) <= 0) {
                 outprintf(minst->heap,
                           "   --saved-pages not supported by the '%s' device.\n",
                           pdev->dname);
@@ -874,6 +864,7 @@ int
 gs_main_finit(gs_main_instance * minst, int exit_status, int code)
 {
     i_ctx_t *i_ctx_p = minst->i_ctx_p;
+    gs_dual_memory_t dmem = {0};
     int exit_code;
     ref error_object;
     char *tempnames;
@@ -896,7 +887,6 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
      */
     gs_finit_push_systemdict(i_ctx_p);
 
-#ifndef PSI_INCLUDED
     /* We have to disable BGPrint before we call interp_reclaim() to prevent the
      * parent rendering thread initialising for the next page, whilst we are
      * removing objects it may want to access - for example, the I/O device table.
@@ -912,7 +902,6 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
               .systemvar exec",
             0 , &exit_code, &error_object);
     }
-#endif
 
     /*
      * Close the "main" device, because it may need to write out
@@ -938,11 +927,15 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
                 else {
                     emprintf1(imemory, "UNKNOWN ERROR %d reclaiming the memory while the interpreter finalization.\n", code);
                 }
+#ifdef MEMENTO_SQUEEZE_BUILD
+                if (code != gs_error_VMerror ) return gs_error_Fatal;
+#else
                 return gs_error_Fatal;
+#endif
             }
             i_ctx_p = minst->i_ctx_p; /* interp_reclaim could change it. */
         }
-#ifndef PSI_INCLUDED
+
         if (i_ctx_p->pgs != NULL && i_ctx_p->pgs->device != NULL) {
             gx_device *pdev = i_ctx_p->pgs->device;
             const char * dname = pdev->dname;
@@ -972,7 +965,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
             if (exit_status == 0 || exit_status == gs_error_Quit)
                 exit_status = code;
         }
-#endif
+
       /* Flush stdout and stderr */
       gs_main_run_string(minst,
         "(%stdout) (w) file closefile (%stderr) (w) file closefile \
@@ -991,14 +984,17 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
     if (minst->init_done >= 1) {
         gs_memory_t *mem_raw = i_ctx_p->memory.current->non_gc_memory;
         i_plugin_holder *h = i_ctx_p->plugin_list;
+
+        dmem = *idmemory;
         code = alloc_restore_all(i_ctx_p);
         if (code < 0)
             emprintf1(mem_raw,
                       "ERROR %d while the final restore. See gs/psi/ierrors.h for code explanation.\n",
                       code);
+        i_iodev_finit(&dmem);
         i_plugin_finit(mem_raw, h);
     }
-#ifndef PSI_INCLUDED
+
     /* clean up redirected stdout */
     if (minst->heap->gs_lib_ctx->fstdout2
         && (minst->heap->gs_lib_ctx->fstdout2 != minst->heap->gs_lib_ctx->fstdout)
@@ -1006,7 +1002,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
         fclose(minst->heap->gs_lib_ctx->fstdout2);
         minst->heap->gs_lib_ctx->fstdout2 = (FILE *)NULL;
     }
-#endif
+
     minst->heap->gs_lib_ctx->stdout_is_redirected = 0;
     minst->heap->gs_lib_ctx->stdout_to_stderr = 0;
     /* remove any temporary files, after ghostscript has closed files */
@@ -1018,10 +1014,10 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
         }
         free(tempnames);
     }
-#ifndef PSI_INCLUDED
     gs_lib_finit(exit_status, code, minst->heap);
-#endif
+
     gs_free_object(minst->heap, minst->lib_path.container.value.refs, "lib_path array");
+    ialloc_finit(&dmem);
     return exit_status;
 }
 int
