@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -240,9 +240,18 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
                     return code;
                 if (minst->saved_pages_test_mode) {
                     gx_device *pdev;
+                    int ret;
+                    gxdso_device_child_request child_dev_data;
 
-                    /* get the current device */
+                    /* get the real target (printer) device */
                     pdev = gs_currentdevice(minst->i_ctx_p->pgs);
+                    do {
+                        child_dev_data.target = pdev;
+                        ret = dev_proc(pdev, dev_spec_op)(pdev, gxdso_device_child, &child_dev_data,
+                                                          sizeof(child_dev_data));
+                        if (ret > 0)
+                            pdev = child_dev_data.target;
+                    } while ((ret > 0) && (child_dev_data.n != 0));
                     if ((code = gx_saved_pages_param_process((gx_device_printer *)pdev,
                                (byte *)"print normal flush", 18)) < 0)
                         return code;
@@ -331,11 +340,21 @@ run_stdin:
             code = run_string(minst, ".runstdin", runFlush);
             if (code < 0)
                 return code;
+            /* If in saved_pages_test_mode, print and flush previous job before the next file */
             if (minst->saved_pages_test_mode) {
                 gx_device *pdev;
+                int ret;
+                gxdso_device_child_request child_dev_data;
 
-                /* get the current device */
+                /* get the real target (printer) device */
                 pdev = gs_currentdevice(minst->i_ctx_p->pgs);
+                do {
+                    child_dev_data.target = pdev;
+                    ret = dev_proc(pdev, dev_spec_op)(pdev, gxdso_device_child, &child_dev_data,
+                                                      sizeof(child_dev_data));
+                    if (ret > 0)
+                        pdev = child_dev_data.target;
+                } while ((ret > 0) && (child_dev_data.n != 0));
                 if ((code = gx_saved_pages_param_process((gx_device_printer *)pdev,
                            (byte *)"print normal flush", 18)) < 0)
                     return code;
@@ -352,7 +371,6 @@ run_stdin:
                 break;
             } else if (strncmp(arg, "saved-pages=", 12) == 0) {
                 gx_device *pdev;
-                gx_device_printer *ppdev;
 
                 /* If init2 not yet done, just save the argument for processing then */
                 if (minst->init_done < 2) {
@@ -366,17 +384,29 @@ run_stdin:
                         return gs_error_Fatal;
                     }
                 } else {
+                    int ret;
+                    gxdso_device_child_request child_dev_data;
+
                     /* get the current device */
                     pdev = gs_currentdevice(minst->i_ctx_p->pgs);
-                    if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) == 0) {
+                    if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) <= 0) {
                         outprintf(minst->heap,
                                   "   --saved-pages not supported by the '%s' device.\n",
                                   pdev->dname);
                         arg_finit(pal);
                         return gs_error_Fatal;
                     }
-                    ppdev = (gx_device_printer *)pdev;
-                    code = gx_saved_pages_param_process(ppdev, (byte *)arg+12, strlen(arg+12));
+                    /* get the real target (printer) device */
+                    do {
+                        child_dev_data.target = pdev;
+                        ret = dev_proc(pdev, dev_spec_op)(pdev, gxdso_device_child, &child_dev_data,
+                                                          sizeof(child_dev_data));
+                        if (ret > 0)
+                            pdev = child_dev_data.target;
+                    } while ((ret > 0) && (child_dev_data.n != 0));
+                    if ((code = gx_saved_pages_param_process((gx_device_printer *)pdev,
+                                                              (byte *)arg+12, strlen(arg+12))) < 0)
+                        return code;
                     if (code > 0)
                         if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
                             return code;
@@ -501,11 +531,21 @@ run_stdin:
                 code = argproc(minst, arg);
                 if (code < 0)
                     return code;
+                /* If in saved_pages_test_mode, print and flush previous job before the next file */
                 if (minst->saved_pages_test_mode) {
                     gx_device *pdev;
+                    int ret;
+                    gxdso_device_child_request child_dev_data;
 
-                    /* get the current device */
+                    /* get the real target (printer) device */
                     pdev = gs_currentdevice(minst->i_ctx_p->pgs);
+                    do {
+                        child_dev_data.target = pdev;
+                        ret = dev_proc(pdev, dev_spec_op)(pdev, gxdso_device_child, &child_dev_data,
+                                                          sizeof(child_dev_data));
+                        if (ret > 0)
+                            pdev = child_dev_data.target;
+                    } while ((ret > 0) && (child_dev_data.n != 0));
                     if ((code = gx_saved_pages_param_process((gx_device_printer *)pdev,
                                (byte *)"print normal flush", 18)) < 0)
                         return code;
@@ -527,11 +567,21 @@ run_stdin:
                 minst->run_buffer_size = bsize;
                 if (code < 0)
                     return code;
+                /* If in saved_pages_test_mode, print and flush previous job before the next file */
                 if (minst->saved_pages_test_mode) {
                     gx_device *pdev;
+                    int ret;
+                    gxdso_device_child_request child_dev_data;
 
-                    /* get the current device */
+                    /* get the real target (printer) device */
                     pdev = gs_currentdevice(minst->i_ctx_p->pgs);
+                    do {
+                        child_dev_data.target = pdev;
+                        ret = dev_proc(pdev, dev_spec_op)(pdev, gxdso_device_child, &child_dev_data,
+                                                          sizeof(child_dev_data));
+                        if (ret > 0)
+                            pdev = child_dev_data.target;
+                    } while ((ret > 0) && (child_dev_data.n != 0));
                     if ((code = gx_saved_pages_param_process((gx_device_printer *)pdev,
                                (byte *)"print normal flush", 18)) < 0)
                         return code;
@@ -625,11 +675,18 @@ run_stdin:
             break;
         case 'o':               /* set output file name and batch mode */
             {
+                i_ctx_t *i_ctx_p;
+                uint space;
                 const char *adef;
-                char *str;
+                byte *str;
                 ref value;
                 int len;
 
+                if ((code = gs_main_init1(minst)) < 0)
+                    return code;
+
+                i_ctx_p = minst->i_ctx_p;
+                space = icurrent_space;
                 if (arg[0] == 0) {
                     code = arg_next(pal, (const char **)&adef, minst->heap);
                     if (code < 0)
@@ -638,13 +695,14 @@ run_stdin:
                     adef = arg;
                 if ((code = gs_main_init1(minst)) < 0)
                     return code;
+                ialloc_set_space(idmemory, avm_system);
                 len = strlen(adef);
-                str = (char *)gs_alloc_bytes(minst->heap, (uint)len, "-o");
+                str = ialloc_string(len, "-o");
                 if (str == NULL)
                     return gs_error_VMerror;
                 memcpy(str, adef, len);
-                make_const_string(&value, a_readonly | avm_foreign,
-                                  len, (const byte *)str);
+                make_const_string(&value, a_readonly | avm_system, len, str);
+                ialloc_set_space(idmemory, space);
                 initial_enter_name("OutputFile", &value);
                 initial_enter_name("NOPAUSE", &vtrue);
                 initial_enter_name("BATCH", &vtrue);
@@ -793,18 +851,14 @@ run_stdin:
                         }
                     } else {
                         int len = strlen(eqp);
-                        char *str =
-                        (char *)gs_alloc_bytes(minst->heap,
-                                               (uint) len, "-s");
+                        byte *body = ialloc_string(len, "-s");
 
-                        if (str == 0) {
+                        if (body == NULL) {
                             lprintf("Out of memory!\n");
                             return gs_error_Fatal;
                         }
-                        memcpy(str, eqp, len);
-                        make_const_string(&value,
-                                          a_readonly | avm_foreign,
-                                          len, (const byte *)str);
+                        memcpy(body, eqp, len);
+                        make_const_string(&value, a_readonly | avm_system, len, body);
                         if ((code = try_stdout_redirect(minst, adef, eqp)) < 0)
                             return code;
                     }
