@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -296,12 +296,18 @@ gs_interp_init(i_ctx_t **pi_ctx_p, const ref *psystem_dict,
     /* Create and initialize a context state. */
     gs_context_state_t *pcst = 0;
     int code = context_state_alloc(&pcst, psystem_dict, dmem);
-
-    if (code >= 0)
+    if (code >= 0) {
         code = context_state_load(pcst);
+        if (code < 0) {
+            context_state_free(pcst);
+            pcst = NULL;
+        }
+    }
+
     if (code < 0)
         lprintf1("Fatal error %d in gs_interp_init!\n", code);
     *pi_ctx_p = pcst;
+
     return code;
 }
 /*
@@ -1766,7 +1772,7 @@ res:
         /* but it might be something else someday if we check */
         /* for interrupts in the interpreter loop itself. */
         if (iesp >= estop)
-            code = gs_error_execstackoverflow;
+            ierror.code = gs_error_execstackoverflow;
         else {
             iesp++;
             ref_assign_inline(iesp, IREF);
