@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -1033,11 +1033,16 @@ sfnt_get_glyph_offset(ref *pdr, gs_font_type42 *pfont42, int index,
     sfnts_reader r;
     int glyf_elem_size = (pfont42->data.indexToLocFormat) ? 4 : 2;
 
-    sfnts_reader_init(&r, pdr);
-    r.seek(&r, pfont42->data.loca + index * glyf_elem_size);
-    *offset0 =
-        pfont42->data.glyf + (glyf_elem_size ==
+    if (index < pfont42->data.trueNumGlyphs) {
+        sfnts_reader_init(&r, pdr);
+        r.seek(&r, pfont42->data.loca + index * glyf_elem_size);
+        *offset0 =
+            pfont42->data.glyf + (glyf_elem_size ==
                               2 ? r.rword(&r) * 2 : r.rlong(&r));
+    }
+    else {
+        r.error = true;
+    }
     return (r.error);
 }
 
@@ -1133,7 +1138,7 @@ FAPI_FF_get_glyph(gs_fapi_font *ff, int char_code, byte *buf,
      */
     ref *pdr = pfont_dict(((gs_font_base *) ff->client_font_data2));
 
-    ushort glyph_length;
+    int glyph_length;
     i_ctx_t *i_ctx_p = (i_ctx_t *) ff->client_ctx_p;
 
     if (ff->is_type1) {
@@ -1157,7 +1162,7 @@ FAPI_FF_get_glyph(gs_fapi_font *ff, int char_code, byte *buf,
                  */
                 if (name_ref(ff->memory, ff->char_data,
                              ff->char_data_len, &char_name, -1) < 0)
-                    return -1;
+                    return gs_fapi_glyph_invalid_format;
                 if (buf != NULL) {
                     /*
                      * Trigger the next call to the 'seac' case below.
@@ -1177,23 +1182,25 @@ FAPI_FF_get_glyph(gs_fapi_font *ff, int char_code, byte *buf,
                     if (name_ref
                         (ff->memory, (const byte *)".notdef", 7, &char_name,
                          -1) < 0)
-                        return -1;
+                        return gs_fapi_glyph_invalid_format;
             }
             if (dict_find_string(pdr, "CharStrings", &CharStrings) <= 0)
-                return -1;
+                return gs_fapi_glyph_invalid_format;
 
             if (dict_find(CharStrings, &char_name, &glyph) <= 0) {
                 if (name_ref
                     (ff->memory, (const byte *)".notdef", 7, &char_name,
                      -1) < 0) {
-                    return -1;
+                    return gs_fapi_glyph_invalid_format;
                 }
                 if (dict_find(CharStrings, &char_name, &glyph) <= 0) {
-                    return -1;
+                    return gs_fapi_glyph_invalid_format;
                 }
             }
             if (r_has_type(glyph, t_array) || r_has_type(glyph, t_mixedarray))
-                return -1;
+                return gs_fapi_glyph_invalid_format;
+            if (!r_has_type(glyph, t_string))
+                return 0;
             glyph_length = get_type1_data(ff, glyph, buf, buf_length);
         }
     }
@@ -1237,10 +1244,13 @@ FAPI_FF_get_glyph(gs_fapi_font *ff, int char_code, byte *buf,
                 bool error = sfnt_get_glyph_offset(pdr, pfont42, char_code, &offset0);
 
                 if (error != 0) {
-                    glyph_length = -1;
+                    glyph_length = gs_fapi_glyph_invalid_index;
                 }
                 else if (pfont42->data.len_glyphs) {
-                    glyph_length = pfont42->data.len_glyphs[char_code];
+                    if (char_code <= pfont42->data.numGlyphs)
+                        glyph_length = pfont42->data.len_glyphs[char_code];
+                    else
+                        glyph_length = gs_fapi_glyph_invalid_index;
                 }
                 else {
                     ulong noffs;
@@ -1257,7 +1267,7 @@ FAPI_FF_get_glyph(gs_fapi_font *ff, int char_code, byte *buf,
                          */
                         int code = sfnt_get_sfnt_length(pdr, &noffs);
                         if (code < 0) {
-                            glyph_length = -1;
+                            glyph_length = gs_fapi_glyph_invalid_index;
                         }
                         else {
                             glyph_length = noffs - offset0;
@@ -1276,7 +1286,7 @@ FAPI_FF_get_glyph(gs_fapi_font *ff, int char_code, byte *buf,
                                   min(glyph_length,
                                       buf_length) /* safety */ );
                     if (r.error == 1) {
-                        glyph_length = -1;
+                        glyph_length = gs_fapi_glyph_invalid_index;
                     }
                     /* r.error == 2 means a rangecheck, and probably means that the
                      * font is broken, and the final glyph length is longer than the data available for it.
@@ -1322,7 +1332,8 @@ ps_fapi_get_metrics(gs_fapi_font *ff, gs_string *char_name, int cid,
 
 
 /* forward declaration for the ps_ff_stub assignment */
-static int ps_get_glyphname_or_cid(gs_font_base *pbfont,
+static int ps_get_glyphname_or_cid(gs_text_enum_t *penum,
+                                   gs_font_base *pbfont,
                                    gs_string *charstring, gs_string *name,
                                    int ccode, gs_string *enc_char_name,
                                    char *font_file_path,
@@ -1809,7 +1820,8 @@ find_substring(const byte *where, int length, const char *what)
 }
 
 static int
-ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
+ps_get_glyphname_or_cid(gs_text_enum_t *penum,
+                        gs_font_base *pbfont, gs_string *charstring,
                         gs_string *name, int ccode,
                         gs_string *enc_char_name, char *font_file_path,
                         gs_fapi_char_ref *cr, bool bCID)
@@ -1825,9 +1837,35 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
         ((pbfont->FontType == ft_encrypted
           || pbfont->FontType == ft_encrypted2) && font_file_path == NULL);
     i_ctx_t *i_ctx_p = (i_ctx_t *) I->client_ctx_p;
+    bool unicode_cp = false;
 
     /* Obtain the character name : */
     if (bCID) {
+        if (pbfont->FontType == ft_CID_TrueType && font_file_path) {
+            ref *pdr2, *fidr, *dummy;
+            pdr2 = pfont_dict(gs_rootfont(igs));
+            if (dict_find_string(pdr2, "FontInfo", &fidr) &&
+                dict_find_string(fidr, "GlyphNames2Unicode", &dummy))
+            {
+                unsigned char uc[4] = {0};
+                unsigned int cc = 0;
+                int i, l;
+                byte *c = (byte *)&penum->text.data.bytes[penum->index - penum->bytes_decoded];
+
+                for (i = 0; i < penum->bytes_decoded ; i++) {
+                  cc |= c[i] << ((penum->bytes_decoded - 1) - i) * 8;
+                }
+                l = ((gs_font_base *)gs_rootfont(igs))->procs.decode_glyph(gs_rootfont(igs), cc + GS_MIN_CID_GLYPH, ccode, (unsigned short *)uc, sizeof(uc));
+                if (l > 0 && l < sizeof(uc)) {
+                    cc = 0;
+                    for (i = 0; i < l; i++) {
+                        cc |= uc[i] << ((penum->bytes_decoded - 1) - i) * 8;
+                    }
+                    ccode = cc;
+                    unicode_cp = true;
+                }
+            }
+        }
         client_char_code = ccode;
         make_null(&char_name);
         enc_char_name->data = NULL;
@@ -1877,7 +1915,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
     cr->char_codes_count = 1;
     if (bCID) {
         if (font_file_path != NULL) {
-            ref *Decoding, *TT_cmap, *SubstNWP;
+            ref *Decoding, *TT_cmap = NULL, *SubstNWP;
             ref src_type, dst_type;
             uint c = 0;
 
@@ -1930,9 +1968,10 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
             else {
                 ref *CIDSystemInfo;
                 ref *Ordering;
-                ref *fdict, *CMapDict, *CMapName, CMapNameStr;
+                ref *fdict, *CMapDict, *CMapName, *WMode, CMapNameStr;
                 char *cmapnm = NULL;
                 int cmapnmlen = 0;
+                int wmode = 0;
                 /* leave off the -H or -V */
                 const char * const utfcmap = "Identity-UTF16";
                 int utfcmaplen = strlen(utfcmap);
@@ -1940,6 +1979,10 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                 fdict = pfont_dict(gs_rootfont(igs));
                 code = dict_find_string(fdict, "CMap", &CMapDict);
                 if (code >= 0 && r_has_type(CMapDict, t_dictionary)) {
+                    code = dict_find_string(CMapDict, "WMode", &WMode);
+                    if (code >= 0 && r_has_type(WMode, t_integer)) {
+                        wmode = WMode->value.intval;
+                    }
                     code = dict_find_string(CMapDict, "CMapName", &CMapName);
                     if (code >= 0 && r_has_type(CMapName, t_name)) {
                         name_string_ref(imemory, CMapName, &CMapNameStr);
@@ -1949,7 +1992,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                 }
                 /* We only have to lookup the char code if we're *not* using an identity ordering 
                    with the exception of Identity-UTF16 which is a different beast altogether */
-                if ((cmapnmlen > 0 && !strncmp(cmapnm, utfcmap, cmapnmlen > utfcmaplen ? utfcmaplen : cmapnmlen))
+                if (unicode_cp || (cmapnmlen > 0 && !strncmp(cmapnm, utfcmap, cmapnmlen > utfcmaplen ? utfcmaplen : cmapnmlen))
                     || (dict_find_string(pdr, "CIDSystemInfo", &CIDSystemInfo) >= 0
                     && r_has_type(CIDSystemInfo, t_dictionary)
                     && dict_find_string(CIDSystemInfo, "Ordering",
@@ -1974,9 +2017,17 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                         c = client_char_code;
                     }
                 }
+                if (pbfont->FontType == ft_CID_TrueType)
+                    c = ((gs_font_cid2 *)pbfont)->data.substitute_glyph_index_vertical((gs_font_type42 *)pbfont, c, wmode, ccode);
+            }
+            if (pbfont->FontType == ft_CID_TrueType && c == 0 && TT_cmap) {
+                ref cc32;
+                ref *gid;
+                make_int(&cc32, 32);
+                if (dict_find(TT_cmap, &cc32, &gid) >= 0)
+                    c = gid->value.intval;
             }
             cr->char_codes[0] = c;
-            cr->is_glyph_index = is_glyph_index;
             /* fixme : process the narrow/wide/proportional mapping type,
                using src_type, dst_type. Should adjust the 'matrix' above.
                Call get_font_proportional_feature for proper choice.
@@ -2100,7 +2151,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
             return (gs_error_invalidfont);
 #endif
         }
-        cr->is_glyph_index = true;
+        is_glyph_index = true;
     }
     else if (is_embedded_type1) {
         /*  Since the client passes charstring by callback using I->ff.char_data,
@@ -2256,6 +2307,9 @@ FAPI_char(i_ctx_t *i_ctx_p, bool bBuildGlyph, ref *charstring)
         int cindex = -1;
         ref gname;
 
+        if (I == NULL)
+            return_error(gs_error_invalidfont);
+
         /* initialise the FAPI font, this includes language specific stuff */
         I->ff = ps_ff_stub;
 
