@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -67,8 +67,8 @@ zfont_init(i_ctx_t *i_ctx_p)
         return gs_error_VMerror;
     ifont_dir->ccache.mark_glyph = zfont_mark_glyph_name;
     ifont_dir->global_glyph_code = zfont_global_glyph_code;
-    return gs_register_struct_root(imemory, NULL, (void **)&ifont_dir,
-                                   "ifont_dir");
+    return gs_register_struct_root(imemory, imemory->gs_lib_ctx->font_dir_root,
+        (void **)&ifont_dir, "ifont_dir");
 }
 
 /* <font> <scale> scalefont <new_font> */
