@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -2362,12 +2362,18 @@ parse_font(i_ctx_t *i_ctx_p,  ref *topdict,
 
     memset(&offsets, 0, sizeof(offsets)); /* set defaults for charset_off, encoding_off */
 
-    if ((code = parse_dict(i_ctx_p, topdict, &offsets, strings, data, p_top, pe_top) < 0))
+    if ((code = parse_dict(i_ctx_p, topdict, &offsets, strings, data, p_top, pe_top)) < 0)
         return code;
-    if ((code = parse_dict(i_ctx_p, topdict, &offsets, strings, data,
+
+    /* if the values for the /Private dict are nonsensical, ignore it
+     * and hope the font doesn't actually need it!
+     */
+    if ((offsets.private_off + offsets.private_size) <= pe_all) {
+        if ((code = parse_dict(i_ctx_p, topdict, &offsets, strings, data,
                       p_all + offsets.private_off,
-                      p_all + offsets.private_off + offsets.private_size) < 0))
-        return code;
+                      p_all + offsets.private_off + offsets.private_size)) < 0)
+            return code;
+    }
     if ((code = parse_index(&local_subrs, data,
                (offsets.local_subrs_off ? p_all + offsets.private_off + offsets.local_subrs_off : 0),
                 pe_all)) < 0)
@@ -2463,10 +2469,10 @@ parse_font(i_ctx_t *i_ctx_p,  ref *topdict,
                 return code;
             if ((code = dict_create(5, fdfont)) < 0)
                 return code;
-            if ((code = parse_dict(i_ctx_p, fdfont, &offsets, strings, data, doff, doff + len) < 0))
+            if ((code = parse_dict(i_ctx_p, fdfont, &offsets, strings, data, doff, doff + len)) < 0)
                 return code;
             if ((code = parse_dict(i_ctx_p, fdfont, &offsets, strings, data,
-              p_all + offsets.private_off, p_all + offsets.private_off + offsets.private_size) < 0))
+              p_all + offsets.private_off, p_all + offsets.private_off + offsets.private_size)) < 0)
                 return code;
             if ((code = find_font_dict(i_ctx_p, fdfont, &fdprivate, "Private")) < 0)
                 return code;
