@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -116,8 +116,10 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
     }
     picc_profile->num_comps = ncomps;
 
-    picc_profile->data_cs = gscms_get_profile_data_space(picc_profile->profile_handle);
-    switch( picc_profile->data_cs ) {
+    picc_profile->data_cs =
+        gscms_get_profile_data_space(picc_profile->profile_handle,
+            picc_profile->memory);
+    switch (picc_profile->data_cs) {
         case gsCIEXYZ:
         case gsCIELAB:
         case gsRGB:
@@ -193,11 +195,11 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
             default:
                 break;
         }
-        /* Have one increment from the color space.  Having these tied 
+        /* Have one increment from the color space.  Having these tied
            together is not really correct.  Need to fix that.  ToDo.  MJV */
         rc_adjust(picc_profile, -2, "seticc");
         rc_increment(pcs->cmm_icc_profile_data);
-    } 
+    }
     /* Set the color space.  We are done.  No joint cache here... */
     code = gs_setcolorspace(igs, pcs);
     /* The context has taken a reference to the colorspace. We no longer need
@@ -215,7 +217,7 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
 /*
  *   <dict>  .set_outputintent  -
  *
- * Set and use the specified output intent.  
+ * Set and use the specified output intent.
  *
  */
 static int
@@ -241,7 +243,10 @@ zset_outputintent(i_ctx_t * i_ctx_p)
 
     /* Get the device structure */
     code = dev_proc(dev, get_profile)(dev,  &dev_profile);
-    if (dev_profile == NULL) { 
+    if (code < 0)
+        return code;
+
+    if (dev_profile == NULL) {
         code = gsicc_init_device_profile_struct(dev, NULL, 0);
         if (code < 0)
             return code;
@@ -274,7 +279,9 @@ zset_outputintent(i_ctx_t * i_ctx_p)
         rc_decrement(picc_profile,"zset_outputintent");
         return -1;
     }
-    picc_profile->data_cs = gscms_get_profile_data_space(picc_profile->profile_handle);
+    picc_profile->data_cs =
+        gscms_get_profile_data_space(picc_profile->profile_handle,
+            picc_profile->memory);
     switch (picc_profile->data_cs) {
         case gsCIEXYZ:
         case gsCIELAB:
@@ -293,8 +300,8 @@ zset_outputintent(i_ctx_t * i_ctx_p)
         case gsNCHANNEL:
             expected = 0;
             break;
-        case gsNAMED:            
-        case gsUNDEFINED:        
+        case gsNAMED:
+        case gsUNDEFINED:
             break;
     }
     if (expected && ncomps != expected) {
@@ -305,21 +312,21 @@ zset_outputintent(i_ctx_t * i_ctx_p)
 
     /* All is well with the profile.  Lets set the stuff that needs to be set */
     dev_profile->oi_profile = picc_profile;
-    picc_profile->name = (char *) gs_alloc_bytes(picc_profile->memory, 
+    picc_profile->name = (char *) gs_alloc_bytes(picc_profile->memory,
                                                  MAX_DEFAULT_ICC_LENGTH,
                                                  "zset_outputintent");
     strncpy(picc_profile->name, OI_PROFILE, strlen(OI_PROFILE));
     picc_profile->name[strlen(OI_PROFILE)] = 0;
     picc_profile->name_length = strlen(OI_PROFILE);
     /* Set the range of the profile */
-    gscms_set_icc_range(&picc_profile);
+    gsicc_set_icc_range(&picc_profile);
 
-    /* If the output device has a different number of componenets, then we are 
-       going to set the output intent as the proofing profile, unless the 
-       proofing profile has already been set. 
+    /* If the output device has a different number of componenets, then we are
+       going to set the output intent as the proofing profile, unless the
+       proofing profile has already been set.
 
-       If the device has the same number of components (and color model) then as 
-       the profile we will use this as the output profile, unless someone has 
+       If the device has the same number of components (and color model) then as
+       the profile we will use this as the output profile, unless someone has
        explicitly set the output profile.
 
        Finally, we will use the output intent profile for the default profile
@@ -329,22 +336,22 @@ zset_outputintent(i_ctx_t * i_ctx_p)
     dev_comps = dev_profile->device_profile[0]->num_comps;
     index = gsicc_get_default_type(dev_profile->device_profile[0]);
     if (ncomps == dev_comps && index < gs_color_space_index_DevicePixel) {
-        /* The OI profile is the same type as the profile for the device and a 
-           "default" profile for the device was not externally set. So we go 
-           ahead and use the OI profile as the device profile.  Care needs to be 
-           taken here to keep from screwing up any device parameters.   We will 
-           use a keyword of OIProfile for the user/device parameter to indicate 
-           its usage.  Also, note conflicts if one is setting object dependent 
+        /* The OI profile is the same type as the profile for the device and a
+           "default" profile for the device was not externally set. So we go
+           ahead and use the OI profile as the device profile.  Care needs to be
+           taken here to keep from screwing up any device parameters.   We will
+           use a keyword of OIProfile for the user/device parameter to indicate
+           its usage.  Also, note conflicts if one is setting object dependent
            color management */
-        rc_assign(dev_profile->device_profile[0], picc_profile, 
+        rc_assign(dev_profile->device_profile[0], picc_profile,
                   "zset_outputintent");
         if_debug0m(gs_debug_flag_icc, imemory, "[icc] OutputIntent used for device profile\n");
     } else {
         if (dev_profile->proof_profile == NULL) {
-            /* This means that we should use the OI profile as the proofing 
-               profile.  Note that if someone already has specified a 
+            /* This means that we should use the OI profile as the proofing
+               profile.  Note that if someone already has specified a
                proofing profile it is unclear what they are trying to do
-               with the output intent.  In this case, we will use it 
+               with the output intent.  In this case, we will use it
                just for the source data below */
             dev_profile->proof_profile = picc_profile;
             rc_increment(picc_profile);
@@ -359,17 +366,17 @@ zset_outputintent(i_ctx_t * i_ctx_p)
         switch (picc_profile->data_cs) {
             case gsGRAY:
                 if_debug0m(gs_debug_flag_icc, imemory, "[icc] OutputIntent used source Gray\n");
-                rc_assign(icc_manager->default_gray, picc_profile, 
+                rc_assign(icc_manager->default_gray, picc_profile,
                           "zset_outputintent");
                 break;
             case gsRGB:
                 if_debug0m(gs_debug_flag_icc, imemory, "[icc] OutputIntent used source RGB\n");
-                rc_assign(icc_manager->default_rgb, picc_profile, 
+                rc_assign(icc_manager->default_rgb, picc_profile,
                           "zset_outputintent");
                 break;
             case gsCMYK:
                 if_debug0m(gs_debug_flag_icc, imemory, "[icc] OutputIntent used source CMYK\n");
-                rc_assign(icc_manager->default_cmyk, picc_profile, 
+                rc_assign(icc_manager->default_cmyk, picc_profile,
                           "zset_outputintent");
                 break;
             default:
@@ -381,94 +388,6 @@ zset_outputintent(i_ctx_t * i_ctx_p)
     return code;
 }
 
-/*
- *   <dict>  .seticcspace  -
- *
- * Create an ICCBased color space and set it to be the current color space.
- *
- * The PostScript structure of an ICCBased color space is that same as that
- * for a CIEBased* color space:
- *
- *     [ /ICCBased <dictionary> ]
- *
- * As is the for other .setcie*space operators, the operand dictionary rather
- * than the complete color space array is on the stack when this operator
- * is inovked.
- *
- * At the time this procedure is called, the alternative color space for
- * the ICCBased color space is expected to be the current color space,
- * whether that space was explicitly specified or implied by the number
- * of components in the ICCBased color space dictionary. This is consistent
- * with the handling of alternative spaces in Separation, DeviceN, and
- * Indexed color spaces. Unlike the "zset*space" routines for those spaces,
- * however, the current code does not attempt to build the color space
- * "in place" in the graphic state.
- *
- * The procedure that invokes this operator will already have checked that
- * the operand is a dictionary, is readable, and defines the key /N
- * (number of components).
- */
-static int
-zseticcspace(i_ctx_t * i_ctx_p)
-{
-    os_ptr                  op = osp;
-    int                     code;
-    gs_color_space *  palt_cs;
-    ref *                   pnval;
-    ref *                   pstrmval;
-    stream *                s;
-    int                     i, ncomps;
-    float                   range_buff[8];
-    static const float      dflt_range[8] = { 0, 1,   0, 1,   0, 1,   0, 1 };
-
-    code = dict_find_string(op, "N", &pnval);
-    if (code < 0)
-        return code;
-    ncomps = pnval->value.intval;
-    if (2*ncomps > sizeof(range_buff)/sizeof(range_buff[0]))
-        return_error(gs_error_rangecheck);
-    /* verify the DataSource entry */
-    if (dict_find_string(op, "DataSource", &pstrmval) <= 0)
-        return_error(gs_error_undefined);
-    check_read_file(i_ctx_p, s, pstrmval);
-    /*
-     * Verify that the current color space can be a alternative color space.
-     * The check for ICCBased color space is a hack to avoid introducing yet
-     * another category indicator into the gs_color_space_type structur.
-     */
-    palt_cs = gs_currentcolorspace(igs);
-    if ( !palt_cs->type->can_be_alt_space                                ||
-         gs_color_space_get_index(palt_cs) == gs_color_space_index_ICC  )
-        return_error(gs_error_rangecheck);
-    /*
-     * Fetch and verify the Range array.
-     *
-     * The PDF documentation is unclear as to the purpose of this array.
-     * Essentially all that is stated is that "These values must match the
-     * information in the ICC profile" (PDF Reference, 2nd ed., p. 174).
-     * If that is the case, why not use the information in the profile?
-     * The only reason we can think of is range specification is intended
-     * to be used to limit the range of values passed to the alternate
-     * color space (the range may be smaller than the native range of values
-     * provided by that color space).
-     *
-     * Because the cms code will perform normalization based on color
-     * space, we use the range values only to restrict the set of input
-     * values; they are not used for normalization.
-     */
-    code = dict_floats_param( imemory,
-                              op,
-                              "Range",
-                              2 * ncomps,
-                              range_buff,
-                              dflt_range );
-    for (i = 0; i < 2 * ncomps && range_buff[i + 1] >= range_buff[i]; i += 2)
-        ;
-    if (i != 2 * ncomps)
-        return_error(gs_error_rangecheck);
-    return seticc(i_ctx_p, ncomps, op, range_buff);
-}
-
 /* Install a ICC type color space and use the ICC LABLUT profile. */
 int
 seticc_lab(i_ctx_t * i_ctx_p, float *white, float *black, float *range_buff)
@@ -591,7 +510,9 @@ znumicc_components(i_ctx_t * i_ctx_p)
         make_int(op, expected);
         return 0;
     }
-    picc_profile->data_cs = gscms_get_profile_data_space(picc_profile->profile_handle);
+    picc_profile->data_cs =
+        gscms_get_profile_data_space(picc_profile->profile_handle,
+            picc_profile->memory);
 
     switch (picc_profile->data_cs) {
         case gsCIEXYZ:
@@ -622,7 +543,6 @@ znumicc_components(i_ctx_t * i_ctx_p)
 
 const op_def    zicc_ll3_op_defs[] = {
     op_def_begin_ll3(),
-    { "1.seticcspace", zseticcspace },    
     { "1.set_outputintent", zset_outputintent },
     { "1.numicc_components", znumicc_components },
     op_def_end(0)
